Unit CrtES;                       { Crt enhanced }
{$N+,E+}                      { PetriSim version }

{ This unit adds some routines to work with CRT. }

Interface

Uses Crt, Dos, Graph, GGraph;

Const KNull   = #0;   { Null }
      KEsc    = #27;  { Esc }
      KAltX   = #173; { Alt-X }
      KUp     = #200; { arrow up }
      KDown   = #208; { arrow down }
      KLeft   = #203; { arrow left }
      KRight  = #205; { arrow right }
      KF1     = #187; { F1 }
      KF2     = #188; { F2 }
      KF3     = #189; { F3 }
      KF4     = #190; { F4 }
      KF5     = #191; { F5 }
      KF6     = #192; { F6 }
      KF7     = #193; { F7 }
      KF8     = #194; { F8 }
      KF9     = #195; { F9 }
      KF10    = #196; { F10 }
      KIns    = #210; { Insert }
      KCR     = #13;  { CR (Enter) }
      KLF     = #10;  { LF }
      KEnd    = #207; { End }
      KHome   = #199; { Home }
      KPgUp   = #201; { PgUp }
      KPgDn   = #209; { PgDn }
      KBS     = #8;   { Backspace }

Type  TLine   = string[80];
      TPage   = array[1..23] of TLine;

Var   CuMeCo   :  byte;         { Current normal menu color }
      CuEmCo   :  byte;     { Current emphasized menu color }
      CuPrCo   :  byte;              { Current prompt color }
      CuBkCo   :  byte;          { Current background color }

  Function  ReadK:char;                       { Returns key }
                      { For special keys incremented by 128 }

  Procedure CursorOn;               { Switch on text cursor }

  Procedure CursorOff;             { Switch off text cursor }   

  Procedure Wx(x,y:byte; s:string);
                   { GoToXY + Writing in current menu color }

  Procedure Wex(x,y:byte; s:string);
                { GoToXY + Writing in emphasized menu color }

  Function ErrorMessage(e:integer):string;
                                  { Some I/O error messages }

  Procedure DOSHalt (code: word);
                { Setting correct DOS settings + Halt(code) }

  Procedure GraphInfo;         { Information about graphics }


  Procedure Upper(var s:string);  { Conversion to uppercase }

  Function FileIsEmpty (Name: string): boolean;
  { Returns true if the file does not exist or if it is empty. }

Implementation

   Var  Reg : Registers;                 { registers of CPU }

{***********************************************************}

   Function ReadK:char;            
                      { For special keys incremented by 128 }
   var c:char;
   begin
      c := ReadKey; 
      if c=#0 then begin
         c := ReadKey; c := Chr(Ord(c) + 128);
      end;
      ReadK := c;
   end;

{***********************************************************}

  Procedure CursorOn;               { Switch on text cursor }
  Var   VideoMode : Byte;
  begin                                      { monitor type }
     VideoMode := Mem[Seg0040: $0049];
     If VideoMode = 7 then
       Reg.CX := $0C0D              { monochromatic monitor }
     else
       Reg.CX := $0607;                     { color monitor }
     Reg.AX :=$0100;
     Intr($10,Reg);
  end;           

{***********************************************************}

  Procedure CursorOff;             { Switch off text cursor }
  begin
     Reg.AX := $0100;
     Reg.CX := $2020;
     Intr($10,Reg);
  end;

{***********************************************************}

  Procedure Wx(x,y:byte; s:string);
                   { GoToXY + Writing in current menu color }
  var x0,y0 : integer;
  begin
     x0 := 8*(x-1);
     y0 := 8 + 16*(y-1);
     SetTextStyle(MenuFont, HorizDir, MenuSize);
     SetTextJustify(LeftText, CenterText);
     SetColor(CuMeCo);
     MoveTo(x0,y0);
     While GetY<(TextHeight(S) div 2) do MoveRel(0,1);
     OutText(S);
  end;

{***********************************************************}

  Procedure Wex(x,y:byte; s:string);
                  { GoToXY + Writing in Emphasized menu color }
  var x0,y0 : integer;
  begin
     x0 := 8*(x-1);
     y0 := 8 + 16*(y-1);
     SetTextStyle(MenuFont, HorizDir, MenuSize);
     SetTextJustify(LeftText, CenterText);
     SetColor(CuEmCo);
     MoveTo(x0,y0);
     While GetY<(TextHeight(S) div 2) do MoveRel(0,1);
     OutText(S);
  end;

{***********************************************************}

Function ErrorMessage(e:integer):string;
{ Some DOS error messages }
Begin
   If      e=2   then ErrorMessage := 'File not found'
   else if e=3   then ErrorMessage := 'Path not found'            
   else if e=4   then ErrorMessage := 'Too many open files'
   else if e=5   then ErrorMessage := 'File access denied'
   else if e=6   then ErrorMessage := 'Invalid file handle'
   else if e=8   then ErrorMessage := 'Not enough memory'
   else if e=10  then ErrorMessage := 'Invalid environment'
   else if e=11  then ErrorMessage := 'Invalid format'
   else if e=18  then ErrorMessage := 'No more files'
   else if e=100 then ErrorMessage := 'Disk read error'
   else if e=101 then ErrorMessage := 'Disk write error'
   else if e=150 then ErrorMessage := 'Disk write-protected'
   else if e=152 then ErrorMessage := 'Drive not ready'
   else          ErrorMessage := 'Find it in the handbook';
End;

{***********************************************************}

  Procedure DOSHalt (code: word);
                { Setting correct DOS settings + Halt(code) }
  Begin
     RestoreCrtMode;                  { normal DOS settings }
     CursorOn;
     TextColor(LightGray);
     TextBackground(Black);
     ClrScr;
     Halt(code);
  End;

{***********************************************************}

Procedure GraphInfo;           { Information about graphics }
                          { To be called in graphics mode ! }
Var Min,Max,d,l : integer;
    s           : string;
Begin
   ClearDevice;
   GetModeRange(GrDriver,Min,Max);
   Wex( 3,5,'Current graph driver: ' + GetDriverName
           + '  In mode: ' + GetModeName(GetGraphMode));
   Str(Min:3,s);
   Wx( 5,7,'Lowest mode  = ' + s);
   Str(GetMaxX:3,s);
   Wx(40,7,'MaxX = ' + s);
   Str(Max:3,s);
   Wx( 5,8,'Highest mode = ' + s);
   Str(GetMaxY:3,s);
   Wx(40,8,'MaxY = ' + s);

   Wx(3,11,'The modes supported by the ' + GetDriverName
           + ' are:');
   l := 13;
   For d:=0 to GetMaxMode do begin
      Str(d:3,s);
      Wx(5,l,s + ' : ' + GetModeName(d));
      Inc(l);
   end;
end;

{***********************************************************}

  Procedure Upper(var s:string);  { Conversion to uppercase }
  Var i: integer;
  Begin
     For i:=1 to Length(s) do s[i] := UpCase(s[i])
  End;

{***********************************************************}

  Function FileIsEmpty (Name: string): boolean;
  { Returns true if the file does not exist or if it is empty. }

  Var FileInfo: SearchRec;

  Begin
     FindFirst(Name, AnyFile, FileInfo);
     FileIsEmpty := (DosError>0) or (FileInfo.Size=0);
  End;


{************ Initialisation code **************************}

Begin                                   { Default settings: }
      CuMeCo := Green;          { Current normal menu color }
      CuEmCo := Yellow;     { Current emphasized menu color }
      CuPrCo := Cyan;                { Current prompt color }
      CuBkCo := Black;           { Current background color }
      MenuFont := DefaultFont;         { Menu font and size }
      MenuSize := 1;
      CursorOff;
end.