UNIT EMouseGS; { Enhanced mouse operations - menu only graph}
           { Arrows for menu selection ! + General graphics }
{$N+,E+}        { Simpet version  J. Sklenar     April 1993 }

Interface

 Uses Crt, Dos, Graph, GGraph, CrtES;

{ GGraph   = General graph (various cards to VGA 640x480) }
{ CrtES    = Crt Enhanced }

 Const

       WaitTime = 250; { Waiting after pressing button - ms }
       CrossCol = Cyan;              { Keyboard cross color }
       CrossSize = 6;                 { Keyboard cross size }
       IncX = 3;                  { Increment of X (arrows) }
       IncY = 3;                  { Increment of Y (arrows) }

Type   PositionG = record      { Graph mode position }
         x1,y1:      word;     { upper left }
         x2,y2:      word;     { lower right }
         opt:        char;     { option }
       end;

     Set_of_char = set of char;

Var  MouseOK : boolean;         { Mouse driver present ? }
     MCurOn  : boolean;              { Mouse cursor on ? }
     MKX     : word;        { Mouse keyboard coordinates }
     MKY     : word;
     PullDown: boolean;  { PullDown/Popup menu (fefault) }
     SPrompt : String;                  { Prompt message }

 Function  MouseInit: boolean;
                       { Mouse initialization. True = OK }
 Procedure MouseCursorOn;
                            { Switch the mouse cursor on }
 Procedure MouseCursorOff;
                           { Switch the mouse cursor off }
 Function  MousePosition ( var MX, MY : word) : word;
                         { Returns the mouse coordinates }
                         { and the status of buttons :   }
                         { Left     1 -,----,---,
                                       5    |   |
          Not all mice ! ->Middle   4 -'-,--7   3
                                         6  |   |
                           Right    2 ---'--'---'  }

 Procedure SetMousePos (MX, MY : word);
                        { Places mouse cursor to (MX,MY) }

 Procedure SetMouseXRange(Xmin,Xmax : integer);

 Procedure SetMouseYRange(Ymin,Ymax : integer);

 Procedure Wait;
  { Waiting for pressing any key or any mouse button }

 Procedure WaitUp;
  { Waiting for releasing a mouse button }

 Function CWait(var b:word):char;
  { Waiting for pressing any key or any mouse button. }
  { Returns the key or 0 if mouse button was pressed. }
  { "b" is the button pressed or 0 if key pressed. }

  Function CWaitNW(var b:word):char;
  { Waiting for pressing any key or any mouse button. }
  { Returns the key or 0 if mouse button was pressed. }
  { "b" is the button pressed or 0 if key pressed. }
  { NO WAITING !!! }

  Function KeyOrMouseG(s:set_of_char;
             GMA:array of PositionG;OldOp,MOut:char): char;
  { Returns an upper case option from either the set s or
    the graph menu array GMA }

  Procedure DPrompt;        { Displaying prompt in Sprompt }

  Procedure EPrompt;                      { Erasing prompt }

{**********************************************************}

Implementation  

Var Reg  : Registers;        

    Procedure Cross(X,Y:word);
    begin
       Line(X-CrossSize,Y,X+CrossSize,Y);
       Line(X,Y-CrossSize,X,Y+CrossSize);
    end;

{**********************************************************}

Function MouseInit : boolean;
begin
  with Reg do begin
    AX := 0;
    Intr( $33,Reg);       { Mouse driver interrupt }
    MouseInit := not (AX = 0);
    MouseOK   := not (AX = 0);
  end;
  MKX := VGAX div 2;           { Mouse by keyboard }
  MKY := VGAY div 2;
  MCurOn := false;
end;              

{**********************************************************}

Procedure MouseCursorOn;
begin
    Reg.AX := 1;
    Intr($33,Reg);
    MCurOn := true;
    If not MouseOK then begin
       SetColor(CrossCol);         { Display cross }
       Cross(MKX,MKY);
    end;
end;

{**********************************************************}

Procedure MouseCursorOff;
begin
    Reg.AX := 2;
    Intr($33,Reg);
    MCurOn := false;
    If not MouseOK then begin
       SetColor(Black);         { Clear cross }
       Cross(MKX,MKY);
    end;
end;        

{**********************************************************}

Function MousePosition ( var MX, MY : word) : word;

Var  c  : char;

Begin
If MouseOK then
  With Reg do begin
    AX := 3;
    Intr($33,Reg);
    MX := CX;                    { x-coordinate }
    MY := DX;                    { y-coordinate }
    MousePosition := BX;        { button number }
  end{with}
else begin                  { Mouse by keyboard }
 MousePosition := 0;
 If Keypressed then begin
  c := ReadK;
  If MCurOn then begin
    SetColor(Black);          { Clear old cross }
    Cross(MKX,MKY);
  end{if};
  Case c of
    KRight: begin         { "right" }
      Inc(MKX,IncX);
      If MKX>VGAX then MKX := VGAX;
     end;
    KLeft: begin           { "left" }
      If MKX<IncX then MKX := 0
                  else Dec(MKX,IncX);
     end;
    KUp: begin               { "up" }
      If MKY<IncY then MKY := 0
                  else Dec(MKY,IncY);
     end;
    KDown: begin           { "down" }
      Inc(MKY,IncY);
      If MKY>VGAY then MKY := VGAY;
     end;
    KCR:             { Enter = left }
      MousePosition := 1;
    KEsc:             { Esc = right }
      MousePosition := 2;
  end{case};
  If MCurOn then begin
    SetColor(CrossCol);         { Display cross }
    Cross(MKX,MKY);
  end{if};
 end{if Keypressed};
 MX := MKX;
 MY := MKY;
end{if};
end;

{**********************************************************}

Procedure SetMousePos( MX, MY : word);
begin
 If MouseOK then
  With Reg do begin
    AX := 4;
    DX := MY;
    CX := MX;
    Intr($33,Reg);
  end{with}
 else begin                 { Mouse by keyboard }
  If MCurOn then begin
    SetColor(Black);          { Clear old cross }
    Cross(MKX,MKY);
  end{if};
  MKX := MX;
  MKY := MY;
  If MCurOn then begin
    SetColor(CrossCol);         { Display cross }
    Cross(MKX,MKY);
  end{if};
 end{if};
end;

{**********************************************************}

Procedure SetMouseXRange(Xmin,Xmax : integer);
Begin
  With Reg do begin
    AX := 7;
    DX := Xmax;
    CX := Xmin;
    Intr($33,Reg);
  end{with};
  MouseCursorOff;
  MouseCursorOn;
End;

{**********************************************************}

Procedure SetMouseYRange(Ymin,Ymax : integer);
Begin
  With Reg do begin
    AX := 8;
    DX := Ymax;
    CX := Ymin;
    Intr($33,Reg);
  end{with};
  MouseCursorOff;
  MouseCursorOn;
End;

{**********************************************************}

  Procedure Wait;
  { Waiting for pressing any key or any mouse button }
  var x,y:word;
      c:char;
  begin
     If MouseOK then
       Repeat until KeyPressed or (MousePosition(x,y)>0)
     else
       Repeat until KeyPressed;
     If KeyPressed then
       c := ReadK
     else
       Delay(WaitTime);
  end;

{**********************************************************}

 Procedure WaitUp;
  { Waiting for releasing all mouse buttons }
  var x,y:word;
  begin
     If MouseOK then Repeat until MousePosition(x,y)=0;
  end;

{**********************************************************}

  Function CWait(var b:word):char;
  { Waiting for pressing any key or any mouse button. }
  { Returns the key or 0 if mouse button was pressed. }
  { "b" is the button pressed or 0 if key pressed. }
  Var x,y:word;
  Begin
     Repeat
       If MouseOK then b := MousePosition(x,y)
                  else b := 0
     until KeyPressed or (b>0);
     If KeyPressed then
       CWait := ReadK
     else begin
       Delay(WaitTime);
       CWait := KNull;
     end
  end;

{**********************************************************}

  Function CWaitNW(var b:word):char;
  { Waiting for pressing any key or any mouse button. }
  { Returns the key or 0 if mouse button was pressed. }
  { "b" is the button pressed or 0 if key pressed. }
  { NO WAITING !!! }
  Var x,y:word;
  Begin
     Repeat
       If MouseOK then b := MousePosition(x,y)
                  else b := 0
     until KeyPressed or (b>0);
     If KeyPressed then
       CWaitNW := ReadK
     else begin
       CWaitNW := KNull;
     end
  end;

{**********************************************************}

  Function KeyOrMouseG(s:set_of_char;
             GMA:array of PositionG; OldOp,MOut:char): char;
  { Returns an upper case option from either the set s or
    the graph menu array GMA. OldOp is the previous option.
    Changes option according to MOut (KRight=next,KLeft=
    previous). Skipping nonactive options). }

  Label 1;

  Var done,selected,on :boolean;
      x,y              :word;
      i,selpos         :integer;
      button           :word;
      c,c1             :char;
      MoveN,MoveP      :char;  { Character to move to next/previous }
      Outs             :set of char;         { Set ending selection }

  Begin
     While Keypressed do ReadK;     { Clearing keyboard buffer }
{ Taking next/previous option: }
     i := -1;
     Repeat
        Inc(i);
        If (i>High(GMA)) then i:= 0;
     Until GMA[i].opt=OldOp;        { Old option }
     If MOut=KRight then begin
        Repeat
           Inc(i);
           If (i>High(GMA)) then i:= 0;
        Until (GMA[i].x1<GetMaxX) and (GMA[i].opt<>KF1); { Move to next }
        KeyOrMouseG := GMA[i].opt;
        GoTo 1;
     end{if};
     If MOut=KLeft then begin
        Repeat
           Dec(i);
           If i<0 then i:= High(GMA);
        Until (GMA[i].x1<GetMaxX) and (GMA[i].opt<>KF1); { Move back }
        KeyOrMouseG := GMA[i].opt;
        GoTo 1;
     end{if};

{ Selecting: }
     If PullDown then begin
         MoveN := KDown; MoveP := KUp;
         Outs := s + [KLeft,KRight,KEsc];
       end
     Else begin                      { Bar menu }
         MoveN := KRight; MoveP := KLeft;
         Outs := s + [KEsc];
     end;
     done := false;
     on := false;
     selected := false;
     selpos := -1;
     SetColor(CuMeCo);
     SetLineStyle(SolidLn,0,NormWidth);
     With GMA[i] do begin              { Placing mouse pointer }
       SetMousePos((x1+x2) div 2, (y1+y2) div 2);
     end;
     MouseCursorOn;

     Repeat
     {------------------------------------------------------}
     If mouseOK then begin             { Mouse and keyboard }
      button := MousePosition(x,y);
      If Keypressed or (button>0) then begin
       If Keypressed then begin
          c := ReadK;
          { If c=KAltX then DOSHalt(0);}
          If ('a'<=c) and (c<='z') then c := UpCase(c);
          done := (c in Outs);
          If (c=KCR) and selected then begin
            c := c1;
            done := true;
           end
          else if (c = MoveP) and selected then begin
            i := selpos;
            Repeat
               Dec(i); If i<0 then i:= High(GMA);
            Until (GMA[i].x1<GetMaxX) and (GMA[i].opt<>KF1); { Move back }
            With GMA[i] do begin    { placing mouse pointer }
                SetMousePos((x1+x2) div 2, (y1+y2) div 2);
            end;
           end
          else if (c = MoveN) and selected then begin
            i := selpos;
            Repeat
               Inc(i); If (i>High(GMA)) then i:= 0;
            Until (GMA[i].x1<GetMaxX) and (GMA[i].opt<>KF1); { Move to next }
            With GMA[i] do begin    { placing mouse pointer }
                SetMousePos((x1+x2) div 2, (y1+y2) div 2);
            end;
          end{if}
        end{then}
       else begin                    { Mouse button pressed }
         If button=2 then begin        { right button = Esc }
            done := true;
            c := KEsc;
          end{then}
         else if button=1 then begin    { left button = KCR }
          i := -1;
          Repeat
            Inc(i);
            With GMA[i] do begin
             done := (x>=x1) and (x<=x2) and (y>=y1) and (y<=y2);
             c := opt;
            end{with};
          Until done or (i=High(GMA));
         end{if};
       end{if};
      end{then}
     else begin              { nothing pressed }
          i := -1;
          Repeat
            Inc(i);
            With GMA[i] do begin
             selected := (x>=x1) and (x<=x2)
                         and (y>=y1) and (y<=y2);
             If selected and (selpos<>i) then begin
                MouseCursorOff;
                If on then With GMA[selpos] do begin
                   SetColor(CuBkCo);     { Erasing rectangle }
                   Rectangle(x1, y1, x2, y2);
                end{with_if};
                selpos := i;
                SetColor(CuMeCo);
                Rectangle(x1, y1, x2, y2); { Draw rectangle }
                MouseCursorOn;
                on := true;
                c1 := opt;
             end{if};
            end{with};
          Until selected or (i=High(GMA));
          If not selected and on then
            With GMA[selpos] do begin
             MouseCursorOff;
             SetColor(CuBkCo);          { Erasing rectangle }
             Rectangle(x1, y1, x2, y2);
             SetColor(CuMeCo);
             MouseCursorOn;
             on := false;
             selpos := -1;
          end{with_if}
      end{If}
      end{if MouseOK}
     {------------------------------------------------------}
     Else begin                             { keyboard only }
       button := MousePosition(x,y);
          i := -1;
          Repeat
            Inc(i);
            With GMA[i] do begin
             selected := (x>=x1) and (x<=x2)
                         and (y>=y1) and (y<=y2);
             If selected and (selpos<>i) then begin
                MouseCursorOff;
                If on then With GMA[selpos] do begin
                   SetColor(CuBkCo);     { Erasing rectangle }
                   Rectangle(x1, y1, x2, y2);
                end{with_if};
                selpos := i;
                SetColor(CuMeCo);
                Rectangle(x1, y1, x2, y2); { Draw rectangle }
                MouseCursorOn;
                on := true;
                c1 := opt;
             end{if};
            end{with};
          Until selected or (i=High(GMA));
          c := ReadK;
          { If c=KAltX then DOSHalt(0); }
          If ('a'<=c) and (c<='z') then c := UpCase(c);
          done := (c in Outs);
          If (c=KCR) and selected then begin
            c := c1;
            done := true;
           end
          else if (c = MoveP) and selected then begin
            i := selpos;
            Repeat
               Dec(i); If i<0 then i := High(GMA);
            Until (GMA[i].x1<GetMaxX) and (GMA[i].opt<>KF1); { Move back }
            With GMA[i] do begin    { placing mouse pointer }
                SetMousePos((x1+x2) div 2, (y1+y2) div 2);
            end;
           end
          else if (c = MoveN) and selected then begin
            i := selpos;
            Repeat
               Inc(i); If (i>High(GMA)) then i:= 0;
            Until (GMA[i].x1<GetMaxX) and (GMA[i].opt<>KF1); { Move to next }
            With GMA[i] do begin    { placing mouse pointer }
                SetMousePos((x1+x2) div 2, (y1+y2) div 2);
            end;
          end{if};
     end{IF}
     Until done;
     KeyOrMouseG := c;
     If on then
       With GMA[selpos] do begin
         MouseCursorOff;
         SetColor(CuBkCo);            { Erasing rectangle }
         Rectangle(x1, y1, x2, y2);
         SetColor(CuMeCo);
         MouseCursorOn;
     end{with_if};
     If MouseOK then Delay(WaitTime);
 1:
     Eprompt;
  end{KeyOrMouse};

{**********************************************************}

    Procedure DPrompt;                 { Displaying prompt }
    Begin
       SetTextStyle(MenuFont,HorizDir,MenuSize);
       SetTextJustify(LeftText,CenterText);
       SetColor(CuPrCo);
       OutTextXY(0, VGAY - 8, SPrompt);
    End;

{----------------------------------------------------------}

    Procedure EPrompt;                    { Erasing prompt }
    Var l:word;
    Begin
       SetFillStyle(SolidFill,Black);
       Bar(0, VGAY - 15, VGAX, VGAY);
    End;

{**********************************************************}

Begin
   MouseOK := false;               { Mouse not initialised }
   MKX := VGAX div 2;
   MKY := VGAY div 2;
   MCurOn := false;
   PullDown := true;
end.





