Unit GrEditS; { Version Simpet MOUSE, Line style, no windows }
{$N+,E+}                             { Coordinates integer ! }
{$C FIXED PRELOAD PERMANENT} { Both Real and Protected mode }

Interface

Uses Dos, Crt, Graph, GGraph, PNList, CrtES, EMouseGS;

{ GGraph   = General graph }
{ PNList   = Two way linked lists (dynamic data structures) }
{ CrtES    = Crt Enhanced }
{ EMouseGS = Enhanced Mouse functions (Graphics only) }

{ This unit defines the following objects:

 POLY is a polyline. It has the following attributes:
        - Corners:Listpoint  (list of points - see PNList)
        - Color:byte
        - Width:byte
        - Style:byte.
    It has these methods:
        - Init   = setting Color and Width.
        - Create = creating the edge (Corners) using mouse:
                    any button   = points,
                    right button = last point.
        - Show = displaying the edge.
        - ShowWithArrow = edge + arrow.
        - EraseWithArrow = erasing edge + arrow.
        - Destroy = destroying corners.

 PNT is a record with coordinates Xp and Yp.

 POINT is a point with attributes:
        - Xp,Yp:word    (coordinates)
    Methods:
        - SetPos = fixing position using mouse (any button).

 PPOLY is a pointer to POLY.
 PPNT is a pointer to PNT.
 PPoint is a pointer to POINT.       }


{************************* Objects **************************}

Type   PPoly  = ^Poly;         { Pointers }
       PPnt   = ^Pnt;
       PPoint = ^Point;

     Poly = object
       Corners : Listpoint;    { List of points }
       Color   : byte;
       Width   : byte;
       Style   : byte;
      {Methods:}
       Constructor Init(Col:byte;Wid:byte;Sty:byte);
       Procedure Create;
       Procedure ColorShow(Clr : Byte);
       Procedure Show;
       Procedure Erase;
       Procedure ColorShowWithArrow(Clr : Byte);
       Procedure ShowWithArrow;
       Procedure EraseWithArrow;
       Function  FirstX : Integer;  { First Corner }
       Function  FirstY : Integer;
       Function  LastX  : Integer;   { Last Corner }
       Function  LastY  : Integer;
       Procedure Destroy;
     end;

     Point = object
       Xp,Yp:integer;                { Coordinates }
      {Methods:}
       Procedure SetPos;       { Sets the position }
     end;

     Pnt = record                    { Coordinates }
       Xp,Yp:integer
     end;


     Procedure Arrow(x1,y1,x2,y2:integer; d:byte; ar:real);
        { x1,y1 = The last but one line point }
        { x2,y2 = The last line point }
        { d = Arrow length in pixels }
        { ar = Half arrow angle in degres }

{***************************************************************}

Implementation

Const  LineColor = Cyan;       { Color of the polyline }
       CrosSize  = 7;          { Half size of moving cross }
       CrossColor= LightRed;   { Cross color }
       PolyDel   = 300;        { Initial delay - polyline }
       Arrlen    =  9;         { Arrow length }
       Arrang    = 11;         { Half arrow angle in degrees }

{********************* Global Procedures ***********************}

Procedure Arrow(x1,y1,x2,y2:integer; d:byte; ar:real);
{ x1,y1 = The last but one line point }
{ x2,y2 = The last line point }
{ d = Arrow length in pixels }
{ ar = Half arrow angle in degres }

Var a:real;              { Angle of the last line segment }
    x:real;               { Angle of the lower arrow part }
    y:real;               { Angle of the upper arrow part }
    dx1,dx2,dy1,dy2:integer;  { Increments for arrow ends }

Begin
   ar := ar*Pi/180.0;
   If x1=x2 then a := Pi/2
            else a := ArcTan(Abs(y1-y2)/Abs(x2-x1));
   x := Pi/2 - a - ar;
   y := Pi/2 - a + ar;
   If x2>x1 then begin    { 1st and 4th quadrants }
                    dx1 := -round(d*Sin(x));
                    dx2 := -round(d*Sin(y));
                 end
            else begin    { 2nd and 3rd quadrants }
                    dx1 := round(d*Sin(x));
                    dx2 := round(d*Sin(y));
                 end;
   If y2>y1 then begin    { 3rd and 4th quadrants }
                    dy1 := -round(d*Cos(x));
                    dy2 := -round(d*Cos(y));
                 end
            else begin    { 1st and 2nd quadrants }
                    dy1 := round(d*Cos(x));
                    dy2 := round(d*Cos(y));
                 end;
   MoveTo(x2,y2);      { To be sure }
   LineRel(dx1,dy1);
   MoveTo(x2,y2);
   LineRel(dx2,dy2);
   MoveTo(x2,y2);
End{of arrow};

{*********************** Methods ***************************}

  Constructor Poly.Init;
  begin
     Color := Col;
     Width := Wid;
     Style := Sty;
     Corners := nil;
  end;

{***********************************************************}

  Procedure Poly.Create;    { Creates a Poly made of edges }
  { No mouse initiation !!! - is slow }

  Var b       :word;                      { Button pressed }
      xw,yw   :word;                   { Mouse coordinates }
      x,y     :integer;                { Mouse coordinates }
      xb,yb   :integer;                {  Line's beginning }
      xe,ye   :integer;                      {  Line's end }
      m,n     :Nodepoint;                   { List element }
               { List elements; contain pointers to points }
      p,q     :PPnt;
      i       :integer;
      a       :array[1..32] of word;
      Reg     :Registers;
      c       :char;
          
    {------------------------------------------------------}

    procedure DrawLine;         { Connects last two points }
    begin
       GSetColor(LineColor);
       Line(xb,yb,xe,ye);
    end;

    {------------------------------------------------------}

    procedure EraseLine;    { Erases line between last two }
    begin
       DrawLine;
    end;

    {------------------------------------------------------}

    procedure LineMN;   { Connects points "m" and "n" }
    begin
       p := m^.el; q := n^.el;
       GSetColor(LineColor);
       Line(p^.Xp,p^.Yp,q^.Xp,q^.Yp);
    end;

    {------------------------------------------------------}

    procedure InsCorner;             { Inserts point (X,Y) }
    var p:PPnt;
        n:Nodepoint;
    begin
       new(p); p^.Xp := X; p^.Yp := Y;
       new(n); n^.el := p;
       InsLast(n,Corners);
    end;

    {--------------------- body ---------------------------}

   Begin                             { Body of Poly.Create }
     SetWriteMode(XORPut);
     Corners := NewList;
     SetLineStyle(SolidLn,0,NormWidth);

 (* Does not work for some mice !!! { Mouse cursor = fine cross: }
    If MouseOK then begin
     For i:=17 to 32 do  a[ i] := $0100;
                         a[24] := $FFFF;
     For i:= 1 to 16 do  a[ i] := $FFFF;
     With Reg do begin
       AX := 9;
       BX := 7;
       CX := 7;
       ES := Seg(a);
       DX := Ofs(a);
       Intr($33,Reg);
     end;
    end{if};      *)

     MouseCursorOn;
     Delay(PolyDel);
     b := MousePosition(xw,yw);
     xe := xw; ye := yw;
     Repeat
        b := MousePosition(xw,yw);
        x := xw; y := yw;
        If (xe<>x) or (ye<>y) then begin
           xe := x; ye := y;
        end{if};
        If b=1 then begin  { Waiting for the 1st point }
          InsCorner;                       { 1st point }
          xb := x; yb := y;
        end{if}
     until not Empty(Corners) or (b=2);
     Delay(PolyDel);

     Repeat                         { Inputting points }
        b := MousePosition(xw,yw);
        x := xw; y := yw;
        If Keypressed then begin
           c := UpCase(ReadK);
           If      c='H' then y := yb
           else If c='V' then x := xb;
           SetMousePos(x,y);
        end{if};
        If (xe<>x) or (ye<>y) then begin
           MouseCursorOff;
             EraseLine;
           xe := x; ye := y;
             DrawLine;
           MouseCursorOn;
        end{if};
        If b>0 then begin                  { new point }
           InsCorner;
           xb := x; yb := y;
           If b=1 then Delay(PolyDel);
        end{if};
     until (b=2) or Empty(Corners);
(*     If MouseOK then
       If MouseInit then;  { Restoring default cursor shape }   *)
     MouseCursorOff;
     SetWriteMode(NormalPut);
     ShowWithArrow;
  end; {of Poly.Create}

{***********************************************************}

  Procedure Poly.ColorShow(Clr : Byte);  { Shows a Polyline }
  Var m,n :Nodepoint;
          { List elements; contain pointers to points }
      p,q :PPnt;
      
    {------------------------------------------------------}

    procedure LineMN;   { Connects points "m" and "n" }
    begin
       p := m^.el; q := n^.el;
       Line(p^.Xp,p^.Yp,q^.Xp,q^.Yp);
    end;

    {------------------------------------------------------}

    Begin                                 { Poly.ColorShow }
      If Corners<>nil then
       If not Empty(Corners) then begin
         GSetColor(Clr);
         SetLineStyle(Style,0,Width);
         m := TakeFirst(Corners);
         n := TakeNext(m);
         while n<>nil do begin
            LineMN;
            m := n;
            n := TakeNext(n);
         end{while};
       end{if}
    End{Poly.ColorShow};

{***********************************************************}

  Procedure Poly.Show;      { Shows a Polyline in its color }
  Begin
     ColorShow(Color)
  End;

{***********************************************************}

  Procedure Poly.Erase;                 { Erases a Polyline }
  Begin
     ColorShow(Black)
  End;

{***********************************************************}

  Procedure Poly.ColorShowWithArrow(Clr : Byte);
                              { Shows a Polyline with arrow }
  Var m,n,k :Nodepoint;
          { List elements; contain pointers to points }
      p,q   :PPnt;
      
    {------------------------------------------------------}

    procedure LineMN;   { Connects points "m" and "n" }
    begin
       p := m^.el; q := n^.el;
       Line(p^.Xp,p^.Yp,q^.Xp,q^.Yp);
    end;

    {------------------------------------------------------}

  Begin                          { Poly.ColorShowWithArrow }
      If Corners<>nil then
       If not Empty(Corners) then begin
         GSetColor(Clr);
         SetLineStyle(Style,0,Width);
         m := TakeFirst(Corners);
         k := m;
         n := TakeNext(m);
         while n<>nil do begin
            LineMN;
            k := m;
            m := n;
            n := TakeNext(n);
         end{while};
         If k<>m then begin           { Drawing arrow }
            p := k^.el; q := m^.el;
            SetLineStyle(SolidLn,0,Width);
            Arrow(p^.Xp,p^.Yp,q^.Xp,q^.Yp,ArrLen,ArrAng);        
         end{if};
       end{if}
  End{Poly.ColorShowWithArrow};

{***********************************************************}

  Procedure Poly.ShowWithArrow;
                 { Shows a Polyline with arrow in its color }
  Begin
     ColorShowWithArrow(Color)
  End;

{***********************************************************}

  Procedure Poly.EraseWithArrow;
                 { Shows a Polyline with arrow in its color }
  Begin
     ColorShowWithArrow(Black)
  End;

{***********************************************************}

  Function  Poly.FirstX : Integer;           { First Corner }
  Var m : Nodepoint;
          { List elements; contain pointers to points }
      p : PPnt;
  Begin
     FirstX := -1;
     If Corners<>nil then
      If not Empty(Corners) then begin
        m := TakeFirst(Corners);
        p := m^.el;
        FirstX := p^.Xp;
      end{if};
  End;

{***********************************************************}

  Function  Poly.FirstY : Integer;           { First Corner }
  Var m : Nodepoint;
          { List elements; contain pointers to points }
      p : PPnt;
  Begin
     FirstY := -1;
     If Corners<>nil then
      If not Empty(Corners) then begin
        m := TakeFirst(Corners);
        p := m^.el;
        FirstY := p^.Yp;
      end{if};
  End;

{***********************************************************}

  Function  Poly.LastX : Integer;             { Last Corner }
  Var m : Nodepoint;
          { List elements; contain pointers to points }
      p : PPnt;
  Begin
     LastX := -1;
     If Corners<>nil then
      If not Empty(Corners) then begin
        m := TakeLast(Corners);
        p := m^.el;
        LastX := p^.Xp;
      end{if};
  End;

{***********************************************************}

  Function  Poly.LastY : Integer;             { Last Corner }
  Var m : Nodepoint;
          { List elements; contain pointers to points }
      p : PPnt;
  Begin
     LastY := -1;
     If Corners<>nil then
      If not Empty(Corners) then begin
        m := TakeLast(Corners);
        p := m^.el;
        LastY := p^.Yp;
      end{if};
  End;

{***********************************************************}

  Procedure Poly.Destroy;                { Destroys corners }

  Var n    :Nodepoint;
      p    :PPnt;

  Begin
    If Corners<>nil then begin
     While not Empty(Corners) do begin
        n := RemFirst(Corners);
        p := n^.el;
        Dispose(p);
        Dispose(n);
     end{while};
     Dispose(Corners);
     Corners := nil;
    end{if};
  End;

{***********************************************************}

  Procedure Point.SetPos;
  var X,Y:word;
      b:word;
  begin                    { Body of Point.SetPos }
     MouseCursorOn;
     Repeat
        b := MousePosition(X,Y);
        If b>0 then begin
           Xp := X;
           Yp := Y;
        end{if};
     until b>0;
     MouseCursorOff;
  end{of Point.SetPos};

end.