Unit Inhibits;                      { Inhibitor arcs }

Interface

  Uses Graph, GGraph, PNList, Petri;

  Type  InhibitorArc = Object(PetriArc)
           Procedure AShow;
           Procedure AErase;
        End{InhibitorArc};

        InhibitorPlace = Object(PetriPlace)
        End{InhibitorPlace};

        InhibitorTransition = Object(PetriTransition)
           Function  Enabled : Boolean;
           Procedure TShow;
           Procedure ShowStatus;
           Procedure StartFire;             { Remove tokens }
           Procedure EndFire;                  { Add tokens }
           Procedure Fire;           { Firing: Remove & Add }
           { Others the same as Petri objects }
        End{Transition};

  Const InitialInhRadius = 3;

  Var InhCircleRadius : Byte;

Implementation

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

  Procedure InhibitorArc.AShow;
  Begin
     If AType<=0 then      { Normal arc }
       ShowWithArrow
     Else begin
       Show;
       If AType=1 then    { Inhibitor arc }
          Circle(LastX ,LastY ,InhCircleRadius);
     end;
     ShowWeight;
  end;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

  Procedure InhibitorArc.AErase;
  Begin
     If AType<=0 then      { Normal arc }
       EraseWithArrow
     Else begin
       Erase;
       If AType=1 then    { Inhibitor arc }
          Circle(LastX ,LastY ,InhCircleRadius);
     end;
     EraseWeight;
  end;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

  Function  InhibitorTransition.Enabled:Boolean;
  var n:Nodepoint;
      a:PetriAP;            { Arc }
      p:PetriPP;          { Place }
      en:Boolean;
  Begin
     If Empty(Inlist) then Enabled := false
     Else Begin
      n := TakeFirst(Inlist);
      Repeat
        a := n^.el;
        p := a^.Place;
        Case a^.AType of                     { This place OK or not }
         0,2: en := p^.Marking >= a^.Weight; { Normal/Testing arc }
           1: en := p^.Marking < a^.Weight;  { Inhibitor arc }
        end{case};
        n := TakeNext(n);
      Until (n=nil) or not en;
      Enabled := en;
     end{if};
  end;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

  Procedure InhibitorTransition.TShow;
  Begin
     ShowShape;
     Showstatus;
  end;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

  Procedure InhibitorTransition.ShowStatus;
  Var s : String;
  Begin
     If Enabled or Firing then Begin
       If GBandW then Begin
         GSetColor(White);
         If Firing then
           SetLineStyle(DottedLn,0,NormWidth)
         else
           SetLineStyle(SolidLn,0,NormWidth);
         If Direct=Horiz then
           Rectangle(Xpos-Size div 2-3,Ypos-4,Xpos+Size div 2+3,Ypos+4)
         else
           Rectangle(Xpos-4,Ypos-Size div 2-3,Xpos+4,Ypos+Size div 2+3);
        end
       else Begin
         If Firing then
           SetFillStyle(SlashFill,Red)
         else
           SetFillStyle(SolidFill,Red);
         If Direct=Horiz then
           Bar(Xpos-Size div 2-2,Ypos-3,Xpos+Size div 2+2,Ypos+3)
         else
           Bar(Xpos-3,Ypos-Size div 2-2,Xpos+3,Ypos+Size div 2+2);
       end{if};
       If Firing then Begin
        Str(FireEnd:3:2,s);
        GSetColor(TimCol);
        SetTextStyle(MenuFont,HorizDir,MenuSize);
        SetTextJustify(CenterText,CenterText);
        If Direct=Horiz then Begin
           SetTextJustify(LeftText,BottomText);
           OutTextXY(Xpos - Size div 2 - 4, Ypos - 8, s);
          end{then}
        else Begin
           SetTextJustify(CenterText,BottomText);
           OutTextXY(Xpos, Ypos - Size div 2 - 4, s);
        end{if};
       end{if};
     end{if};
  end;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

  Procedure InhibitorTransition.StartFire;
  var n:Nodepoint;
      a:PetriAP;            { Arc }
      p:PetriPP;          { Place }
  Begin
      n := TakeFirst(Inlist);
      While n<>nil do Begin
        a := n^.el;
        p := a^.Place;
        If a^.AType=0 then Dec(p^.Marking,a^.Weight);
        n := TakeNext(n);
      end{while};
  end;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

  Procedure InhibitorTransition.EndFire;
  var n:Nodepoint;
      a:PetriAP;            { Arc }
      p:PetriPP;          { Place }
  Begin
      n := TakeFirst(Outlist);
      While n<>nil do Begin
        a := n^.el;
        p := a^.Place;
        {If a^.AType=0 then} Inc(p^.Marking,a^.Weight);
        n := TakeNext(n);
      end{while};
  end;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

  Procedure InhibitorTransition.Fire;
  Begin
      StartFire;
      EndFire;
  end;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

 Begin
    InhCircleRadius := InitialInhRadius
 End.
