Unit PNet6;                    { Version 6 - User objects }
{$N+,E+}

Interface

Uses Crt, Graph, GGraph, CrtES, EMouseGS, PNList, GrEditS,
     WindowsS, GPNHelp, Petri, UserPart, Dos, Simpet5, Randoms;

{ GGraph    = General graph }
{ CrtES     = Crt enhanced (some more routines) }
{ EMouseGS  = Enhanced mouse functions (Graphics only) }
{ PNList    = two way linked lists (dynamic data struct.) }
{ GrEditS   = graphical editor (objects: polyline, point) }
{ WindowsS  = windows in graphics }
{ GPNHelp   = Context Help routines - graphics }

{***************** Public declarations *******************}

Const { Environment: }
      MNTCol      = LightGreen;   { Window normal text color }
      WBCol       = Black;         { Window background color }
      METCol      = LightMagenta;  { Window emphasized color }
      PCol        = LightCyan;        { Prompt message color }

Type  PNp = ^PNPlace;                  { Pointers to objects }
      PNt = ^PNTransition;
      PNa = ^PNArc;
      PNn = ^PNNotice;
      PN  = ^PNet;

     PNPlace = Object(UserPlace)
        Procedure PLocate;
     End{Place};

     PNTransition = Object(UserTransition)
        Procedure TLocate;
     End{Transition};

     PNArc = Object (UserArc)
     End{Arc};

     PNNotice = Record                  { Event notice }
        NTime  : Real;
        NTrans : PNt;
        NNext  : PNn;
     End{PNNotice};

     PNet = Object
        Nopl : Integer;             { Number of places }
        Notr : Integer;        { Number of transitions }
        NotrG : Integer;   { Next generated transition }
                                  { All initially zero }
        Name : string[Namelen];                 { Name }
        FName : string[50];                { File Name }
        Places : Listpoint;           { List of places }
        Transitions : Listpoint; { List of transitions }
        Time : Real;                      { Model time }
        SQS : PNn;         { Sequencing set - calendar }
        Terminate : Boolean;       { End of simulation }
        FireTran : PNt;       { Transition being fired }
       {Methods:}
        Procedure Create(var created: Boolean);
        Procedure Show;
        Function SelectPlace : PNp;
        Function TestPlace: PNp;
        Function GivePlace(W:DWindowP; var found:Boolean):PNp;
        Function SelectTrans : PNt;
        Function TestTrans: PNt;
        Function GiveTrans(W:DWindowP; var found:Boolean):PNt;
        Function TestArc(var typ:Char; var point: NodePoint;
                         var tran:PNt; var direct:Char): Nodepoint;
           { Searching for arc. Testarc,point=LIST POINT!
             typ=P(oint)/L(ine segment). For L point=first point.
             tran=transition. direct=Direction I/O }
        Procedure Modify;
        Procedure Simulate;
        Procedure Store (var F: Text; Option: Char);
        Procedure Load  (var F: Text);
        Procedure Destroy;
               { Destroying P-net from RAM - not net as such ! }
     end{PNet};

Var   Nets     :Listpoint;     { List of existing P-nets }
      SNet     :PN;                     { Selected P-net }
      Net      :PN;                      { Current P-net }
      MenuOut1 :char;{ Menu output character at levels 1 }
      Selected :Boolean;                { P-net selected }
      ArcName  :string;              { Selected arc name }

  Procedure MainTitles(On:Boolean);  { Main bar menu titles }

Implementation

{****************************************************************}

Const
      AWidth     = 1;                                { Arc width }
      FireDelayf = 1500;          { Delays between firings in ms }
      WaitTime   = 250;                     { Mouse waiting time }
      CurDelay   = 30;               { Delay to see mouse cursor }
      LastL      = 13;                { Display window last line }

    { NOTE: Constants in routines are only windows coordinates ! }

{-------------------- Global procedures -------------------------}

  Procedure MainTitles(On:Boolean);       { Main bar menu titles }
  { On = whether the menu is active. If not nothing highlighted. }
   Begin
     If Selected and On then Wex( 2,1,'E')
                        else Wx ( 2,1,'E'); Wx(3,1,'dit');
     If Selected and On then Wex( 8,1,'S')
                        else Wx ( 8,1,'S'); Wx(9,1,'imulate');
     If On then Wex(18,1,'F') else Wx(18,1,'F'); Wx(19,1,'ile');
     If On then Wex(24,1,'M') else Wx(24,1,'M'); Wx(25,1,'anagement');
     If On then Wex(36,1,'H') else Wx(36,1,'H'); Wx(37,1,'elp');
     If On then Wex(42,1,'Q') else Wx(42,1,'Q'); Wx(43,1,'uit');
     If On then Wex(78,1,'F1') else begin
       SetFillStyle(SolidFill,Black);
       Bar(607,0,639,16);
     end;
   End;

{******************** Methods for PNplace **********************}

  Procedure PNPlace.PLocate;

  Var x,y   : Word;
      dx,dy : Integer;
      n,m,k : Nodepoint;
      t     : PNt;
      a     : PNa;
      p     : PPNt;

  Begin
   MouseCursorOn;
   While     MouseOK and (MousePosition(x,y)<>0) or
         not MouseOK and (MousePosition(x,y)<>2) do begin
     dx := Integer(x) - Xpos;             { Shifts }
     dy := Integer(y) - Ypos;
     MouseCursorOff;
     PErase;                              { Erasing place }

     n := Takefirst(SNet^.Transitions);
     While (n<>nil) do begin
       t := n^.el;

       m := Takefirst(t^.InList);         { Searching in Inlist }
       While (m<>nil) do begin
         a := m^.el;
         With a^ do
           If (Place^.Name=Name) and (Corners<>nil) then begin
             AErase;
             k := TakeFirst(Corners);
             p := k^.el;
             p^.Xp := p^.Xp + dx;
             p^.Yp := p^.Yp + dy;
             AShow;
           end;
         m := Takenext(m)
       end{while};

       m := Takefirst(t^.OutList);        { Searching in Outlist }
       While (m<>nil) do begin
         a := m^.el;
         With a^ do
           If (Place^.Name=Name) and (Corners<>nil) then begin
             AErase;
             k := TakeLast(Corners);
             p := k^.el;
             p^.Xp := p^.Xp + dx;
             p^.Yp := p^.Yp + dy;
             AShow;
           end;
         m := Takenext(m)
       end{while};

       n := Takenext(n)
     end{while};

     Xpos := x; Ypos := y;
     If Xpos>0 then PShow;                { Displaying place }
     MouseCursorOn;
     Delay(CurDelay);
   end{while};
  End{PLocate};

{**************** Methods for PNtransition *******************}

  Procedure PNTransition.TLocate;

  Var x,y   : Word;
      dx,dy : Integer;
      n,m   : Nodepoint;
      a     : PNa;
      p     : PPNt;

  Begin
   MouseCursorOn;
   While     MouseOK and (MousePosition(x,y)<>0) or
         not MouseOK and (MousePosition(x,y)<>2) do begin
     dx := Integer(x) - Xpos;       { Shifts }
     dy := Integer(y) - Ypos;
     MouseCursorOff;
     TErase;                      { Erasing transition }

     n := TakeFirst(InList);         { Erasing arcs in }
     While n<>nil do begin
       a := n^.el;
       If a^.Corners<>nil then begin
        a^.AErase;
        m := TakeLast(a^.Corners);
        p := m^.el;
        p^.Xp := p^.Xp + dx;
        p^.Yp := p^.Yp + dy;
       end;
       n := TakeNext(n);
     End{while};

     n := TakeFirst(OutList);     { Erasing arcs out }
     While n<>nil do begin
       a := n^.el;
       If a^.Corners<>nil then begin
        a^.AErase;
        m := TakeFirst(a^.Corners);
        p := m^.el;
        p^.Xp := p^.Xp + dx;
        p^.Yp := p^.Yp + dy;
       end;
       n := TakeNext(n);
     End{while};

     Xpos := x; Ypos := y;
     If Xpos>0 then TShow;        { Displaying transition }

     n := TakeFirst(InList);      { Displaying arcs in }
     While n<>nil do begin
        a := n^.el;
        a^.AShow;
        n := TakeNext(n);
     End{while};

     n := TakeFirst(OutList);     { Displaying arcs out }
     While n<>nil do begin
        a := n^.el;
        a^.AShow;
        n := TakeNext(n);
     End{while};

     MouseCursorOn;
     Delay(CurDelay);
   end{while};
  End{TLocate};

{******************** Methods for PNArc ********************}

{********************* Methods for PNet ********************}

  Procedure PNet.Create(var created: Boolean);

  Const  x1 =  100;                    { Create window - Wn }
         y1 =   64;
         x2 =  390;
         y2 =  204;

  Var i    :Integer;
      net  :PN;
      n    :Nodepoint;
      s,z,r :string;
      c    :Char;
      found,OK:Boolean;
      Wn   :DWindowP;

  Begin
   created := false;
   New(Wn);
   With Wn^ do begin
    WOpen(x1,y1,x2,y2,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
    If WOK then begin
     Wrt (2,2,'Enter the name of the new P-net'); WrtE(2,3,':');
     WaitUp;
     WRSDOS (4,3,s);
     Wrub (4,3,s);
     If s<>'' then begin
      s := Copy(s,1,8);
      WrtE(4,3,s);
      r := s;
      Upper(s);
      found := false;
      n := Takefirst(Nets);
      While (n<>nil) and not found do begin
        net := n^.el;
        z := net^.Name;
        Upper(z);
        found := z=s;
        n := Takenext(n)
      end{while};
      WClear;
      If found then begin
        WrtNEN (3,2,'P-net ',r,'');
        Wrt (3,3,'already exists !');
        WrtE(3,4,'Use other name');
        Wrt (3,7,'Press any key'); Wait;
       end{then}
    else begin                      { OK - not found }
     Name  := r;
     FName := r;
     Nopl  := 0;
     Notr  := 0;
     NotrG := 0;
     Time  := 0;
     Places := Newlist;
     Transitions := Newlist;
     SQS := nil;
     Terminate := false;
     created := true;

    end{if};
    end{if};
    WClose;
   end{WOK};
  end{with};
  Dispose(Wn);
 end;

{***********************************************************}

  Function PNet.SelectPlace : PNp;
          { Selects the place using mouse. Returns pointer. }

  Var  Selected :Boolean;
       n        :Nodepoint;
       p        :PNp;
       x,y,b    :word;

   {...............................................}
   Function CursorIn: Boolean;
   var d,e  :real;
   begin
    d := p^.Xpos-Integer(x);
    e := p^.Ypos-Integer(y);
    CursorIn := ((d*d + e*e) < Sqr(p^.Size))
                and (p^.Xpos>0);
   end;
   {...............................................}

  Begin
   Selected := false; 
   MouseCursorOn;
   Repeat
     p := nil;
     b := MousePosition (x,y);
     If b=1 then begin              { Left button pressed }
       n := TakeFirst(Places);      { Searching in Places }
       If n<>nil then p := n^.el;
       While (n<>nil) and not CursorIn do begin
         n := TakeNext(n);               
         If n<>nil then p := n^.el;
       end{while};
       Selected := (n<>nil);
     end{if};
   Until Selected or (b=2);
   MouseCursorOff;
   SelectPlace := p;
   Delay(WaitTime);
  End{SelectPlace};

{***********************************************************}

  Function PNet.TestPlace : PNp;
{ Tests whether the place is selected. Returns pointer. }

  Var  n        :Nodepoint;
       p        :PNp;
       x,y,b    :word;

   {...............................................}
   Function CursorIn: Boolean;
   var d,e  :real;
   begin
    d := p^.Xpos-Integer(x);
    e := p^.Ypos-Integer(y);
    CursorIn := ((d*d + e*e) < Sqr(p^.Size))
                and (p^.Xpos>0);
   end;
   {...............................................}

  Begin
       b := MousePosition (x,y);
       n := TakeFirst(Places);      { Searching in Places }
       If n<>nil then p := n^.el;
       While (n<>nil) and not CursorIn do begin
         n := TakeNext(n);
         If n<>nil then p := n^.el;
       end{while};
       { Selected := (n<>nil); }
       If (n<>nil) then TestPlace := p else TestPlace := nil;
  End{TestPlace};

{***********************************************************}

  Function PNet.GivePlace(W:DWindowP;var found:Boolean):PNp;

     Var s    :string;
         n    :NodePoint;
         p    :PNp;
         c    :char;
         b    :word;

     Begin
       found := false; p := nil;
       With W^ do begin
          Wrt (1,1,'Enter place name');
          Wrt (1,2,'or select by mouse');
          WrtE(1,3,': ');
          MouseCursorOn;
          c := CWaitNW(b);
          MouseCursorOff;

         If     MouseOK and (c<>KNull) or             { key pressed }
            not MouseOK and not(c in [KUp,KDown,KLeft,KRight])
            then begin                          { Place by name }
          WRSC(3,3,s,c);
          s := Copy(s,1,15);
          If s='' then { Write(#7) }
          else begin                         { name given }
           n := Takefirst( Places);
           While (n<>nil) and not found do begin
             p := n^.el;
             If (p^.Name = s) then found := true;
             n := Takenext(n)
           end{while};
           If not found then begin
             WClear; p := nil;
             WrtE(1,1,'Place '); WrtE(7,1,s);
             WrtE(1,2,'has not been found !');
             Wrt (1,4,'Press any key');
             Wait;
           end{if}
          end{if}
         end{then}

         Else begin                      { Place by mouse }
           p := SelectPlace;
           found := p<>nil
         End;
         GivePlace := p;
       end{with};
  End{GivePlace};

{***********************************************************}

  Function PNet.SelectTrans: PNt;
     { Selects the transition using mouse. Returns pointer. }

  Var  Selected :Boolean;
       n        :Nodepoint;
       t        :PNt;
       x,y,b    :Word;

   (*{........................................................}
   Function CursorIn: Boolean;
   Var dx,dy: Integer;
   begin
    If t^.Xpos>x then dx := t^.Xpos - x else dx := x - t^.Xpos;
    If t^.Ypos>y then dy := t^.Ypos - y else dy := y - t^.Ypos;
    If t^.Direct=Horiz then
      CursorIn := (dy <= 4) and (dx <= (t^.Size div 2 + 3))
                  and (t^.Xpos>0)
    else
      CursorIn := (dx <= 4) and (dy <= (t^.Size div 2 + 3))
                  and (t^.Xpos>0);
   end;
   {........................................................}*)
   {...............................................}
   Function CursorIn: Boolean;
   var d,e,f  :real;
   begin
    d := t^.Xpos - Integer(x);
    e := t^.Ypos - Integer(y);
    f := t^.Size/2;
    CursorIn := ((d*d + e*e) < f*f) and (t^.Xpos>0);
   end;
   {...............................................}

  Begin
   Selected := false;
   MouseCursorOn;
   Repeat
     t := nil;
     b := MousePosition (x,y);
     If b=1 then begin                   { Left button pressed }
       n := TakeFirst(Transitions); { Searching in Transitions }
       If n<>nil then t := n^.el;
       While (n<>nil) and not CursorIn do begin
         n := TakeNext(n);               
         If n<>nil then t := n^.el;
       end{while};
       Selected := (n<>nil);
     end{if};
   Until Selected or (b=2);
   MouseCursorOff;
   SelectTrans := t;
   WaitUp;
{   Delay(WaitTime);  }
  End{SelectTrans};

{***********************************************************}

  Function PNet.TestTrans: PNt;
{ Tests whether the transition is selected. Returns pointer. }

  Var  n        :Nodepoint;
       t        :PNt;
       x,y,b    :word;

   (*{........................................................}
   Function CursorIn: Boolean;
   Var dx,dy: Integer;
   begin
    If n<>nil then begin
     If t^.Xpos>x then dx := t^.Xpos - x else dx := x - t^.Xpos;
     If t^.Ypos>y then dy := t^.Ypos - y else dy := y - t^.Ypos;
     If t^.Direct=Horiz then
      CursorIn := (dy <= 4) and (dx <= (t^.Size div 2 + 3))
                  and (t^.Xpos>0)
     else
      CursorIn := (dx <= 4) and (dy <= (t^.Size div 2 + 3))
                  and (t^.Xpos>0);
    end{if};
   end;
   {........................................................}*)
   {...............................................}
   Function CursorIn: Boolean;
   var d,e,f  :real;
   begin
    d := t^.Xpos - Integer(x);
    e := t^.Ypos - Integer(y);
    f := t^.Size/2;
    CursorIn := ((d*d + e*e) < f*f) and (t^.Xpos>0);
   end;
   {...............................................}

  Begin
       b := MousePosition (x,y);
       n := TakeFirst(Transitions); { Searching in Transitions }
       If n<>nil then t := n^.el;
       While (n<>nil) and not CursorIn do begin
         n := TakeNext(n);
         If n<>nil then t := n^.el;
       end{while};
       { Selected := (n<>nil); }
       If (n<>nil) then TestTrans := t else TestTrans := nil;
  End{TestTrans};

{***************************************************************}

  Function PNet.GiveTrans(W:DWindowP;var found:Boolean):PNt;

     Var s    :string;
         n    :NodePoint;
         t    :PNt;
         c    :char;
         b    :word;

     Begin
       found := false; t := nil;
       With W^ do begin
        Wrt (1,1,'Enter transition name');
        Wrt (1,2,'or select by mouse');
        WrtE(1,3,': ');
        MouseCursorOn;
        c := CWaitNW(b);
        MouseCursorOff;

         If     MouseOK and (c<>KNull) or        { key pressed }
            not MouseOK and not(c in [KUp,KDown,KLeft,KRight])
            then begin                    { Transition by name }
          WRSC(3,3,s,c);
          s := Copy(s,1,15);
          If s='' then { Write(#7) }
          else begin                          { name given }
           n := Takefirst(Transitions);
           While (n<>nil) and not found do begin
             t := n^.el;
             If (t^.Name = s) then found := true;
             n := Takenext(n)
           end{while};
           If not found then begin
             WClear; t := nil;
             WrtE(1,1,'Transition '); WrtE(12,1,s);
             WrtE(1,2,'has not been found !');
             Wrt (1,4,'Press any key');
             Wait;
           end{if}
          end{if}
        end{then}

         Else begin                  { Transition by mouse }
           t := SelectTrans;
           found := t<>nil
         End;
         GiveTrans := t;
       end{with};
     End{GiveTrans};

{***************************************************************}

   Function PNet.TestArc(var typ:Char; var point: NodePoint;
                         var tran:PNt; var direct:Char): Nodepoint;
           { Searching for arc. Testarc,point=LIST POINT!
             typ=P(oint)/L(ine segment). For L point=first point.
             tran=transition. direct=Direction I/O }

     Const d=5; e=5; f=5;

     Var n,m,k,nk :NodePoint;
         a    :PNa;
         t    :PNt;
         p,np   :PPnt;
         b,x,y  :word;
         mx,my,x1,y1,x2,y2 :longint;
         found :Boolean;

     {.............................................}

     Function MinI(i1,i2:longint):longint;
     begin if i1>i2 then MinI:=i2 else MinI:=i1 end;

     {.............................................}

     Function MaxI(i1,i2:longint):longint;
     begin if i1>i2 then MaxI:=i1 else MaxI:=i2 end;

     {.............................................}

     Function IsNear(x1,y1, x2,y2, x,y, d:longint): Boolean;
     var a2,b2,c2:longint;
     begin
       a2:=(x-x1)*(x-x1)+(y-y1)*(y-y1);
       b2:=(x-x2)*(x-x2)+(y-y2)*(y-y2);
       c2:=(x2-x1)*(x2-x1)+(y2-y1)*(y2-y1);
       IsNear:= (b2 - ((c2+b2-a2) / (4*c2) *(c2+b2-a2)))<=(d*d);
     end;

     {.............................................}

     Procedure TestArcList;
                       { TESTING POINTS & SEGMENTS }
     Begin                            { m prepared }
       While (m<>nil) and not found do begin
         a := m^.el;
         k := TakeFirst(a^.Corners);
         If k<>nil then begin
          p := k^.el;
          x1 := p^.Xp;               { First Point coordinates }
          y1 := p^.Yp;

          If (x1-d<mx) and (mx<x1+d) and    { Point Selected ? }
              (y1-d<my) and (my<y1+d) then begin
                found:=true;
                typ := 'P';
                point := k;
                Testarc := m;
          end;
         end{first point};

         While (k<>nil) and not found do begin
           nk := TakeNext(k);
           If not found and (nk<>nil) then begin
             np := nk^.el;
             x2 := np^.Xp;             { Next point coordinates }
             y2 := np^.Yp;
                                        { Next Point Selected ? }
             If (x2-d<mx) and (mx<x2+d) and
              (y2-d<my) and (my<y2+d) then begin
                found:=true;
                typ := 'P';
                point := nk;
                Testarc := m;
             end{next point};

             If not found then             { Segment selected ? }
              If (MinI(x1,x2)-e<mx) and (mx<MaxI(x1,x2)+e) and
                 (MinI(y1,y2)-e<my) and (my<MaxI(y1,y2)+e)
              then If IsNear(x1,y1, x2,y2, mx,my, f) then begin
                found:=true;
                typ := 'L';
                point := k;
                Testarc := m;
             end{if};
           end{next&segment};
           k := nk;
           x1 := x2; y1 := y2;
         end{while};
         m := TakeNext(m)
       end{while};
     End;

  {.................PNet.TestArc.body...........................}

  Begin
     TestArc := nil; found := false;
     b := MousePosition(x,y);
     mx := x; my := y;
     n := Takefirst(Transitions);
     While (n<>nil) and not found do begin
       t := n^.el; tran := t;
       m := Takefirst(t^.InList);         { Searching in Inlist }
       If m<>nil then begin
         TestArcList;
         ArcName := a^.Place^.Name + ' -> ' + t^.Name;
         direct := 'I';
       end;
       If not found then begin
         m := Takefirst(t^.OutList);     { Searching in Outlist }
         If m<>nil then begin
           TestArcList;
           ArcName := t^.Name + ' -> ' + a^.Place^.Name;
           direct := 'O';
         end;
       end{if};
       n := Takenext(n);
     end{while};
  End{PNet.TestArc};

{***************************************************************}


  Procedure PNet.Modify;

  Const  x1 =   0; y1 =  23;   { Modify (Edit) main window - Wn }
         x2 = 127; y2 =  87;

         x1d = 380; y1d =  26;              { Dialog window - Wd}
         x2d = 631; y2d = 103;

  Var Option   :char;
      Wn       :PWindowP;             { Modification window }
      Wd       :DWindowP;                   { Dialog window }
      Size     :Integer;
      c        :char;
      p        :Point;
      pl       :PNp;
      tr       :PNt;
      ar       :PNa;
      m,n      :Nodepoint;
      po       :PPnt;                      { Pointer to Pnt }
      found    :Boolean;
      X,Y      :word;          { Start coordinates for arcs }
      Lines    :byte;
      OldOp,MOut : char;

{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

    Function MenuMod(OldOp,MOut:char):char;
                                  { Main modification menu }
    Const
     MenuModGA:    { Positions and options for this menu }
     array[1..3] of PositionG =
                 (( x1:  3; y1: 31; x2:124; y2: 47; opt:'S' ),
                  ( x1:  3; y1: 47; x2:124; y2: 63; opt:'R' ),
                  ( x1:  3; y1: 63; x2:124; y2: 79; opt:'D' ));
    Begin
     With Wn^ do begin
       WClear;
       WrtNEN(1,1,'','S','creen editing');
       WrtNEN(1,2,'','R','elocations');
       WrtNEN(1,3,'','D','isplays');
       WaitUp;
       MouseCursorOn;
 MenuMod := KeyOrMouseG(['S','R','D'],MenuModGA,OldOp,MOut);
       MouseCursorOff;
       WClear;
       Wrt(1,1,'Screen editing');
       Wrt(1,2,'Relocations');
       Wrt(1,3,'Displays');
     end{with};
    end;

   {---------------------------------------------------------}

   Procedure Generation;
   Const
            x1d = 130; y1d =  50;       { Dialog window - Wd }
            x2d = 450; y2d = 200;

   Var  Wd  : DWindowP;
        F   : Text;
        IORes : Integer;
        OK  : Boolean;
        s   : String;
        Fnm : String;
        n   : Nodepoint;
        t   : PNt;

   Begin
           New(Wd);
           With Wd^ do begin
               WOpen(x1d,y1d,x2d,y2d,cumeco,CuBkCo,cumeco,CuEmCo);
               If WOK then begin
                Wrt ( 2,2,'Generating Include Files.');
                Wrt ( 2,4,'Wait please ...');

                Assign(F,'%%S.PAS');                   { Starting }
                {$I-} Rewrite(F); {$I+}
                IORes := IOResult;
                OK := IORes=0;
                If not OK then begin
                   WClear;
                   Wrt ( 2,1,'Some troubles when creating the file');
                   WrtE( 2,3,'%%S.PAS');
                   Wrt ( 2,5,'IOResult error code: ');
                   Str(IORes:4,s);
                   WrtE(23,5,s);
                   Wrt ( 2,6,'Error message: ');
                   WrtE(17,6,ErrorMessage(IORes));
                   Wrt ( 2,8,'Press any key');
                   Wait;
                  end{then}
                Else begin
                   WriteLn(F,'Procedure Firing_Starts; {Starting snippets}');
                   n := TakeFirst(Transitions);     { All transitions }
                   While n<>nil do begin
                      t := n^.el;             { pointer to transition }
                      Str(t^.TranNum,s);                     { Number }
                      Fnm := '%%S' + s + '.PAS';
                      If not FileIsEmpty(Fnm) then
                          Writeln(F,'  {$I ' + Fnm + '   Procedure _PS',
                                    s,' }');
                      n := TakeNext(n);  { Next transition }
                   end{while};
                   WriteLn(F,'Begin');
                   WriteLn(F,' Case TNum of');
                   n := TakeFirst(Transitions);     { All transitions }
                   While n<>nil do begin
                      t := n^.el;             { pointer to transition }
                      Str(t^.TranNum,s);                     { Number }
                      WriteLn(F,' '+s+': begin {'+t^.Name+'}');
                      Fnm := '%%S' + s + '.PAS';
                      If FileIsEmpty(Fnm) then
                        Writeln(F,'     {not found}')
                      Else begin
                        Writeln(F,'     _PS' + s + ';');
                      End{If};
                      WriteLn(F,'    end;');
                      n := TakeNext(n);  { Next transition }
                   end{while};
                   WriteLn(F,' End{Case}');
                   WriteLn(F,'End;');
                   Close(F);
                End{If};

               If OK then begin
                Assign(F,'%%E.PAS');                   { Ending }
                {$I-} Rewrite(F); {$I+}
                IORes := IOResult;
                OK := IORes=0;
                If not OK then begin
                   WClear;
                   Wrt ( 2,1,'Some troubles when creating the file');
                   WrtE( 2,3,'%%E.PAS');
                   Wrt ( 2,5,'IOResult error code: ');
                   Str(IORes:4,s);
                   WrtE(23,5,s);
                   Wrt ( 2,6,'Error message: ');
                   WrtE(17,6,ErrorMessage(IORes));
                   Wrt ( 2,8,'Press any key');
                   Wait;
                  end{then}
                Else begin
                   WriteLn(F,'Procedure Firing_Ends; {Ending snippets}');
                   n := TakeFirst(Transitions);     { All transitions }
                   While n<>nil do begin
                      t := n^.el;             { pointer to transition }
                      Str(t^.TranNum,s);                     { Number }
                      Fnm := '%%E' + s + '.PAS';
                      If not FileIsEmpty(Fnm) then
                          Writeln(F,'  {$I ' + Fnm + '   Procedure _PE',
                                    s,' }');
                      n := TakeNext(n);  { Next transition }
                   end{while};
                   WriteLn(F,'Begin');
                   WriteLn(F,' Case TNum of');
                   n := TakeFirst(Transitions);     { All transitions }
                   While n<>nil do begin
                      t := n^.el;             { pointer to transition }
                      Str(t^.TranNum,s);                     { Number }
                      WriteLn(F,' '+s+': begin {'+t^.Name+'}');
                      Fnm := '%%E' + s + '.PAS';
                      If FileIsEmpty(Fnm) then
                        Writeln(F,'     {not found}')
                      Else begin
                        Writeln(F,'     _PE' + s + ';');
                      End{If};
                      WriteLn(F,'    end;');
                      n := TakeNext(n);  { Next transition }
                   end{while};
                   WriteLn(F,' End{Case}');
                   WriteLn(F,'End;');
                   Close(F);
                End{If};
               End{If};

                Wrt ( 2,8,'Press any key');
                If OK then
                   Wrt ( 2,6,'Generation completed')
                Else begin
                   Wrt ( 2,6,'Generation failed - try again');
                   Wait;
                end{else};
                WClose;
               end{WOK};
          end{with};
          Dispose(Wd);
    End{Generation};

{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

Procedure Editing;                   { Full screen editing }

Type Typtype = (Firstp,Middlep,Lastp);

Var EdFin, Refresh : Boolean;
    x1,y1,b : word;
    c,d : char;
    t : PNt;
    p : PNp;
    arc : PNa;
    a,point :Nodepoint;
    typ : Char;
    Option : char;

{----------------------------------------------------------}

    Procedure ChangePAtt(pl:PNp);

    Const
      MenuPAttGA:     { Positions and options for MenuPAtt }
      array[1..7] of PositionG =
                  (( x1:  0; y1:  0; x2:  0; y2:  0; opt:'M' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'N' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'Z' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'C' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'A' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'T' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'G' ));


      Menu1GA:        { Positions and options for Menu1 }
      array[1..3] of PositionG =
                  (( x1: 0; y1:0; x2:0; y2:0; opt:'C' ),
                   ( x1: 0; y1:0; x2:0; y2:0; opt:'H' ),
                   ( x1: 0; y1:0; x2:0; y2:0; opt:'V' ));

    Var    Wpa,Wrr  :PWindowP;
           Wpad     :DWindowP;
           p        :PNp;
           OK,given :Boolean;
           c,d      :char;
           i        :longint;
           x11,y11,y1c :Integer;
           s        :string;
           OldOp,MOut : char;

    {......................................................}

    Procedure Prepare;
    Var i: Integer;
    Begin
      With pl^ do begin
                                       { Preparing menu array }
       If XPos+160+Size > 639 then begin { Window coordinates }
          x11 := XPos-Size-127;
          If x11+160>639 then x11:=639-160;
         end
       else
          x11 := XPos+Size;
       If YPos+176+Size > 479 then begin
          y11 := YPos-Size-130;
          If y11+176>479 then y11:=479-176;
         end
       else
          y11 := YPos+Size+4;
       For i:=0 to 6 do
          With MenuPAttGA[i+1] do begin
             x1 := x11+3;   y1 := y11+8 + i*16;
             x2 := x11+124; y2 := y11+24 + i*16;
          end{with};
      end{with};
    End;

    {......................................................}

    Function MenuPAtt(OldOp,MOut:char):char;
                                   { Place attributes menu }
    begin
      With Wpa^ do begin
       WClear;
       WrtNEN (1,1,'Change ','m','arking');
       WrtNEN (1,2,'Change ','n','ame');
       WrtNEN (1,3,'Change si','z','e');
       WrtNEN (1,4,'Change ','c','olor');
       WrtNEN (1,5,'M','a','rking color');
       WrtNEN (1,6,'Place ','t','ype');
       If pl^.Timing then WrtNEN (1,7,'Timin','g',' is ON ')
                     else WrtNEN (1,7,'Timin','g',' is OFF');
       WaitUp;
       MouseCursorOn;
       MenuPAtt := KeyOrMouseG(['M','N','Z','C','A','T','G'],
                               MenuPAttGA,OldOp,MOut);
       MouseCursorOff;
       WClear;
       Wrt(1,1,'Change marking');
       Wrt(1,2,'Change name');
       Wrt(1,3,'Change size');
       Wrt(1,4,'Change color');
       Wrt(1,5,'Marking color');
       Wrt(1,6,'Place type');
       If pl^.Timing then Wrt(1,7,'Timing is ON ')
                     else Wrt(1,7,'Timing is OFF');
      end{with};
    end;

    {.................. ChangePAtt.body .....................}

    Begin
     OldOp := 'M'; MOut := KEsc;
     With pl^ do begin
      Prepare;
      New(Wpa);
      Wpa^.WOpen(x11,y11,x11+127,y11+128,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
      If Wpa^.WOK then begin
       New(Wpad);
       With Wpad^ do begin
       Repeat
          EPrompt;
          Sprompt := 'Change parameters of the place ' + Name;
          Dprompt;
          c := MenuPAtt(OldOp,MOut);
          Case c of

           'M': begin                          { Marking }
                  i := Marking;
                  WOpen(x11+3,y11+24,x11+132,y11+72,
                        CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                  If WOK then begin
                   Wrt(1,1,'Enter marking'); WrtE(1,2,':');
                   WaitUp;
                   WRI(3,2,i,OK);
                   WClose;
                   If Ok and (i>=0) then begin
                     PErase;
                     Marking := i;
                     PShow;
                   end;
                  end{WOK};
                  OldOp := 'M';
                end;

          'N': begin                             { Name }
                WOpen(x11+3,y11+40,x11+160,y11+120,
                        CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                If WOK then begin
                 Wrt(1,1,'Enter place name'); WrtE(1,3,':');
                 WaitUp;
                 WRS(3,3,s);
                 If s<>'' then begin
                  n := Takefirst(Places);
                  found := false;
                  While (n<>nil) and not found do begin
                   p := n^.el;
                   If (p^.Name = s) then found := true;
                   n := Takenext(n)
                  end{while};
                  WClear;
                  If found then begin
                    WrtNEN (1,1,'Place ',s,'');
                    Wrt (1,2,'already exists !');
                    WrtE(1,3,'Use other name');
                    Wrt (1,4,'Press any key');
                    Wait;
                    WClose
                   end{then}
                  else begin     { OK }
                    WClose;
                    PErase;
                    Name := s;
                    PShow;
                    EPrompt;
                    Sprompt := 'Change parameters of the place '
                               + Name;
                    Dprompt;
                  end{if}
                  end{then}
                 else            { s='' }
                  WClose;
                end{WOK};
                OldOp := 'N';
               end;

           'Z': begin                             { Size }
                 WOpen(x11+3,y11+56,x11+160,y11+120,
                        CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                 If WOK then begin
                  Repeat
                    Str(Size:3,s);
                    WrtNEN (1,1,'Place size =',s,'');
                    Str(DPSize:3,s);
                    WrtNEN (1,2,'Default value =',s,'');
                    i := Size;
                    Wrt(1,3,'Enter size'); WrtE(11,3,':');
                    WaitUp;
                    WRI(13,3,i,given);
                    OK := (i>0) and (i<VGAY div 2);
                    {If not OK then Write(#7);}
                    WClear;
                  Until OK or not given;
                  WClose;
                  Wpa^.WClose;
                  PErase;
                  Size := i;
                  PShow;
                  Prepare;
                  Wpa^.WOpen(x11,y11,x11+127,y11+128,
                             CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                 end{WOK};
                 OldOp := 'Z';
                end;

           'C': begin                            { Color }
                  EPrompt;
                  Sprompt := 'Change color of the place ' + Name;
                  Dprompt;
                  SetColor(Color);
                  OutTextXY(490, VGAY - 8, 'Current');
                  SetColor(DPCol);
                  OutTextXY(583, VGAY - 8, 'Default');
                  If y11+3<191 then y1c:=y11+3 else y1c:=191;
                  WaitUp;
                  i := GetColor('0',KEsc,x11+3,y1c);
                  Eprompt;
                  If i>=0 then begin
                     Color := i;
                     PShow;
                  end;
                  OldOp := 'C';
                end;

           'A': begin                    { Marking Color }
                  EPrompt;
                  Sprompt := 'Change marking color of the place '
                             + Name;
                  Dprompt;
                  SetColor(MColor);
                  OutTextXY(490, VGAY - 8, 'Current');
                  SetColor(DPMCol);
                  OutTextXY(583, VGAY - 8, 'Default');
                  If y11+3<191 then y1c:=y11+3 else y1c:=191;
                  WaitUp;
                  i := GetColor('0',KEsc,x11+3,y1c);
                  Eprompt;
                  If i>=0 then begin
                     MColor := i;
                     PShow;
                  end;
                  OldOp := 'A';
                end;

           'T': begin                                   { Type }
                 New(Wrr);
                 Wrr^.WOpen(x11+3,y11+104,x11+107,y11+168,
                             CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                 If Wrr^.WOK then begin
                  Wrr^.WrtNEN(1,1,'','C','ircle');
                  Wrr^.WrtNEN(1,2,'','H','oriz queue');
                  Wrr^.WrtNEN(1,3,'','V','ert queue');
                  With Menu1GA[1] do begin
                    x1:=x11+6; y1:=y11+112; x2:=x11+104; y2:=y11+128
                  end{with};
                  With Menu1GA[2] do begin
                    x1:=x11+6; y1:=y11+128; x2:=x11+104; y2:=y11+144
                  end{with};
                  With Menu1GA[3] do begin
                    x1:=x11+6; y1:=y11+144; x2:=x11+104; y2:=y11+160
                  end{with};
                  WaitUp;
                  MouseCursorOn;
                   d := KeyOrMouseG(['C','H','V'],Menu1GA,'C',KEsc);
                  MouseCursorOff;
                  PErase;
                  Case d of
                   'C': PType := 0;
                   'H': PType := 1;
                   'V': PType := 2;
                  end;
                  PShow;
                  Waitup;
                  Wrr^.WClose;
                 end{WOK};
                 Dispose(Wrr);
                 OldOp := 'T';
                end;

           'G': begin                                { Timing }
                 pl^.Timing := not pl^.Timing;
                 OldOp := 'G';
                end;

           End{case};
        Until c in [KEsc];
       End{with};
       Dispose(Wpad);
       Wpa^.WClose;
      end{WOK};
      Dispose(Wpa);
     End{with};
    End{ChangePAtt};

{---------------------------------------------------------------}

    Procedure ChangeTAtt(tr:PNt);

    Const
            x1d = 130; y1d =  50;        { Dialog window - Wd }
            x2d = 450; y2d = 200;

      MenuTAttGA:        { Positions and options for MenuTAtt }
      array[1..10] of PositionG =
                  (( x1:  0; y1:  0; x2:  0; y2:  0; opt:'D' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'N' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'Z' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'C' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'T' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'P' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'F' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'B' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'S' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'E' ));

      Menu1GA:        { Positions and options for Menu1 }
      array[1..2] of PositionG =
                  (( x1: 0; y1:0; x2:0; y2:0; opt:'H' ),
                   ( x1: 0; y1:0; x2:0; y2:0; opt:'V' ));

      Menu2GA:        { Positions and options for Menu2 }
      array[1..4] of PositionG =
                  (( x1: 0; y1:0; x2:0; y2:0; opt:'I' ),
                   ( x1: 0; y1:0; x2:0; y2:0; opt:'T' ),
                   ( x1: 0; y1:0; x2:0; y2:0; opt:'G' ),
                   ( x1: 0; y1:0; x2:0; y2:0; opt:'B' ));

      Menu3GA:        { Positions and options for Menu3 }
      array[1..10] of PositionG =
                  (( x1: 0; y1:0; x2:0; y2:0; opt:'A' ),
                   ( x1: 0; y1:0; x2:0; y2:0; opt:'D' ),
                   ( x1: 0; y1:0; x2:0; y2:0; opt:'C' ),
                   ( x1: 0; y1:0; x2:0; y2:0; opt:'E' ),
                   ( x1: 0; y1:0; x2:0; y2:0; opt:'R' ),
                   ( x1: 0; y1:0; x2:0; y2:0; opt:'U' ),
                   ( x1: 0; y1:0; x2:0; y2:0; opt:'T' ),
                   ( x1: 0; y1:0; x2:0; y2:0; opt:'N' ),
                   ( x1: 0; y1:0; x2:0; y2:0; opt:'S' ),
                   ( x1: 0; y1:0; x2:0; y2:0; opt:'W' ));

    Var    Wta,Wrr  :PWindowP;
           Wtad     :DWindowP;
           t        :PNt;
           OK,given :Boolean;
           c,d      :char;
           i        :longint;
           x1,y1,x11,y11,y1c,line,code :Integer;
           s,z,si,Fnm  :string;
           OldOp,MOut  : char;
           F        :text;
           IORes,dy,j :Integer;
           m        :Nodepoint;
           p        :PetriPP;
           a        :PNa;
           x,y,pr   :Real;
           b        :Word;
           ndel     :PointP;
    {........................................................}

    Procedure Prepare;
    Var i: Integer;
    Begin
      With tr^ do begin
                                      { Preparing menu array }
       If XPos+185+15 > GetMaxX then begin  { Window coordinates }
          x11 := XPos-15-143;
          If x11+185 > GetMaxX then x11:=GetMaxX-185;
         end
       else
          x11 := XPos+15;
       If YPos+176+15 > GetMaxY then begin
          y11 := YPos-15-96;
          If y11+182 > GetMaxY then y11:=GetMaxY-182;
         end
       else
          y11 := YPos+15;
       For i:=0 to 9 do
          With MenuTAttGA[i+1] do begin
             x1 := x11+3;   y1 := y11+8 + i*16;
             x2 := x11+140; y2 := y11+24 + i*16;
          end{with};
      end{with};
    End;

    {......................................................}

    Function MenuTAtt(OldOp,MOut:char):char;
                              { Transition attributes menu }
    begin
      With Wta^ do begin
       WClear;
       WrtNEN (1,1,'Change ','d','irection');
       WrtNEN (1,2,'Change ','n','ame');
       WrtNEN (1,3,'Change si','z','e');
       WrtNEN (1,4,'Change ','c','olor');
       WrtNEN (1,5,'Transition ','t','ype');
       WrtNEN (1,6,'','P','riority');
       WrtNEN (1,7,'','F','iring delay');
       If tr^.TType=3 then WrtNEN (1,8,'','B','ranching')
                      else Wrt    (1,8,'Branching');
       WrtNEN (1,9,'','S','tarting snippet');
       WrtNEN (1,10,'','E','nding snippet');
       WaitUp;
       MouseCursorOn;
       If tr^.TType=3 then
         MenuTAtt := KeyOrMouseG(['D','N','Z','C','T','F','B','S','E','P'],
                               MenuTAttGA,OldOp,MOut)
       else
         MenuTAtt := KeyOrMouseG(['D','N','Z','C','T','F','S','E','P'],
                               MenuTAttGA,OldOp,MOut);
       MouseCursorOff;
       WClear;
       Wrt(1,1,'Change direction');
       Wrt(1,2,'Change name');
       Wrt(1,3,'Change size');
       Wrt(1,4,'Change color');
       Wrt(1,5,'Transition type');
       Wrt(1,6,'Priority');
       Wrt(1,7,'Firing delay');
       Wrt(1,8,'Branching');
       Wrt(1,9,'Starting snippet');
       Wrt(1,10,'Ending snippet');
      end{with};
    end;

    {.................. ChangeTAtt.body .....................}

   Begin
    OldOp := 'D'; MOut := KEsc;
    With tr^ do begin
     Prepare;
     New(Wta);
     Wta^.WOpen(x11,y11,x11+143,y11+176,
                 CuMeCo,CuBkCo,CuMeCo,CuEmCo);
     If Wta^.WOK then begin
      New(Wtad);
      With Wtad^ do begin
       Repeat
         EPrompt;
         Sprompt := 'Change parameters of the transition '
                    + Name;
         Dprompt;
         c := MenuTAtt(OldOp,MOut);
         Case c of

           'D': begin                           { Direction }
                 New(Wrr);
                 Wrr^.WOpen(x11+3,y11+24,x11+99,y11+72,
                             CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                 If Wrr^.WOK then begin
                  Wrr^.WrtNEN(1,1,'','H','orizontal');
                  Wrr^.WrtNEN(1,2,'','V','ertical');
                  With Menu1GA[1] do begin
                    x1:=x11+6; y1:=y11+32; x2:=x11+96; y2:=y11+48
                  end{with};
                  With Menu1GA[2] do begin
                    x1:=x11+6; y1:=y11+48; x2:=x11+96; y2:=y11+64
                  end{with};
                  WaitUp;
                  MouseCursorOn;
                   d := KeyOrMouseG(['H','V'],Menu1GA,'H',KEsc);
                  MouseCursorOff;
                  TErase;
                  If      d='H' then Direct := Horiz
                  else if d='V' then Direct := Vert;
                  TShow;
                  WaitUp;
                  Wrr^.WClose;
                 end{WOK};
                 Dispose(Wrr);
                 OldOp := 'D';
                end;

          'N': begin                                { Name }
                WOpen(x11+3,y11+40,x11+185,y11+120,
                        CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                If WOK then begin
                 Wrt(1,1,'Enter transition name'); WrtE(1,3,':');
                 WaitUp;
                 WRS(3,3,s);
                 If s<>'' then begin
                  n := Takefirst(Transitions);
                  found := false;
                  While (n<>nil) and not found do begin
                   t := n^.el;
                   If (t^.Name = s) then found := true;
                   n := Takenext(n)
                  end{while};
                  WClear;
                  If found then begin
                    WrtNEN (1,1,'Transition ',s,'');
                    Wrt (1,2,'already exists !');
                    WrtE(1,3,'Use other name');
                    Wrt (1,4,'Press any key');
                    Wait;
                    WClose
                   end{then}
                  else begin     { OK }
                    WClose;
                    TErase;
                    Name := s;
                    TShow;
                    EPrompt;
                    Sprompt := 'Change parameters of the transition '
                               + Name;
                    Dprompt;
                  end{if}
                  end{then}
                 else            { s='' }
                  WClose;
                end{WOK};
                OldOp := 'N';
               end;

          'Z': begin                                { Size }
                 WOpen(x11+3,y11+56,x11+185,y11+120,
                        CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                 If WOK then begin
                  Repeat
                    Str(Size:3,s);
                    WrtNEN (1,1,'Transition size =',s,'');
                    Str(DTSize:3,s);
                    WrtNEN (1,2,'Default value =',s,'');
                    i := Size;
                    Wrt(1,3,'Enter size'); WrtE(11,3,':');
                    WaitUp;
                    WRI(13,3,i,given);
                    OK := (i>0) and (i<VGAX);
                    {If not OK then Write(#7);}
                    WClear;
                  Until OK or not given;
                  WClose;
                  TErase;
                  Size := i;
                  TShow;
                 end{WOK};
                 OldOp := 'Z';
                end;

           'C': begin                               { Color }
                  EPrompt;
                  Sprompt := 'Change color of the transition '
                             + Name;
                  Dprompt;
                  SetColor(Color);
                  OutTextXY(490, VGAY - 8, 'Current');
                  SetColor(DTCol);
                  OutTextXY(583, VGAY - 8, 'Default');
                  If y11+3<191 then y1c:=y11+3 else y1c:=191;
                  WaitUp;
                  i := GetColor('0',KEsc,x11+3,y1c);
                  Eprompt;
                  If i>=0 then begin
                     Color := i;
                     TShow;
                  end;
                  OldOp := 'C';
                end;

           'T': begin                                   { Type }
                 New(Wrr);
                 Wrr^.WOpen(x11+3,y11+88,x11+99,y11+168,
                             CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                 If Wrr^.WOK then begin
                  Wrr^.WrtNEN(1,1,'','I','mmediate');
                  Wrr^.WrtNEN(1,2,'','T','imed');
                  Wrr^.WrtNEN(1,3,'','G','enerator');
                  Wrr^.WrtNEN(1,4,'','B','ranching');
                  With Menu2GA[1] do begin
                    x1:=x11+6; y1:=y11+96; x2:=x11+96; y2:=y11+112
                  end{with};
                  With Menu2GA[2] do begin
                    x1:=x11+6; y1:=y11+112; x2:=x11+96; y2:=y11+128
                  end{with};
                  With Menu2GA[3] do begin
                    x1:=x11+6; y1:=y11+128; x2:=x11+96; y2:=y11+144
                  end{with};
                  With Menu2GA[4] do begin
                    x1:=x11+6; y1:=y11+144; x2:=x11+96; y2:=y11+160
                  end{with};
                  WaitUp;
                  MouseCursorOn;
                   d := KeyOrMouseG(['I','T','G','B'],Menu2GA,'I',KEsc);
                  MouseCursorOff;
                  TErase;
                  Case d of
                   'I': TType := 0;
                   'T': TType := 1;
                   'G': TType := 2;
                   'B': TType := 3;
                  end;
                  TShow;
                  Waitup;
                  Wrr^.WClose;
                 end{WOK};
                 Dispose(Wrr);
                 OldOp := 'T';
                end;

           'F': begin                        { Firing duration }
                 Sprompt := 'Entering firing delay of the transition '
                            + Name;
                 Dprompt;
                 New(Wrr);
                 Wrr^.WOpen(x11+3,y11+3,x11+153,y11+180,
                             CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                 If Wrr^.WOK then begin
                  Wrr^.WrtNEN(1,1,'Const','a','nt');
                  Wrr^.WrtNEN(1,2,'Table ','d','iscrete');
                  Wrr^.WrtNEN(1,3,'Table ','c','ontinuous');
                  Wrr^.WrtNEN(1,4,'','E','xponential');
                  Wrr^.WrtNEN(1,5,'E','r','lang');
                  Wrr^.WrtNEN(1,6,'','U','niform');
                  Wrr^.WrtNEN(1,7,'','T','riangular');
                  Wrr^.WrtNEN(1,8,'','N','ormal');
                  Wrr^.WrtNEN(1,9,'Normal in n ','s','igma');
                  Wrr^.WrtNEN(1,10,'Sho','w',' & seed');
                  dy := 11;
                  For i:=1 to 10 do begin
                    With Menu3GA[i] do begin
                      x1:=x11+6; y1:=y11+dy; x2:=x11+150; y2:=y11+dy+16
                    end{with};
                    dy := dy+16;
                  end;
                  WaitUp;
                  MouseCursorOn;
                   d := KeyOrMouseG(['A','D','C','E','R','U',
                                     'T','N','S','W'],Menu3GA,'A',KEsc);
                  MouseCursorOff;
                  WaitUp;
                  Wrr^.WClose;
                 end{WOK};
                 Dispose(Wrr);
                 If d<>KEsc then begin
                  Wta^.WClose;
                  { Selecting the quadrant: }
                  if tr^.XPos>320 then x1 := 20
                                  else x1 := 340;
                  if tr^.YPos>240 then y1 := 20
                                  else y1 := 240;
                  WOpen(x1,y1,x1+290,y1+220,
                        CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                  If WOK then begin
                   Sprompt := 'Entering firing delay of the transition '
                              + Name;
                   Dprompt;

                   If (tr^.RandDur.FXList<>nil) and (d<>'W') then begin
                    WrtE  (2,2,'Current table will be deleted !');
                    WrtNEN(2,4,'Continue? (','Y','/N)');
                    c := UpCase(CWait(b));
                    If c='Y' then begin
                      ndel := tr^.RandDur.FXList;
                      tr^.RandDur.FXList := nil;
                      repeat
                        Dispose(ndel);
                        ndel := ndel^.Next;
                      until ndel=nil;
                      WClear;
                    end;
                   end;
                  If (tr^.RandDur.FXList=nil) or (d='W') then
                   Case d of

                   'A': begin              { Constant duration }
                      WrtNEN (2,1,'','Constant',' delay');
                      Str(tr^.RandDur.P1:10:4,s);
                      Wrt (2,3,'Old value: ' + s);
                      Wrt (2,4,'New value:');
                      repeat
                        Waitup;
                        WRS (13,4,z);
                        Wrub(13+Length(z),4,'_');
                        If z='' then begin
                          OK := true;
                          Wrt(13,4,s);
                         end
                        else begin
                          Val(z,x,code);
                          OK := (code=0) and (x>=0);
                          If OK then
                            tr^.RandDur.P1 := x
                          else
                            Wrub(13,4,z);
                        end;
                      until OK;
                      tr^.RandDur.DistType := 0;
                     end{A};

        'D','C': With tr^.RandDur do begin       { Empirical tables }
                    WrtNEN (2,1,'','Empirical table',' distribution ('+d+')');
                    Wrt(2,2,'Value(none quits) Prob        Cdf');
                    line := 3;
                    y := 0;
                    repeat
                     Waitup;
                     repeat
                      WRS(2,line,s);
                      If s='' then code := 0
                              else Val(s,x,code);
                      if code<>0 then Wrub(2,line,s);
                      Wrub(2+Length(s),line,'_');
                     until code=0;
                     if s<>'' then begin
                      repeat
                       WRS(20,line,z);
                       Val(z,pr,code);
                       if code<>0 then Wrub(20,line,z);
                       Wrub(20+Length(z),line,'_');
                      until code=0;
                      if not FxAdd(x,pr) then begin
                        Wrub(2,line,s);
                        Wrub(20,line,z);
                       end
                      else begin      { Point OK }
                        y := y + pr;
                        Str(y:6:4,z);
                        Wrt(29,line,z);
                        Inc(line);
                        If line>11 then begin
                          WClear;
                          WrtNEN (2,1,'','Empirical table',' distribution ('+d+')');
                          Wrt(2,2,'Value(none quits) Prob        Cdf');
                          line := 3;
                        end;
                      end;
                     end;
                    until s='';
                    If y<>1.0 then
                      WrtE(1,line+1,'Warning ! Last Cdf not (exactly) 1')
                    else
                      WrtE(1,line+1,'Cdf OK');
                    Wrt (1,line+2,'Press a key to continue');
                    Wait;
                    If d='D' then DistType := 1
                             else DistType := 2;
             end{D,C};

                   'E': begin       { Exponential distribution }
                      WrtNEN (2,1,'','Exponential',' distribution');
                      Str(tr^.RandDur.P1:10:4,s);
                      Wrt (2,3,'Old mean value: ' + s);
                      Wrt (2,4,'New mean value:');
                      repeat
                        Waitup;
                        WRS (18,4,z);
                        Wrub(18+Length(z),4,'_');
                        If z='' then begin
                          OK := true;
                          Wrt(18,4,s);
                         end
                        else begin
                          Val(z,x,code);
                          OK := (code=0) and (x>=0);
                          If OK then
                            tr^.RandDur.P1 := x
                          else
                            Wrub(18,4,z);
                        end;
                      until OK;
                      tr^.RandDur.DistType := 3;
                     end{E};

                   'R': begin            { Erlang distribution }
                      WrtNEN (2,1,'','Erlang',' distribution');
                      Str(tr^.RandDur.P1:10:4,s);
                      Wrt (2,3,'Old mean value: ' + s);
                      Wrt (2,4,'New mean value:');
                      repeat
                        Waitup;
                        WRS (18,4,z);
                        Wrub(18+Length(z),4,'_');
                        If z='' then begin
                          OK := true;
                          Wrt(18,4,s);
                         end
                        else begin
                          Val(z,x,code);
                          OK := (code=0) and (x>=0);
                          If OK then
                            tr^.RandDur.P1 := x
                          else
                            Wrub(18,4,z);
                        end;
                      until OK;
                      With tr^.RandDur do If PI1=0 then PI1 := 2;
                      Str(tr^.RandDur.PI1,s);
                      Wrt (2,6,'Old shape value: ' + s);
                      Wrt (2,7,'New shape value:');
                      repeat
                        Waitup;
                        WRS (19,7,z);
                        Wrub(19+Length(z),7,'_');
                        If z='' then begin
                          OK := true;
                          Wrt(19,7,s);
                         end
                        else begin
                          Val(z,j,code);
                          OK := (code=0) and (j>0);
                          If OK then
                            tr^.RandDur.PI1 := j
                          else
                            Wrub(19,7,z);
                        end;
                      until OK;
                      tr^.RandDur.DistType := 4;
                     end{R};

                   'U': begin           { Uniform distribution }
                      WrtNEN (2,1,'','Uniform',' distribution');
                      Str(tr^.RandDur.P1:10:4,s);
                      Wrt (2,3,'Old min value: ' + s);
                      Wrt (2,4,'New min value:');
                      repeat
                        Waitup;
                        WRS (17,4,z);
                        Wrub(17+Length(z),4,'_');
                        If z='' then begin
                          OK := true;
                          Wrt(17,4,s);
                          x := tr^.RandDur.P1;
                         end
                        else begin
                          Val(z,x,code);
                          OK := (code=0) and (x>=0);
                          If OK then
                            tr^.RandDur.P1 := x
                          else
                            Wrub(17,4,z);
                        end;
                      until OK;
                      With tr^.RandDur do If P2<x then P2 := x + 1;
                      Str(tr^.RandDur.P2:10:4,s);
                      Wrt (2,6,'Old max value: ' + s);
                      Wrt (2,7,'New max value:');
                      repeat
                        Waitup;
                        WRS (17,7,z);
                        Wrub(17+Length(z),7,'_');
                        If z='' then begin
                          OK := true;
                          Wrt(17,7,s);
                         end
                        else begin
                          Val(z,y,code);
                          OK := (code=0) and (y>=x);
                          If OK then
                            tr^.RandDur.P2 := y
                          else
                            Wrub(17,7,z);
                        end;
                      until OK;
                      tr^.RandDur.DistType := 5;
                     end{U};

                   'T': begin        { Triangular distribution }
                      WrtNEN (2,1,'','Triangular',' distribution');
                      Str(tr^.RandDur.P1:10:4,s);
                      Wrt (2,3,'Old min value: ' + s);
                      Wrt (2,4,'New min value:');
                      repeat
                        Waitup;
                        WRS (17,4,z);
                        Wrub(17+Length(z),4,'_');
                        If z='' then begin
                          OK := true;
                          Wrt(17,4,s);
                          x := tr^.RandDur.P1;
                         end
                        else begin
                          Val(z,x,code);
                          OK := (code=0) and (x>=0);
                          If OK then
                            tr^.RandDur.P1 := x
                          else
                            Wrub(17,4,z);
                        end;
                      until OK;
                      With tr^.RandDur do If P2<x then P2 := x + 1;
                      Str(tr^.RandDur.P2:10:4,s);
                      Wrt (2,6,'Old likely value: ' + s);
                      Wrt (2,7,'New likely value:');
                      repeat
                        Waitup;
                        WRS (20,7,z);
                        Wrub(20+Length(z),7,'_');
                        If z='' then begin
                          OK := true;
                          Wrt(20,7,s);
                          y := tr^.RandDur.P2;
                         end
                        else begin
                          Val(z,y,code);
                          OK := (code=0) and (y>=x);
                          If OK then
                            tr^.RandDur.P2 := y
                          else
                            Wrub(20,7,z);
                        end;
                      until OK;
                      With tr^.RandDur do If P3<y then P3 := y + 1;
                      Str(tr^.RandDur.P3:10:4,s);
                      Wrt (2,9 ,'Old max value: ' + s);
                      Wrt (2,10,'New max value:');
                      repeat
                        Waitup;
                        WRS (17,10,z);
                        Wrub(17+Length(z),10,'_');
                        If z='' then begin
                          OK := true;
                          Wrt(17,10,s);
                         end
                        else begin
                          Val(z,x,code);
                          OK := (code=0) and (x>=y);
                          If OK then
                            tr^.RandDur.P3 := x
                          else
                            Wrub(17,10,z);
                        end;
                      until OK;
                      tr^.RandDur.DistType := 6;
                     end{T};

                   'N': begin           { Normal distribution }
                      WrtNEN (2,1,'','Normal',' distribution');
                      Str(tr^.RandDur.P1:10:4,s);
                      Wrt (2,3,'Old mean value: ' + s);
                      Wrt (2,4,'New mean value:');
                      repeat
                        Waitup;
                        WRS (18,4,z);
                        Wrub(18+Length(z),4,'_');
                        If z='' then begin
                          OK := true;
                          Wrt(18,4,s);
                         end
                        else begin
                          Val(z,x,code);
                          OK := (code=0) and (x>=0);
                          If OK then
                            tr^.RandDur.P1 := x
                          else
                            Wrub(18,4,z);
                        end;
                      until OK;
                      With tr^.RandDur do If P2=0 then P2 := 1;
                      Str(tr^.RandDur.P2:10:4,s);
                      Wrt (2,6,'Old StDev value: ' + s);
                      Wrt (2,7,'New StDev value:');
                      repeat
                        Waitup;
                        WRS (19,7,z);
                        Wrub(19+Length(z),7,'_');
                        If z='' then begin
                          OK := true;
                          Wrt(19,7,s);
                         end
                        else begin
                          Val(z,y,code);
                          OK := (code=0) and (y>0);
                          If OK then
                            tr^.RandDur.P2 := y
                          else
                            Wrub(19,7,z);
                        end;
                      until OK;
                      tr^.RandDur.DistType := 7;
                     end{N};

                   'S': begin     { Normal in n sigma distribution }
                      WrtNEN (2,1,'','Normal in n sigma',' distribution');
                      Str(tr^.RandDur.P1:10:4,s);
                      Wrt (2,3,'Old mean value: ' + s);
                      Wrt (2,4,'New mean value:');
                      repeat
                        Waitup;
                        WRS (18,4,z);
                        Wrub(18+Length(z),4,'_');
                        If z='' then begin
                          OK := true;
                          Wrt(18,4,s);
                         end
                        else begin
                          Val(z,x,code);
                          OK := (code=0) and (x>=0);
                          If OK then
                            tr^.RandDur.P1 := x
                          else
                            Wrub(18,4,z);
                        end;
                      until OK;
                      With tr^.RandDur do If P2=0 then P2 := 1;
                      Str(tr^.RandDur.P2:10:4,s);
                      Wrt (2,6,'Old StDev value: ' + s);
                      Wrt (2,7,'New StDev value:');
                      repeat
                        Waitup;
                        WRS (19,7,z);
                        Wrub(19+Length(z),7,'_');
                        If z='' then begin
                          OK := true;
                          Wrt(19,7,s);
                         end
                        else begin
                          Val(z,y,code);
                          OK := (code=0) and (y>0);
                          If OK then
                            tr^.RandDur.P2 := y
                          else
                            Wrub(19,7,z);
                        end;
                      until OK;
                      With tr^.RandDur do If PI1=0 then PI1 := 3;
                      Str(tr^.RandDur.PI1,s);
                      Wrt (2,9 ,'Old n value: ' + s);
                      Wrt (2,10,'New n value:');
                      repeat
                        Waitup;
                        WRS (15,10,z);
                        Wrub(15+Length(z),10,'_');
                        If z='' then begin
                          OK := true;
                          Wrt(15,10,s);
                         end
                        else begin
                          Val(z,j,code);
                          OK := (code=0) and (j>0);
                          If OK then
                            tr^.RandDur.PI1 := j
                          else
                            Wrub(15,10,z);
                        end;
                      until OK;
                      tr^.RandDur.DistType := 8;
                     end{S};

                'W': begin            { Show distribution & seed }
                   With tr^.RandDur do begin
                    Str(P1:10:4,s);
                    Str(P2:10:4,z);
                    Str(PI1:10,si);
                    Case DistType of

                    0:begin
                        WrtNEN (2,1,'','Constant',' delay');
                        Wrt (2,3,'Delay = ' + s);
                      end;

                 1,2:begin
                      If DistType=1 then c:='D'
                                    else c:='C';
                      WrtNEN (2,1,'','Empirical table',' distribution ('+c+')');
                      Wrt(2,2,'    Value        Prob        Cdf');
                      line := 3;
                      y := 0;
                      ndel := FXList;
                      while ndel<>nil do begin
                        Str(ndel^.Value:10:4,z);
                        Wrt(2,line,z);
                        Str(ndel^.Px:6:4,s);
                        Wrt(17,line,s);
                        Str(ndel^.Fx:6:4,si);
                        Wrt(28,line,si);
                        Inc(line);
                        If line>8 then begin
                          Wrt (1,line+1,'Press a key to continue');
                          Wait;
                          WClear;
                          WrtNEN (2,1,'','Empirical table',' distribution ('+c+')');
                          Wrt(2,2,'    Value        Prob        Cdf');
                          line := 3;
                        end;
                        ndel := ndel^.Next;
                      end{while};
                     end;

                    3:begin
                        WrtNEN (2,1,'','Exponential',' distribution');
                        Wrt (2,3,'Mean = ' + s);
                      end;

                    4:begin
                        WrtNEN (2,1,'','Erlang',' distribution');
                        Wrt (2,3,'Mean  = ' + s);
                        Wrt (2,5,'Shape = ' + si);
                      end;

                    5:begin
                        WrtNEN (2,1,'','Uniform',' distribution');
                        Wrt (2,3,'Min = ' + s);
                        Wrt (2,5,'Max = ' + z);
                      end;

                    6:begin
                        WrtNEN (2,1,'','Triangular',' distribution');
                        Wrt (2,3,'Min    = ' + s);
                        Wrt (2,5,'Likely = ' + z);
                        Str(P3:10:4,z);
                        Wrt (2,7,'Max    = ' + z);
                      end;

                    7:begin
                        WrtNEN (2,1,'','Normal',' distribution');
                        Wrt (2,3,'Mean  = ' + s);
                        Wrt (2,5,'StDev = ' + z);
                      end;

                    8:begin
                        WrtNEN (2,1,'','Normal in n sigma',' distribution');
                        Wrt (2,3,'Mean  = ' + s);
                        Wrt (2,5,'StDev = ' + z);
                        Wrt (2,7,'n     = ' + si);
                      end;

                    end{case};
		    Wrt (2,10,'Enter seed (zero to randomize)');
                    Str(tr^.RandDur.InitSeed,s);
                    Wrt (2,11,'Old value: ' + s);
                    Wrt (2,12,'New value:');
                    repeat
                     Waitup;
                     WRS (13,12,s);
                     Wrub(13+Length(s),12,'_');
                     If s='' then begin
                       OK := true;
                      end
                     else begin
                       Val(s,i,code);
                       OK := (code=0) and (i>=0);
                       If OK then
                         InitSeed := i
                       else
                         Wrub(13,12,s);
                     end;
                    until OK;
                   end{with};
                  end{W};
                 end{case};
                 WClose;
                end{WOK};
                Wta^.WOpen(x11,y11,x11+143,y11+176,
                              CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                end;
                OldOp := 'F';
               end{F};

           'B': begin                              { Branching }
                 OldOp := 'F';
                 if tr^.TType=3 then begin
                  Wta^.WClose;
                  { Selecting the quadrant: }
                  if tr^.XPos>320 then x1 := 20
                                  else x1 := 340;
                  if tr^.YPos>240 then y1 := 20
                                  else y1 := 240;
                  WOpen(x1,y1,x1+290,y1+220,
                        CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                  If WOK then begin
                   Sprompt := 'Editing branching probabilities of the transition '
                              + Name;
                   Dprompt;
                   m := TakeFirst(Outlist);      { Output arcs }
                   If m=nil then begin
                     WrtE(1,2,'There are no output arcs');
                     Wrt (1,3,'Press a key to continue');
                     Wait;
                    end
                   else begin
                    WrtE(1,1,'Place           Prob      New/Cdf');
                    line := 2;
                    y := 0;
                    while m<>nil do begin
                     a := m^.el;              { pointer to arc }
                     p := a^.Place;
                     Wrt(1,line,p^.Name);
                     Str(a^.BranchPro:6:4,z);
                     Wrt(17,line,z);
                     repeat
                       Waitup;
                       WRS(27,line,s);
                       Wrub(27,line,s);
                       Wrub(27+Length(s),line,'_');
                       If s='' then begin
                          OK := true;
                        end
                       else begin
                          Val(s,x,code);
                          OK := (code=0) and (x>=0) and (x<=1);
                          If OK then begin
                             Wrub(17,line,z);
                             a^.BranchPro := x;
                             Str(x:6:4,z);
                             Wrt(17,line,z);
                          end;
                       end;
                     until OK;
                     y := y + a^.BranchPro;
                     a^.BranchCdf := y;
                     Str(y:6:4,s);
                     Wrt(27,line,s);
                     Inc(line);
                     If line>11 then begin
                       WClear;
                       WrtE(1,1,'Place           Prob      New/Cdf');
                       line := 2;
                     end;
                     m := TakeNext(m);
                    end{while};
                    If y<>1.0 then
                     If y=0 then
                      WrtE(1,line+1,'All zeros = Uniform distribution')
                     else
                      WrtE(1,line+1,'Warning ! Last Cdf not (exactly) 1')
                    else
                     WrtE(1,line+1,'Cdf OK');
                    Wrt (1,line+2,'Press a key to continue');
                    Wait;
                   end{if m=nil};
                   WClose;
                  end{WOK};
                  OldOp := c;
                  Wta^.WOpen(x11,y11,x11+143,y11+176,
                             CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                 end;
               end;

           'S','E': begin                               { Editing }
              Str(TranNum,s);
              Case c of
               'S': Fnm := '%%S'+s+'.PAS';     { Starting snippet }
               'E': Fnm := '%%E'+s+'.PAS';       { Ending snippet }
              end;
              OK := not FileIsEmpty(Fnm);
              If not OK then begin
               New(Wd);
               With Wd^ do begin
                WOpen(x1d,y1d,x2d,y2d,cumeco,CuBkCo,cumeco,CuEmCo);
                If WOK then begin
                 Case c of
                 'S': Wrt ( 2,1,'The starting snippet file '+Fnm);
                 'E': Wrt ( 2,1,'The ending snippet file '+Fnm);
                 end;
                 WrtNEN ( 2,3,'of the transition ',Name,'');
                 Wrt    ( 2,5,'so far does not exist !');
                 WrtNEN ( 2,7,'','C','reate the new file');
                 WrtNEN ( 2,8,'','Q','uit (if you are not sure)');
                 Repeat d:=UpCase(ReadK) Until d in ['C','Q',KEsc];
                 If d='C'then begin
                   Assign(F,Fnm);
                   {$I-} Rewrite(F); {$I+}
                   IORes := IOResult;
                   If IORes>0 then begin
                     WClear;
                     Wrt ( 2,1,'Some troubles when creating the file');
                     WrtE( 2,3,Fnm);
                     Wrt ( 2,5,'IOResult error code: ');
                     Str(IORes:4,s);
                     WrtE(23,5,s);
                     Wrt ( 2,6,'Error message: ');
                     WrtE(17,6,ErrorMessage(IORes));
                     Wrt ( 2,8,'Press any key');
                     Wait;
                    end{then}
                   Else begin
                     OK := true;
                     Case c of
                     'S': WriteLn(F,'Procedure _PS'+s+'; '+
                                    '{Starting snippet of the transition t'
                                    + s +' (' + Name + ')}');
                     'E': WriteLn(F,'Procedure _PE'+s+'; '+
                                    '{Ending snippet of the transition t'
                                    + s +' (' + Name + ')}');
                     end;
                     WriteLn(F,'Begin');
                     WriteLn(F,'End;');
                     Close(F);
                     Generation;
                   End{If};
                 End{If};
                 WClose;
                end{WOK};
               end{with};
               Dispose(Wd);
              end{If};
              If OK then begin
                 Wta^.WClose;
                 RestoreCrtMode;
                 Exec(EditorName,Fnm);           { Editing }
                 SetGraphMode(GrMode);
                 OK := MouseInit;
                 Snet^.Show;
                 Wta^.WOpen(x11,y11,x11+143,y11+176,
                            CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                 Wx (1,1,'Screen editing');
               end;
               OldOp := c;
             End{S&E};

           'P': begin                           { Priority }
                 WOpen(x11+3,y11+104,x11+185,y11+168,
                        CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                 If WOK then begin
                  Repeat
                    Str(Priority:4,s);
                    WrtNEN (1,1,'Priority = ',s,'');
                    Str(100:3,s);
                    WrtNEN (1,2,'Default value = ',s,'');
                    i := Size;
                    Wrt(1,3,'Enter priority'); WrtE(15,3,':');
                    WaitUp;
                    WRI(17,3,i,given);
                    OK := (i>=0) and (i<65536);
                    {If not OK then Write(#7);}
                    WClear;
                  Until OK or not given;
                  WClose;
                  If OK and given then Priority := i;
                 end{WOK};
                 OldOp := 'P';
                end{P};

          End{case};
        Until c in [KEsc];
       End{with};
       Dispose(Wtad);
       Wta^.WClose;
      end{WOK};
      Dispose(Wta);
     end{with tr};
    End{ChangeTAtt};

{---------------------------------------------------------------}

    Procedure Aditions;

    Var    p     :PetriPP;
           t     :PNt;
           arc   :PNa;
           n,m   :Nodepoint;
           b,x1,y1 :word;
           c,d   :char;
           s     :string;
           OK,found   :Boolean;

    {---------------------- Aditions.body -----------------------}

    Begin
     Case Option of

     'A': begin                         { Add new arc }
           Eprompt;
           SPrompt := 'Create a new arc:   ' +
                      'Select the arc origin (place or transition)';
           DPrompt;
           d := KNull;
           MouseCursorOn;
           Repeat
             Repeat                     { Waiting for mouse button }
              b := MousePosition(x1,y1)
             Until b>0;
             WaitUp;
             If b=1 then begin                         { b=2 = Esc }
               t := TestTrans;         { Searching for transitions }
               If t<>nil then      { Transition found = OUTPUT ARC }
                 d := 'O'

               Else begin  { Searching for places: (no transition) }
                p := TestPlace;
                If p<>nil then
                    d := 'I';
               End{if};
             End{if b=1 };
           Until (b=2) or (d<>KNull);
           EPrompt;
           OK := false;

           If d='I' then begin                   { Maybe Input arc }
             SPrompt := 'Creating an input arc from ' +
                        p^.Name + ':   Select the target transition';
             DPrompt;
             Wx(19,1,p^.Name + ' -> ');
             Repeat
              Repeat                    { Waiting for mouse button }
                 b := MousePosition(x1,y1)
              Until b>0;
              WaitUp;
              t := nil;
              If b=1 then                              { b=2 = Esc }
                 t := TestTrans;        { Searching for transition }
             Until (b=2) or (t<>nil);
             EPrompt;

             If t<>nil then begin                   { INPUT ARC OK }
              SPrompt := 'Arc from ' + p^.Name + ' to '
                         + t^.Name + ':    ' +
                         'Left=Drawing  Right=Invisible';
              DPrompt;
              Wx(19,1,p^.Name + ' -> ' + t^.Name);
              OK := true;
             end{OK}
            end{input place}

           Else If d='O' then begin             { Maybe Output arc }
             SPrompt := 'Creating an output arc from ' +
                        t^.Name + ':   Select the target place';
             DPrompt;
             Wx(19,1,t^.Name + ' -> ');
             Repeat
              Repeat                    { Waiting for mouse button }
                 b := MousePosition(x1,y1)
              Until b>0;
              WaitUp;
              p := nil;
              If b=1 then                              { b=2 = Esc }
                p := TestPlace;              { Searching for place }
             Until (b=2) or (p<>nil);
             EPrompt;

             If p<>nil then begin                  { OUTPUT ARC OK }
              SPrompt := 'Arc from ' + t^.Name + ' to '
                         + p^.Name + ':    ' +
                         'Left=Drawing  Right=Invisible';
              DPrompt;
              Wx(19,1,t^.Name + ' -> ' + p^.Name);
              OK := true;
             end{OK}
           End{output place};

           If OK then begin                      { CREATING AN ARC }
                                       { Test if it already exists }
            If d='I' then n := TakeFirst(t^.InList)
                     else n := TakeFirst(t^.OutList);
            found := false;
            While (n<>nil) and not found do begin
               arc := n^.el;
               If (arc^.Place = p) then begin
                  found := true; m := n;
               end{if};
               n := Takenext(n)
            end{while};

            If not found then begin  { No = creating new empty arc }
                    new(arc);
                    new(m);
                    m^.el := arc;
                    If d='I' then begin
                          InsLast(m,t^.Inlist);
                          arc^.AInit(IACol,AWidth,SolidLn,1,0);
                        end{then}
                    else begin 
                          InsLast(m,t^.Outlist);
                          arc^.AInit(OACol,AWidth,SolidLn,1,-1);
                    end{if};
                    arc^.Place := p;
                    arc^.Transition := t;
                  end{then}
            Else begin                 { Yes = destroying line }
                  MouseCursorOff;
                  arc^.AErase;
                  MouseCursorOn;
                  arc^.Destroy;
            End{if};
            { Now there is an arc withouth corners }
            Repeat                     { Waiting for mouse button }
               b := MousePosition(x1,y1)
            Until b>0;

            If b=1 then begin    { b=1 - drawing, b=2 - invisible }
               EPrompt;
               SPrompt := 'Draw an arc:   '
                          +'Left button to fix points,'
                          + 'Right button for last one,'
                          + 'H/V to adjust';
               DPrompt;
               If d='I' then
                  SetMousePos(p^.Xpos,p^.Ypos)
               else
                  SetMousePos(t^.Xpos,t^.Ypos);
               WaitUp;
               arc^.Create;           { Creating new corners }
               arc^.AShow;
               If MouseOK then  { Restoring default cursor shape }
                If MouseInit then begin
                   SetMouseXRange(0,637);
                   SetMouseYRange(0,477);
                end;
               EPrompt;
            end{drawing};

           End{OK-creating arc};
          End{ Add new arc };

     'P': begin                       { Add new place }
           Inc(Nopl);
           New(p);
           Str(Nopl,s);
           s := 'p' + s;
           p^.PInit(268,45,DPsize,DPcol,DPMCol,0,s,0);
           New(n);
           n^.el := p;
           Inslast(n,Places);
           p^.PShow;
          end{ Add new place };

     'T': begin                   { Add new transition }
           Inc(NotrG);
           Inc(Notr);
           New(t);
           Str(NotrG,s);
           s := 't' + s;
           t^.TInit(344,45,DTsize,DTcol,Vert,s,NotrG,0);
           New(n);
           n^.el := t;
           Inslast(n,Transitions);
           t^.TShow;
          end{ Add new transition };

         end{case};
    End{Aditions};

{------------------------------------------------------------}

    Procedure EditArc(a:Nodepoint; typ:char; point:Nodepoint);
    { Editing arc - left pressed }

    Var
         b,x,y  :word;
         arc  :PNa;
         p,np :PPnt;
         n    :Nodepoint;

    Function CtrlPressed: Boolean;
(* Bit number:
 0 Right Shift  (  1),
 1 Left Shift   (  2),
 2 Ctrl         (  4),
 3 Alt          (  8),
 4 Scroll Lock  ( 16),
 5 Num Lock     ( 32),
 6 Caps Lock    ( 64),
 7 Insert       (128) *)

    Var Keys : byte absolute $40:$17;  (* Current Shift Status *)
    Begin
       CtrlPressed := (Keys and 4) <> 0
    End;

    Begin
     MouseCursorOn;
     b := MousePosition(x,y);
     arc := a^.el;
     p := point^.el;
     If CtrlPressed and (typ='P') then begin { Deleting point }
       MouseCursorOff;
       arc^.AErase;
       Dispose(p);
       Remove(point,arc^.Corners);
       Dispose(point);
       arc^.AShow;
       MouseCursorOn;
      end{deleting}

     Else begin               { Editing arc }
      If typ='L' then begin   { Creating new point }
       MouseCursorOff;
       arc^.AErase;
       MouseCursorOn;
       New(np);
       np^.Xp := x;  np^.Yp := y;
       New(n);
       n^.el := np;
       InsAfter(n,point,arc^.Corners);
       p := np;
      end{if};
      While     MouseOK and (MousePosition(x,y)<>0) or
           not MouseOK and (MousePosition(x,y)<>2) do begin
        MouseCursorOff;
        arc^.AErase;
        p^.Xp := x;
        p^.Yp := y;
        arc^.AShow;
        MouseCursorOn;
        Delay(CurDelay);
      end{while};
     end{editing};
    End;

{------------------------------------------------------------}

    Procedure ChangeAAtt(a:Nodepoint; t:PNt; direct:char);

    Const
      MenuAPGA:      { Positions and options for MenuAP }
      array[1..7] of PositionG =
                  (( x1:  0; y1:  0; x2:  0; y2:  0; opt:'C' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'D' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'S' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'W' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'T' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'R' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'H' ));


      Menu2GA:        { Positions and options for Menu3 }
      array[1..2] of PositionG =
                  (( x1:  0; y1:  0; x2:  0; y2:  0; opt:'1' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'3' ));

      Menu4GA:        { Positions and options for Menu3 }
      array[1..4] of PositionG =
                  (( x1:  0; y1:  0; x2:  0; y2:  0; opt:'0' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'1' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'2' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'3' ));

      Menu5GA:        { Positions and options for Menu4 }
      array[1..3] of PositionG =
                  (( x1:  0; y1:  0; x2:  0; y2:  0; opt:'N' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'I' ),
                   ( x1:  0; y1:  0; x2:  0; y2:  0; opt:'T' ));

    Var  Wa,Wws   :PWindowP;
         Wad      :DWindowP;
         arc      :PNa;
         c,d      :char;
         b,x,y    :word;
         i        :longint;
         y1c      :Integer;
         OK,given :Boolean;
         s        :string;
         OldOp,MOut : char;

    {......................................................}

    Function MenuAP(OldOp,MOut:char):char;  { ArcParam menu }
    var Options: set of char;
    begin
      With Wa^ do begin
       WClear;
       WrtNEN (1,1,'','C','olor');
       WrtNEN (1,2,'Wi','d','th');
       WrtNEN (1,3,'','S','tyle');
       WrtNEN (1,4,'','W','eight');
       If arc^.AType=-1 then Wrt (1,5,'Type')
                        else WrtNEN(1,5,'','T','ype');
       WrtNEN (1,6,'','R','emove');
       WrtNEN (1,7,'','H','ide');
       Options := ['C','D','S','W','T','R','H'];
       If arc^.AType=-1 then Options := Options - ['T'];
       WaitUp;
       MouseCursorOn;
       MenuAP := KeyOrMouseG(Options,MenuAPGA,OldOp,MOut);
       MouseCursorOff;
       WClear;
       Wrt(1,1,'Color');
       Wrt(1,2,'Width');
       Wrt(1,3,'Style');
       Wrt(1,4,'Weight');
       Wrt(1,5,'Type');
       Wrt(1,6,'Remove');
       Wrt(1,7,'Hide');
      end{with};
    end;

    {.......................................................}

    Procedure Prepare;
    Var i: Integer;
    Begin
                                     { Preparing menu array }
       If X+178 > 639 then             { Window coordinates }
          X := 639-178-10;
       If Y+154 > 479 then
          Y := 479-154-10;
       For i:=0 to 6 do
          With MenuAPGA[i+1] do begin
             x1 := x+3;   y1 := y+8 + i*16;
             x2 := x+61;  y2 := y+24 + i*16;
          end{with};
       If arc^.AType=-1 then
          With MenuAPGA[5] do begin
             x1 := 0; y1 := 0; x2 := 0; y2 := 0;
          end{with};
    End;

    {.................. ChangeAAtt.body ....................}

    Begin
      OldOp := 'C'; MOut := KEsc;
      b := MousePosition(x,y);
      arc := a^.el;
      With arc^ do begin
       Prepare;
       New(Wa);
       Wa^.WOpen(x,y,x+64,y+128,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
       If Wa^.WOK then begin
        Repeat
          EPrompt;
          Sprompt := 'Change parameters of the arc  ' + ArcName;
          Dprompt;
          c := MenuAP(OldOp,MOut);
          Case c of

           'C': begin                            { Color }
                  EPrompt;
                  Sprompt := 'Change color of the arc ' + ArcName;
                  Dprompt;
                  SetColor(Color);
                  OutTextXY(490, VGAY - 8, 'Current');
                  If direct='I' then SetColor(IACol)
                                else SetColor(OACol);
                  OutTextXY(583, VGAY - 8, 'Default');
                  If y+3<191 then y1c:=y+3 else y1c:=191;
                  WaitUp;
                  i := GetColor('0',KEsc,x+3,y1c);
                  If i>=0 then begin
                     Wa^.WClose;
                     AErase;
                     Color := i;
                     AShow;
                     Wa^.WOpen(x,y,x+64,y+128,
                               CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                  end;
                  OldOp := 'C';
                end{C};

           'D': begin                           { Width }
                 EPrompt;
                 Sprompt := 'Change width of the arc ' + ArcName;
                 Dprompt;
                 New(Wws);
                 With Wws^ do begin
                  WOpen(x+3,y+40,x+90,y+88,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                  If WOK then begin
                   WrtE(1,1,'1');
                   WrtE(1,2,'3');
                   With Menu2GA[1] do begin
                     x1:=x+6; y1:=y+48; x2:=x+87; y2:=y+64
                   end{with};
                   SetLineStyle(Style,0,NormWidth);
                   Line(x+23,y+56,x+80,y+56);
                   With Menu2GA[2] do begin
                     x1:=x+6; y1:=y+64; x2:=x+87; y2:=y+80
                   end{with};
                   SetLineStyle(Style,0,ThickWidth);
                   Line(x+23,y+72,x+80,y+72);
                   WaitUp;
                   MouseCursorOn;
                    d := KeyOrMouseG(['1','3'],Menu2GA,'1',KEsc);
                   MouseCursorOff;
                   WClose;
                   If d in ['1','3'] then begin
                     Wa^.WClose;
                     AErase;
                     If d='1' then Width := 1
                              else Width := 3;
                     AShow;
                     Wa^.WOpen(x,y,x+64,y+128,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                   end;
                  end{WOK};
                 end{with};
                 Dispose(Wws);
                 OldOp := 'D';
                end{D};

           'S': begin                           { Style }
                 EPrompt;
                 Sprompt := 'Change style of the arc ' + ArcName;
                 Dprompt;
                 New(Wws);
                 With Wws^ do begin
                  WOpen(x+3,y+56,x+90,y+136,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                  If WOK then begin
                   WrtE(1,1,'0');
                   WrtE(1,2,'1');
                   WrtE(1,3,'2');
                   WrtE(1,4,'3');
                   For i:=0 to 3 do
                    With Menu4GA[i+1] do begin
                     x1:=x+6; y1:=y+64+16*i; x2:=x+87; y2:=y+80+16*i;
                     SetLineStyle(i,0,Width);
                     Line(x+23,y+72+16*i,x+80,y+72+16*i);
                    end{with};
                   WaitUp;
                   MouseCursorOn;
                    d := KeyOrMouseG(['0','1','2','3'],Menu4GA,'0',KEsc);
                   MouseCursorOff;
                   WClose;
                   If d in ['0','1','2','3'] then begin
                     Wa^.WClose;
                     AErase;
                     Style := Ord(d) - Ord('0');
                     AShow;
                     Wa^.WOpen(x,y,x+64,y+128,
                               CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                   end;
                  end{WOK};
                 end{with};
                 Dispose(Wws);
                 OldOp := 'S';
                end{S};

           'W': begin                                 { Weight }
                 EPrompt;
                 Sprompt := 'Change weight of the arc ' + ArcName;
                 Dprompt;
                 New(Wd);
                 With Wd^ do begin
                  WOpen(x+3,y+72,x+178,y+154,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                  If WOK then begin
                   Repeat
                    i := Weight;
                    Str(Weight:4,s);
                    WrtNEN (1,1,'Arc weight =',s,'');
                    Wrt (1,3,'Enter the new weight'); WrtE(1,4,':');
                    WaitUp;
                    WRI(3,4,i,given);
                    OK := (i>0);
                    WClear;
                   Until OK or not given;
                   WClose;
                   Wa^.WClose;
                   AErase;
                   Weight := i;
                   AShow;
                   Wa^.WOpen(x,y,x+64,y+128,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                  end{WOK};
                 end{with};
                 Dispose(Wd);
                 OldOp := 'W';
                end{W};

           'T': begin
                 EPrompt;
                 Sprompt := 'Change type of the arc ' + ArcName;
                 Dprompt;
                 New(Wws);
                 With Wws^ do begin
                  WOpen(x+3,y+88,x+90,y+152,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                  If WOK then begin
                   WrtNEN (1,1,'','N','ormal');
                   WrtNEN (1,2,'','I','nhibitor');
                   WrtNEN (1,3,'','T','esting');
                   For i:=0 to 2 do
                    With Menu5GA[i+1] do begin
                     x1:=x+6; y1:=y+96+16*i; x2:=x+87; y2:=y+112+16*i;
                    end{with};
                   WaitUp;
                   MouseCursorOn;
                    d := KeyOrMouseG(['N','I','T'],Menu5GA,'N',KEsc);
                   MouseCursorOff;
                   WClose;
                   If d in ['N','I','T'] then begin
                     Wa^.WClose;
                     AErase;
                     Case d of
                      'N': AType := 0;
                      'I': AType := 1;
                      'T': AType := 2;
                     end{case};
                     AShow;
                     Wa^.WOpen(x,y,x+64,y+128,
                               CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                   end;
                  end{WOK};
                 end{with};
                 Dispose(Wws);
                 OldOp := 'T';
                end{T};

           'R': begin                        { Remove }
                  Wa^.WClose;
                  AErase;
                  Destroy;
                  Dispose(arc);
                  If direct='I' then Remove(a,t^.Inlist)
                                else Remove(a,t^.Outlist);
                  Dispose(a);
                end{R};

           'H': begin                          { Hide }
                  Wa^.WClose;
                  AErase;
                  Destroy;
                  OldOp := 'H';
                end{H};

       end{case};
      until c in ['R','H',KEsc];
      If c=KEsc then Wa^.WClose;
     end{WOK};
    end{with arc};
    Dispose(Wa);
   End{ArcParam};

   {---------------------------------------------------------}

    Procedure TestHots(Var Option: char);
    { Returns the option selected:
      KNull No                'T'   new transition
      'A'   new arc           KF1   F1
      'P'   new place         KF10  F10 }

    Begin
         Option := KNull;

         If (y1<16) and (x1>199) and (x1<241) then begin  { new arc }
            SetColor(CuMeCo);
            Rectangle(200,0,240,15);
            Option := 'A';
           end
         else begin
            SetColor(Black);
            Rectangle(200,0,240,15);
         end{if};

         If (y1<16) and (x1>240) and (x1<297) then begin  { new place }
            SetColor(CuMeCo);
            Rectangle(241,0,296,15);
            Option := 'P';
           end
         else begin
            SetColor(Black);
            Rectangle(241,0,296,15);
         end{if};

         If (y1<16) and (x1>296) and (x1<392) then begin  { new trans }
            SetColor(CuMeCo);
            Rectangle(297,0,391,15);
            Option := 'T';
           end
         else begin
            SetColor(Black);
            Rectangle(297,0,391,15);
         end{if};

         If (y1<16) and (x1>392) and (x1<578) then begin  { new trans }
            SetColor(CuMeCo);
            Rectangle(392,0,577,15);
            Option := 'N';
           end
         else begin
            SetColor(Black);
            Rectangle(392,0,577,15);
         end{if};

         If (y1<16) and (x1>577) and (x1<603) then begin  { F1 }
            SetColor(CuMeCo);
            Rectangle(578,0,602,15);
            Option := KF1;
           end
         else begin
            SetColor(Black);
            Rectangle(578,0,602,15);
         end{if};

         If (y1<16) and (x1>602) then begin      { F10/Esc }
            SetColor(CuMeCo);
            Rectangle(603,0,637,15);
            If MouseOk then Option := KEsc
                       else Option := KF10;
           end
         else begin
            SetColor(Black);
            Rectangle(603,0,637,15);
         end{if};
    End;

    {------------------------------------------------------------}

    Procedure Globals;              { Global work on USER.PAS }
    Const
            x1 =  397;   x2 =  553;             { Window - Wg }
            y1 =   23;   y2 =  71;

    Var Wg  : PWindowP;
        MOpt,OldOp,MOut : char;
        Fnm : String;
        F   : Text;
        c   : Byte;
        OK  : Boolean;

    {---------------------------------------------------------}

    Function GMenu(OldOp,MOut:char):char;
                                              { Globals menu }
     Const
     GMenuGA:      { Positions and options: }
     array[1..2] of PositionG =
                 (( x1:400; y1: 31; x2:550; y2: 47; opt:'E' ),
                  ( x1:400; y1: 47; x2:550; y2: 63; opt:'H' ));
    Begin
    With Wg^ do begin
     WrtNEN(1,1,'','E','dit USER.PAS');
     WrtNEN(1,2,'','H','elp on user model');
     WaitUp;
     MouseCursorOn;
     GMenu := KeyOrMouseG(['E','H'],
                              GMenuGA,OldOp,MOut);
     MouseCursorOff;
    end{with};
    End{GMenu};

    {------------------ Globals.body -------------------------}

   Begin
     SetColor(Black);
     Rectangle(392,0,577,15);
     OldOp := 'E';
     MOut := KEsc;
     New(Wg);
     Repeat
      Wg^.WOpen(x1,y1,x2,y2,cumeco,CuBkCo,cumeco,CuEmCo);
      If Wg^.WOK then begin
       MOpt := GMenu(OldOp,MOut);
       Wg^.WClose;
       Case MOpt of

       'E': begin                   { Editing USER.PAS }
          Fnm := 'USER.PAS';
          If FileIsEmpty(Fnm) then begin
              New(Wd);
              With Wd^ do begin
                WOpen(x1d,y1d,x2d,y2d,cumeco,CuBkCo,cumeco,CuEmCo);
                If WOK then begin
                 WrtNEN ( 2,1,'The file ',Fnm,' not found !');
                 Wrt ( 2,2,'Create the generic one from the file');
                 WrtE( 2,4,'USERGEN.PAS');            { Globals }
                 Wrt ( 2,6,'Or use this file from another model.');
                 Wrt ( 2,8,'Press any key');
                 Wait;
                 WClose;
                end{WOK};
              end{with};
              Dispose(Wd);
            end{then}
          Else begin
                 RestoreCrtMode;
                 Exec(EditorName,'USER.PAS');              { Globals }
                 SetGraphMode(GrMode);
                 OK := MouseInit;
                 Snet^.Show;
                 Wx (1,1,'Screen editing');
              End;
              OldOp := 'E';
            end;

       'H': begin                          { Help }
              GProgHelp;
              OldOp := 'H';
            end{H};

        end{case};
       end{then WOK}
      Else MOpt:=KEsc;
     Until MOpt in [KEsc];
     Dispose(Wg);
   End{Globals};

{---------------------- Editing.body -------------------------}

Begin
   EdFin := false;
   Refresh := false;
   EditingTF := true;
   Repeat
     If Refresh then begin
        ClearDevice;
        SNet^.Show;
       end
     else begin
        SetFillStyle(SolidFill,Black);
        Bar(0,0,400,16);
        Wex(56,1,'n');
     end;
     Wx (1,1,'Screen editing');
     Wx (22,1,'New:');
     Wex(27,1,'a'); Wx(28,1,'rc');
     Wex(32,1,'p'); Wx(33,1,'lace');
     Wex(39,1,'t'); Wx(40,1,'ransition');
     If MouseOk then Wex(74,1,'F1 Esc')
                else Wex(74,1,'F1 F10');
     MouseCursorOn;

     If MouseOK then begin          { Testing mouse or key }
       Repeat                                   { Mouse OK }
         b := MousePosition(x1,y1);
         TestHots(Option);
       Until KeyPressed or (b>0);
       MouseCursorOff;
       Wx(56,1,'n');
       MouseCursorOn;
       If KeyPressed then c := UpCase(ReadK);
      end

     Else begin                               { No mouse ! }
       Repeat until Keypressed;
       c := UpCase(ReadK);
       b := 0;
       If not (c in [KF1,KF10,'P','T','A','N']) then
          Repeat
             b := MousePosition(x1,y1);
             TestHots(Option);
          Until b>0
     end;

     MouseCursorOff;                { Cursor,new,F1,F10 off }
     SetFillStyle(SolidFill,Black);
     Bar(120,0,400,16);
     Bar(577,0,639,16);

     If b>0 then           { Mouse button (also simulated ) }
       Case Option of

       KF10 : EdFin := Not MouseOK and (b=1);         { F10 }

       KEsc : EdFin := MouseOK and (b=1);             { Esc }

       KF1 : If b=1 then GEditHelp;                    { F1 }

       'A','P','T' : If b=1 then begin        { New objects }
                Aditions;
                Refresh := true;
                WaitUp;
              end;

       'N' : Globals;         { Work on global USER program }

       KNull : begin             { No Option=Some selection }

   { Searching for arcs: }

    a := TestArc(typ,point,t,d);         { Searching for arc: }
    If a<>nil then begin                          { Arc found }
       If b=1 then begin                 { Left = editing arc }
          EditArc(a,typ,point);
          Refresh := true;
         end
       Else begin                    { Right = arc parameters }
          ChangeAAtt(a,t,d);
          Refresh := true;
         end
     end{then-arc}

    Else begin          { Searching for transitions: (no arc) }

      t := TestTrans;
      If t<>nil then                       { Transition found }
         If b=1 then begin         { Left = moving transition }
            t^.TLocate;
            Refresh := true;
           end
         Else begin           { Right = transition parameters }
            ChangeTAtt(t);
            Refresh := true;
         end

      Else begin      { Searching for places: (no transition) }

       p := TestPlace;
       If p<>nil then                           { Place found }
         If b=1 then begin              { Left = moving place }
            p^.PLocate;
            Refresh := true;
           end
         Else begin                { Right = place parameters }
            ChangePAtt(p);
            Refresh := true;
         end

       end{searching places}
      end{searching transitions}
     end{0}
    end{case}
   {end-if-mouse}

     Else begin                         { Key pressed }
      EdFin := Not MouseOK and (c=KF10)         { F10 }
                or MouseOK and (c=KEsc);        { Esc }
      If c=KF1 then GEditHelp;
      If c in ['A','P','T'] then begin
         Option := c;
         Aditions;
         Refresh := true;
      end{if};
      If c = 'N' then Globals;
    end{if};
   Until EdFin;
   {Generation;}
   SetFillStyle(SolidFill,Black);
   Bar(0,0,400,16);
   Bar(577,0,639,16);
   MainTitles(false);
   EditingTF := false;
End{Editing};

{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

    Procedure Locations;

    Const  x1 =    3;  x2 =  173;  { Locations window - Wl }
           y1 =   63;  y2 =  111;

    Var    Wl        :PWindowP;
           num,xp,yp :Integer;
           s         :string;
           OldOp,MOut : char;
    {------------------------------------------------------}

    Function MenuLoc(OldOp,MOut:char):char;
                                           { Location menu }
    Const
      MenuLocGA:    { Positions and options for MenuLoc }
      array[1..2] of PositionG =
                  (( x1:  6; y1: 71; x2:170; y2: 87; opt:'P' ),
                   ( x1:  6; y1: 87; x2:170; y2:103; opt:'T' ));
    begin
      With Wl^ do begin
       WClear;
       WrtNEN (1,1,'Relocate ','p','lace');
       WrtNEN (1,2,'Relocate ','t','ransition');
       WaitUp;
       MouseCursorOn;
        MenuLoc := KeyOrMouseG(['P','T'],MenuLocGA,OldOp,MOut);
       MouseCursorOff;
       WClear;
       Wrt(1,1,'Relocate place');
       Wrt(1,2,'Relocate transition');
      end{with};
    end;

    {------------------- Locations.body ----------------------}

    Begin
     OldOp := 'P'; MOut := KEsc;
     New(Wl);
     Wl^.WOpen(x1,y1,x2,y2,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
     If Wl^.WOK then begin
      Repeat
       c := MenuLoc(OldOp,MOut);
       Case c of

   'P': begin                              { Locate place }
         New(Wd);
         With Wd^ do begin
          WOpen(6,87,200,164,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
          If WOK then begin
           Wrt (1,1,'Enter place name'); WrtE(1,2,':');
           WaitUp;
           WRS(3,2,s);
           s := Copy(s,1,15);
           If s='' then begin
            { Write(#7); }
            WClose
           end{then}
           else begin
            n := Takefirst(Places);
            found := false;
            While (n<>nil) and not found do begin
             pl := n^.el;
             If (pl^.Name = s) then found := true;
             n := Takenext(n)
            end{while};
            If not found then begin
             WClear;
             WrtNEN(1,1,'Place ',s,'');
             WrtE(1,2,'has not been found !');
             Wrt (1,4,'Press any key');
             Wait;
             WClose;
            end{then}
           else begin              { OK }
            With pl^ do begin
             WClose;               { Closing dialog window }
             SPrompt := 'Locate the place ' + pl^.Name +
                        '  (Find position and press any mouse button).';
             DPrompt;
             P.SetPos;
             EPrompt;
             Wl^.WClose;
             If XPos>0 then PErase;         { Deleting old }
             Xpos := p.Xp; Ypos := p.Yp;
             PShow;
             Wl^.WOpen(x1,y1,x2,y2,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
            end{with};
            end{if};
           end{if};
          end{WOK};
         end{with};
         Dispose(Wd);
         OldOp := 'P';
       end{ Locate place };

   'T': begin                         { Locate transition }
         New(Wd);
         With Wd^ do begin
          WOpen(6,103,240,180,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
          If WOK then begin
           Wrt (1,1,'Enter transition name');  WrtE(1,2,':');
           WaitUp;
           WRS(3,2,s);
           s := Copy(s,1,15);
           If s='' then begin
            { Write(#7); }
            WClose
           end{then}
           else begin
            n := Takefirst( Transitions);
            found := false;
            While (n<>nil) and not found do begin
             tr := n^.el;
             If (tr^.Name = s) then found := true;
             n := Takenext(n)
            end{while};
            If not found then begin
             WClear;
             WrtNEN(1,1,'Transition ',s,'');
             WrtE(1,2,'has not been found !');
             Wrt (1,4,'Press any key');
             Wait;
             WClose;
            end{then}
           else begin              { OK }
            With tr^ do begin
             WClose;               { Closing dialog window }
             SPrompt := 'Locate the transition ' + tr^.Name +
             '  (Find position and press any mouse button).';
             DPrompt;
             P.SetPos;
             EPrompt;
             Wl^.WClose;
             If XPos>0 then TErase;         { Deleting old }
             Xpos := p.Xp; Ypos := p.Yp;
             TShow;
             Wl^.WOpen(x1,y1,x2,y2,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
            end{with};
            end{if};
           end{if};
          end{WOK};
         end{With};
         Dispose(Wd);
         OldOp := 'T';
        end{ Locate transition };

         end{case};
      Until c in [KEsc];
      Wl^.WClose;
     end{WOK};
     Dispose(Wl);
    End{ Locations };

{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

    Procedure Displays;

    Const  x1 =    3;  x2 =  189;   { Displays window - Ws }
           y1 =   79;  y2 =  143;

    Var    Ws    :PWindowP;
           Wls   :DWindowP;
           c     :char;
           s     :string;
           b     :word;
           OldOp,MOut : char;

    {------------------------------------------------------}

    Function MenuDisp(OldOp,MOut:char):char;
                                           { Displays menu }
    Const
      MenuDispGA:      { Positions and options for MenuDisp }
      array[1..3] of PositionG =
                  (( x1:  6; y1: 87; x2:186; y2:103; opt:'P' ),
                   ( x1:  6; y1:103; x2:186; y2:119; opt:'T' ),
                   ( x1:  6; y1:119; x2:186; y2:135; opt:'C' ));
    begin
      With Ws^ do begin
       WClear;
       WrtNEN (1,1,'List of ','p','laces');
       WrtNEN (1,2,'List of ','t','ransitions');
       WrtNEN (1,3,'Transition ','c','onnection');
       WaitUp;
       MouseCursorOn;
        MenuDisp := KeyOrMouseG(['P','T','C'],MenuDispGA,OldOp,MOut);
       MouseCursorOff;
       WClear;
       Wrt(1,1,'List of places');
       Wrt(1,2,'List of transitions');
       Wrt(1,3,'Transition connection');
      end{with};
    end;

    {------------------------------------------------------}

    Procedure DispTran(tr:PNt);
                           { Display transition connections}
    Var pl : PetriPP;
        num: Integer;
    Begin
      New(Wls);
      With Wls^ do begin
       WOpen(6,135,386,363,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
       If WOK then begin
        WrtNEN (1,1,'Connection of the transition ',tr^.Name,'');
        Wrt (2,2,' # Input place     Weight');
        Lines := 3;
        num := 0;
        m := TakeFirst(tr^.Inlist);
        While m<>nil do begin
           ar := m^.el;           { pointer to input arc }
           pl := ar^.Place;
           Inc(num);
           Str(num:2,s);
           WrtE(2,Lines,s);
           WrtE(5,Lines,pl^.Name);
           Str(ar^.Weight:8,s);
           WrtE(18,Lines,s);
           Inc(Lines);
           If Lines=LastL then begin
             Wrt(3,Lines,'Press any key');
             WaitUp; Wait;
             WClear;
             WrtNEN (1,1,'Connection of the transition ',tr^.Name,'');
             Wrt (2,2,' # Input place     Weight');
             Lines := 3;
           end{if};
           m := TakeNext(m);
        end{while};
        If Lines>=LastL-2 then begin
          Wrt(3,LastL,'Press any key');
          WaitUp; Wait;
          WClear;
          WrtNEN (1,1,'Connection of the transition ',tr^.Name,'');
          Lines := 2;
        end{if};
        Wrt(2,Lines,' # Output place    Weight');
        Inc(Lines);
        num := 0;
        m := TakeFirst(tr^.Outlist);
        While m<>nil do begin
           ar := m^.el;        { pointer to output arc }
           pl := ar^.Place;
           Inc(num);
           Str(num:2,s);
           WrtE(2,Lines,s);
           WrtE(5,Lines,pl^.Name);
           Str(ar^.Weight:8,s);
           WrtE(18,Lines,s);
           Inc(Lines);
           If Lines=LastL then begin
               Wrt(3,Lines,'Press any key');
               WaitUp; Wait;
               WClear;
               WrtNEN (1,1,'Connection of the transition ',tr^.Name,'');
               Wrt (2,2,' # Output place    Weight');
               Lines := 3;
           end{if};
           m := TakeNext(m);
       end{while};
       Wrt(3,LastL,'Press any key');
       WaitUp; Wait;
       WClose;
      end{WOK};
     end{with};
     Dispose(Wls);
    End{DispTran};

    {------------------- Displays.body ----------------------}

   Begin
    OldOp := 'P'; MOut := KEsc;
    New(Ws);
    Ws^.WOpen(x1,y1,x2,y2,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
    If Ws^.WOK then begin
     Repeat
      c := MenuDisp(OldOp,MOut);
      Case c of

     'P': begin                            { Display places }
           New(Wls);
           With Wls^ do begin
            WOpen(6,103,386,331,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
            If WOK then begin
             WrtNEN (1,1,'Petri net ',Name,' List of places:');
             Wrt(2,2,'Name       Marking Xpos Ypos Size Color Mcol');
             Lines := 3; c := ' ';
             m := Takefirst( Places);
             While (m<>nil) and (c<>KEsc) do begin
               pl := m^.el;             { pointer to place }
               WrtE(2,Lines,pl^.Name);
               Str(pl^.Marking:7,s);  WrtE(13,Lines,s);
               Str(pl^.Xpos:5,s);     WrtE(20,Lines,s);
               Str(pl^.Ypos:5,s);     WrtE(25,Lines,s);
               Str(pl^.Size:5,s);     WrtE(30,Lines,s);
               Str(pl^.Color:6,s);    WrtE(35,Lines,s);
               Str(pl^.MColor:5,s);   WrtE(41,Lines,s);
               Inc(Lines);
               m := TakeNext(m);
               If (Lines=LastL) and (m<>nil) then begin
                  Wrt(2,Lines,'Press any key (Esc to finish)');
                  WaitUp;
                  c := CWait(b);
                  If b=2 then c := KEsc;
                  WClear;
                  WrtNEN (1,1,'Petri net ',Name,' List of places:');
                  Wrt(2,2,'Name       Marking Xpos Ypos Size Color Mcol');
                  Lines := 3;
               end{if};
             end{while};
             If c<>KEsc then begin
               Wrt(2,LastL,'Press any key');
               WaitUp; Wait;
             end;
             WClose;
            end{WOK};
           end{with};
           Dispose(Wls);
           OldOp := 'P';
          end{P};

     'T': begin                       { Display transitions }
           New(Wls);
           With Wls^ do begin
            WOpen(6,119,386,347,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
            If WOK then begin
             WrtNEN (1,1,'Petri net ',Name,' List of transitions:');
             Wrt(2,2,'Name             Xpos Ypos Size  Color Dir');
             Lines := 3; c := ' ';
             m := Takefirst( Transitions);
             While (m<>nil) and (c<>KEsc) do begin
               tr := m^.el;           { pointer to transition }
               WrtE( 2,Lines,tr^.Name);
               Str(tr^.Xpos:4,s);    WrtE(19,Lines,s);
               Str(tr^.Ypos:5,s);    WrtE(23,Lines,s);
               Str(tr^.Size:5,s);    WrtE(28,Lines,s);
               Str(tr^.Color:6,s);   WrtE(34,Lines,s);
               If tr^.Direct=Horiz then WrtE(40,Lines,' Hor')
                                   else WrtE(40,Lines,' Ver');
               Inc(Lines);
               m := TakeNext(m);
               If (Lines=LastL) and (m<>nil) then begin
                  Wrt(2,Lines,'Press any key (Esc to finish)');
                  WaitUp;
                  c := CWait(b);
                  If b=2 then c := KEsc;
                  WClear;
                  WrtNEN (1,1,'Petri net ',Name,' List of transitions:');
             Wrt(2,2,'Name             Xpos Ypos Size  Color Dir');
                  Lines := 3;
               end{if};
             end{while};
             If c<>KEsc then begin
               Wrt(2,LastL,'Press any key');
               WaitUp; Wait;
             end;
             WClose;
            end{WOK};
           end{with};
           Dispose(Wls);
           OldOp := 'T';
          End{T};

    'C': begin                      { Transition connection }
          New(Wd);
          found := false;
          With Wd^ do begin
           WOpen(6,135,230,212,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
           If WOK then begin
            tr := GiveTrans(Wd,found);;
            WClose;
           end{WOK};
          end{with};
          Dispose(Wd);
          If found then
             DispTran(tr); { Displaying transition connection }
          OldOp := 'C';
         End{C};

        end{case};
       Until c=KEsc;
       Ws^.WClose;
      end{WOK};
      Dispose(Ws);
    End{ Displays };

{******************* PNet.Modify.body **********************}

  Begin
    OldOp := 'S';
    MOut := KEsc;
    New(Wn);
    With Wn^ do begin
     WOpen(x1,y1,x2,y2,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
     If WOK then begin
      Repeat
       Option := MenuMod(OldOp,MOut);
       {Show;}
       If Option in ['S','R','D'] then OldOp := Option;
       Case Option of

       'S': begin
               WClose;
               Editing;
               WOpen(x1,y1,x2,y2,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
            end;

       'R': Locations;

       'D': Displays;

      End{case};
     until Option in [KEsc,KLeft,KRight];
     MenuOut1 := Option;
     WClose;
    end{WOK};
   end{with};
   Dispose(Wn);
  end;

{**************************************************************}

  Procedure PNet.Simulate;

  Const  x1 =  48; y1 =  23;          { Simulate window - Ws }
         x2 = 189; y2 = 135;

         x1s =  51; y1s =  47;              { Set mode - Wsm }
         x2s = 253; y2s = 159;

         x1p =  51; y1p = 127;      { Simulation setup - Wsm }
         x2p = 225; y2p = 207;

         x1e =  54; y1e = 167;  { Experiment duration - Wnum }
         x2e = 228; y2e = 215;

         x1n = 584; y1n =  14;        { Dialog window - Wnum }
         x2n = 639; y2n =  40;

      Menu1GA:         { Positions and options for Mode menu }
      array[1..6] of PositionG =
                  (( x1: 54; y1: 55; x2:250; y2: 71; opt:'0' ),
                   ( x1: 54; y1: 71; x2:250; y2: 87; opt:'1' ),
                   ( x1: 54; y1: 87; x2:250; y2:103; opt:'2' ),
                   ( x1: 54; y1:103; x2:250; y2:119; opt:'3' ),
                   ( x1: 54; y1:119; x2:250; y2:135; opt:'4' ),
                   ( x1: 54; y1:135; x2:250; y2:151; opt:'5' ));

      Menu1SP:        { Positions and options for Setup menu }
      array[1..3] of PositionG =
                  (( x1: 54; y1:135; x2:222; y2:151; opt:'S' ),
                   ( x1: 54; y1:151; x2:222; y2:167; opt:'R' ),
                   ( x1: 54; y1:167; x2:222; y2:183; opt:'E' ));

  Var Option  :char;
      Ws,Wsm,Wnum  :PWindowP;
      Wd      :DWindowP;
      Lines   :Byte;
      FinSim  :Boolean;  { Whether to finish simulation run }
      found,OK,OKn :Boolean;
      EnTr    :word;        { Number of enabled transitions }
      Count   :Integer;
      FTr     :longint;
      rt      :word;                    { Random transition }
      c       :char;
      pl,p    :PNp;
      tr,te   :PNt;            { Current,Enabled transition }
      m,n     :Nodepoint;
      x,y,b   :word;
      i,dur   :longint;
      s       :string;
      OldOp,OldS,MOut : char;
      FirePri :word;     { Priority of firing transition(s) }

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

    Function MenuSim(OldOp,MOut:char):char;
                                     { Main simulation menu }
    Const
      MenuSimGA:      { Positions and options for MenuSim }
      array[1..6] of PositionG =
                  (( x1: 51; y1: 31; x2:186; y2: 47; opt:'E' ),
                   ( x1: 51; y1: 47; x2:186; y2: 63; opt:'C' ),
                   ( x1: 51; y1: 63; x2:186; y2: 79; opt:'S' ),
                   ( x1: 51; y1: 79; x2:186; y2: 95; opt:'R' ),
                   ( x1: 51; y1: 95; x2:186; y2:111; opt:'I' ),
                   ( x1: 51; y1:111; x2:186; y2:127; opt:'P' ));
    begin
     With Ws^ do begin
      WClear;
      WrtNEN(1,1,'S','e','t mode');
      WrtNEN(1,2,'','C','hange marking');
      WrtNEN(1,3,'','S','tart simulation');
      WrtNEN(1,4,'','R','ecord marking');
      WrtNEN(1,5,'','I','nitiate');
      WrtNEN(1,6,'Simulation setu','p','');
      WaitUp;
      MouseCursorOn;
      MenuSim := KeyOrMouseG(['E','C','S','R','I','P'],
                             MenuSimGA,OldOp,MOut);
      MouseCursorOff;
      WClear;
      Wrt(1,1,'Set mode');
      Wrt(1,2,'Change marking');
      Wrt(1,3,'Start simulation');
      Wrt(1,4,'Record marking');
      Wrt(1,5,'Initiate');
      Wrt(1,6,'Simulation setup');
     end{with};
    end;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

{    Procedure UserExp;   { User experiment - in SimPet5 }

{******************* PNet.Simulate.body ********************}

   Begin
    Randomize;
    OldOp := 'S';
    MOut := KEsc;
    New(Ws);
    With Ws^ do begin
     WOpen(x1,y1,x2,y2,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
     If WOK then begin
      Repeat
       Show;
       MainTitles(false);
       SetFillStyle(SolidFill,Black);
       Bar(620,0,639,17);
       Str(Mode:2,s);
       Wx(73,1,'Mode ='); Wex(79,1,s);
       WClear;
       Option := MenuSim(OldOp,MOut);
      Case Option of

     'E': begin                              { Set mode }
           New(Wsm);
           With Wsm^ do begin
            WOpen(x1s,y1s,x2s,y2s,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
            If WOK then begin
             WrtNEN(1,1,'','0',' : always stop');
             WrtNEN(1,2,'','1',' : stop if two or more');
             WrtNEN(1,3,'','2',' : just confirmation');
             WrtNEN(1,4,'','3',' : running with delays');
             WrtNEN(1,5,'','4',' : fast running');
             WrtNEN(1,6,'','5',' : user experiment');
             WaitUp;
             MouseCursorOn;
             c := KeyOrMouseG(['0','1','2','3','4','5'],Menu1GA,'1',KEsc);
             MouseCursorOff;
             If c in ['0','1','2','3','4','5'] then
                 Mode := Ord(c) - Ord('0');
             WClose;
            end{WOK};
           end{with};
           Dispose(Wsm);
           OldOp := 'E';
          end{M};

     'C': begin                        { Change marking }
           New(Wd);
           found := false;
           With Wd^ do begin
            WOpen(51,63,256,140,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
            If WOK then begin
             pl := GivePlace(Wd,found);
             If found then begin
               i := pl^.Marking;
               WClear;
               WrtNEN (1,1,'Place ',pl^.Name,'');
               Str(pl^.Marking:5,s);
               WrtNEN (1,2,'Marking = ',s,'');
               Wrt (1,3,'New marking:');
               WaitUp;
               WRI(14,3,i,OK);
               pl^.Marking := i;
             end{if};
             WClose;
            end{WOK};
           end{with};
           Dispose(Wd);
           If found then ClearDevice;
           OldOp := 'C';
          end{C};

   {***************** Simulation.run **************************}

   'S': begin                         { Start simulation }

     If Mode=5 then UserExp            { User experiment }

     Else begin                           { Modes 1 to 4 }

       FinSim := false;
       Repeat                     { Main simulation loop }

{ Display P-net and prompt }
          ClearDevice;
          Delay(200);
          Str(Mode:2,s);
          Wx(73,1,'Mode ='); Wex(79,1,s);
          If Mode>2 then
               Wx(1,1,'Press Esc or right button to stop');
          Show;                          { Display P-net }

{ Searching for enabled transitions }
          n := Takefirst(Transitions);
          EnTr := 0;
          FirePri := 65535;
          While (n<>nil) do begin
            tr := n^.el;
             If tr^.Enabled then begin
               If tr^.Priority<FirePri then begin { Higher - replace }
                 EnTr := 1;
                 te := tr;
                 FirePri := tr^.Priority;
                end
               else if tr^.Priority=FirePri then begin { Same, add }
	         Inc(EnTr);
	         te:=tr;
               end;            { Ignored if lower priority }
	     end;
            n := Takenext(n)
          end{while};

{ Deadlock }
          If EnTr=0 then begin
                SetFillStyle(SolidFill,Black);
                Bar(0,0,400,16);     { Clearing previous text }
                Wex(1,1,'! DEADLOCK !  (No enabled transitions)');
                Wx (1,2,'Press any key');
                Wait;
                FinSim := true;
           end{Deadlock}

{ Asking for transition to be fired - Modes 0 and 1 }
          Else if (Mode=0) or (Mode=1)and(EnTr>1) then begin
             Wx(1,1,'Transitions to fire:');
             Lines := 2;
             Count := 1;
             n := Takefirst( Transitions);
             While n<>nil do begin
               tr := n^.el;           { pointer to transition }
               If tr^.Enabled and (tr^.Priority=FirePri) then begin
                  Str(Count:3,s);
                  Wex(1,Lines,s); Wex(6,Lines,tr^.Name);
                  Lines := Lines mod 24 + 1;
                  Inc(Count);
               end{if};
               If Lines=1 then begin
                  Wx(1,Lines,'Press any key');
                  Wait;
               end{if};
               n := TakeNext(n);
             end{while};
             Wex(24,3,'Right button, Enter or Esc to stop');
             If MouseOk then begin
               Wx (24,2,
                'Select by mouse (left) or type number to be fired:');
               MouseCursorOn;
               Repeat
                  b:=MousePosition(x,y)
               Until KeyPressed or (b>0);
               MouseCursorOff;
              end{mouse}
             else
              Wx (24,2,'Type number to be fired:');

        If KeyPressed or not MouseOk then begin     { number typed }

            New(Wnum);
            With Wnum^ do begin
             WOpen(x1n,y1n,x2n,y2n,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
             If WOK then begin
              Repeat
                FTr := -1;
                WRI(1,1,FTr,OK);        { Number must be entered ! }
                If OK then begin
                    OKn := (Ftr>0) and (Ftr<=EnTr);
                    {If not OKn then Write(#7);}
                end{if};
                WClear;
              until not OK or OK and OKn;
              WClose;
             end{then WOK}
             Else OK := false;
            end{with};
            Dispose(Wnum);
            If not OK then FinSim := true
            else begin
               n := Takefirst(Transitions);
               found := false;
               Count := 1;
               While not found do begin
                  tr := n^.el;
                  if tr^.Enabled and (tr^.Priority=FirePri) then
                    if Count=FTr then found := true
                                 else Inc(Count);
                  n := Takenext(n)
               end{while};
             end{if}
         end{number}

        Else begin                  { Transition by mouse }
           If b=2 then FinSim := true
                  else repeat
                          OK := false;
                          tr := SelectTrans;
                          If tr<>nil then OK := tr^.Enabled
			                        and (tr^.Priority=FirePri)
                                     else FinSim := true;
                          { If not OK then Write(#7); }
                       until OK or FinSim;
        End{if};
        If not FinSim then tr^.Fire;
       end{Asking}

{ Running - modes 2,3,4 }
          Else begin
             If EnTr>1 then begin     { Random selection if more }
                 rt := Random(EnTr) + 1;
                 n := Takefirst(Transitions);
                 while rt>0 do begin
                    te := n^.el;
                    if te^.Enabled and (tr^.Priority=FirePri) then Dec(rt);
                    n := Takenext(n)
                 end{while};
             end{if};          { te = transition to be fired }

{ Confirmation Message }
             If Mode<=2 then begin
                Wx (1,1,'Firing the transition');
                Wex(1,2,te^.Name);
                Wx (31,2,'Press any key to fire ');
                Wex(54,2,'Esc or right button to stop');
                c := ' ';
                If MouseOk then begin
                  Repeat
                   b := MousePosition(x,y)
                  until Keypressed or (b>0);
                  If Keypressed then
                    c:=UpCase(ReadK)
                  else begin
                    if b=2 then c:=KEsc;
                    WaitUp;
                  end{if};
                 end{mouse}
                else
                  c := UpCase(ReadK);
                If c=KEsc then begin
                  FinSim := true;{ Delay(250); }
                end;
             end{if};

{ Delay for Mode 3 }
             If Mode=3 then Delay(FireDelayf);

{ Test if Esc or right button }
             c := ' ';
             b := MousePosition(x,y);
             If KeyPressed or (b>0) then begin
                If Keypressed then c:=UpCase(ReadK)
                              else if b=2 then c:=KEsc;
                If c=KEsc then begin
                    FinSim := true; Delay(250);
                end;
             end{if};

{ Firing }
             If not FinSim then te^.Fire;

          end{Running}
        Until FinSim;
       End{If-Mode=5};
       ClearDevice;
       OldOp := 'S';
     end{S};

   {*******************************************************}

     'R': Begin                           { Record marking }
            n := TakeFirst(Places);  { Searching in Places }
            While (n<>nil) do begin
               p := n^.el;
               p^.OldMark := p^.Marking;
               n := TakeNext(n);
            end{while};
            OldOp := 'R';
          end{R};

     'I': Begin                                 { Initiate }
            n := TakeFirst(Places);  { Searching in Places }
            While (n<>nil) do begin
               p := n^.el;
               p^.Marking := p^.OldMark;
               n := TakeNext(n);
            end{while};
            ClearDevice;
            OldOp := 'I';
          end{I};

     'P': begin                         { Simulation Setup }
          New(Wsm);
          With Wsm^ do begin
           WOpen(x1p,y1p,x2p,y2p,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
           OldS := 'S';
           Repeat
            If WOK then begin
             WClear;
             {If Timing then WrtNEN (1,1,'','T','iming is ON')
                       else WrtNEN (1,1,'','T','iming is OFF');}
             If Confir then WrtNEN (1,1,'','S','tep mode is ON ')
                       else WrtNEN (1,1,'','S','tep mode is OFF');
             If TableFormat then WrtNEN (1,2,'','R','esults in TABLE')
                            else WrtNEN (1,2,'','R','esults SEPARATE');
             WrtNEN(1,3,'','E','xperiment duration');
             Str(MaxTime:19:0,s);
             WrtE(1,4,s);
             WaitUp;
             MouseCursorOn;
             c := KeyOrMouseG(['S','R','E'],Menu1SP,OldS,KEsc);
             MouseCursorOff;
             Case c of
             {'T': begin
                   Timing := Not Timing;
                   OldS := 'T';
                  end;}
             'S': begin
                   Confir := Not Confir;
                   OldS := 'S';
                  end;
             'R': begin
                   TableFormat := Not TableFormat;
                   OldS := 'R';
                  end;
             'E': begin
                New(Wnum);
                With Wnum^ do begin
                  WOpen(x1e,y1e,x2e,y2e,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                  If WOK then begin
                   Repeat
                     Wrt(1,1,'Enter the duration:');
                     WRI(1,2,dur,OK);    { Number must be entered ! }
                     If OK then begin
                       OKn := (dur>0);
                       {If not OKn then Write(#7);}
                     end{if};
                     WClear;
                   until not OK or OK and OKn;
                   WClose;
                   If (OK and OKn) then MaxTime := dur;
                  end{then WOK};
                end{with};
                Dispose(Wnum);
                OldS := 'E';
               End{E};
             End{case};
             WClear;
            end{WOK};
           Until c=KEsc;
           WClose;
          end{with};
          Dispose(Wsm);
          OldOp := 'P';
         end{P};

      end{case};
     until Option in [KEsc,KLeft,KRight];
     WClose;
    end{WOK};
   end{with};
   Dispose(Ws);
   MenuOut1 := Option;
  End{Simulate};

{**************************************************************}

  Procedure PNet.Show;
  { Displays all located places, transitions and arcs }

  var p:PNp;
      t:PNt;
      a:PNa;
      old:byte;
      n,m:Nodepoint;

  begin
     If GBandW then begin
        old := CuMeCo;
        CuMeCo := White;
        Wx (54,1,'P-net ');
        Wx (60,1,Name);
        CuMeCo := old;
      end
     else begin
        Wx (54,1,'P-');
	if EditingTF then Wex(56,1,'n')
	             else Wx (56,1,'n');
	Wx(57,1,'et ');
        Wx(60,1,Name)
     end{if};
     n := Takefirst(Transitions);
     while n<>nil do begin
        t := n^.el;            { pointer to transition }
        m := TakeFirst(t^.Inlist);
        while m<>nil do begin
           a := m^.el;         { pointer to input arc }
           if not Empty(a^.Corners) then a^.AShow;
           m := TakeNext(m);
        end{while};
        m := TakeFirst(t^.Outlist);
        while m<>nil do begin
           a := m^.el;        { pointer to output arc }
           if not Empty(a^.Corners) then a^.AShow;
           m := TakeNext(m);
        end{while};
        if t^.Xpos > 0 then t^.TShow;
        n := TakeNext(n);
     end{while};
     n := Takefirst(Places);
     while n<>nil do begin
        p := n^.el;           { pointer to place }
        if p^.Xpos>0 then p^.PShow;
        n := TakeNext(n);
     end{while};
  end;

{**************************************************************}

  Procedure PNet.Store (var F: Text; Option: Char);
      { Storing the net to F: Text, F after successful Rewrite }
                       { Option: 'A' = all, 'V' = only visible }
  Var m,n,r:Nodepoint;
      ndel:PointP;
      p:PetriPP;
      t:PNt;
      a:PNa;
      po:PPnt;
      Nop,Nott:Integer; { Counters }

  Begin
     Writeln(F,Name);                   { Net name }
     WriteLn(F,9:3);                      { Format }

     If Option='V' then begin   { Counting visible }
        n := TakeFirst(Places);           { Places }
        Nop := 0;
        while n<>nil do begin
           p := n^.el;          { pointer to place }
           If (p^.XPos>0) then Inc(Nop);
           n := TakeNext(n);
        end{while};
        n := TakeFirst(Transitions); { Transitions }
        Nott := 0;
        while n<>nil do begin
           t := n^.el;     { pointer to transition }
           If (t^.XPos>0) then Inc(Nott);
           n := TakeNext(n);
        end{while};
      end
     else begin
        Nop := Nopl;
        Nott := Notr;
     end{counting};

     Writeln(F,Nop,' ',Nott);

     n := TakeFirst(Places);              { Places }
     while n<>nil do begin
        p := n^.el;             { pointer to place }
        If (p^.XPos>0) or (Option='A') then begin
           Writeln(F,p^.Name);
           Writeln(F,p^.Xpos,' ',p^.Ypos,' ',p^.Size,' ',
                     p^.Color,' ',p^.MColor,' ',p^.Marking,
                     ' ',p^.PType);
           If p^.Timing then Writeln(F,'T  Place timing true')
                        else Writeln(F,'F  Place timing false');
        end;
        n := TakeNext(n);
     end{while};

     n := TakeFirst(Transitions);     { Transitions }
     while n<>nil do begin              
      t := n^.el;           { pointer to transition }
      If (t^.XPos>0) or (Option='A') then begin
        Writeln(F,t^.Name);
        Writeln(F,t^.Xpos,' ',t^.Ypos,' ',t^.Size,' ',
                  t^.Color,' ',t^.Direct,' ',t^.TType);
        Writeln(F,t^.TranNum);             { Number }
        Writeln(F,t^.Priority);          { Priority }
        With t^.RandDur do begin
          Writeln(F,DistType);
          Writeln(F,InitSeed);
          Case DistType of
          0,3: WriteLn(F,P1);      { Constant, Exponential }
          1,2: begin
                ndel := FXList;
                while ndel<>nil do begin
                  WriteLn(F,ndel^.Value:20:10,'   ',ndel^.Px:10:8);
                  ndel := ndel^.Next;
                end{while};
                WriteLn(F,-1.0:20:10,'   ',-1.0:10:8);  {Last row }
               end;
          4  : begin
                WriteLn(F,P1);     { Erlang }
                WriteLn(F,PI1);
               end;
          5,7: begin
                WriteLn(F,P1);     { Uniform, Normal }
                WriteLn(F,P2);
               end;
          6  : begin
                WriteLn(F,P1);     { Triangular }
                WriteLn(F,P2);
                WriteLn(F,P3);
               end;
          8  : begin
                WriteLn(F,P1);     { Normal in n sigma }
                WriteLn(F,P2);
                WriteLn(F,PI1);
               end;
          end{case};
        end{with};

        Writeln(F,GetSize(t^.Inlist));  { Number of Input arcs }
        m := TakeFirst(t^.Inlist);      { Input arcs }
        while m<>nil do begin
           a := m^.el;      { pointer to arc }
           p := a^.Place;
           Writeln(F,p^.Name);
           WriteLn(F,a^.Color,' ',a^.Width,' ',
                     a^.Style,' ',a^.Weight,' ',a^.AType);
           { Probability not saved for input arcs !! }
           Writeln(F,GetSize(a^.Corners));  { Number of points }
           r := TakeFirst(a^.Corners);      { Points }
           while r<>nil do begin
              po := r^.el;   { pointer to point }
              Writeln(F,po^.Xp,' ',po^.Yp);
              r := TakeNext(r);
           end{while};
           m := TakeNext(m);
        end{while};

        Writeln(F,GetSize(t^.Outlist));  { Number of Output arcs }
        m := TakeFirst(t^.Outlist);      { Output arcs }
        while m<>nil do begin
           a := m^.el;      { pointer to arc }
           p := a^.Place;
           Writeln(F,p^.Name);
           WriteLn(F,a^.Color,' ',a^.Width,' ',
                     a^.Style,' ',a^.Weight); { No type for output arcs }
           WriteLn(F,a^.BranchPro:10:8);
           Writeln(F,GetSize(a^.Corners));  { Number of points }
           r := TakeFirst(a^.Corners);      { Points }
           while r<>nil do begin
              po := r^.el;   { pointer to point }
              Writeln(F,po^.Xp,' ',po^.Yp);
              r := TakeNext(r);
           end{while};
           m := TakeNext(m);
        end{while};
      end{if saved};
      n := TakeNext(n);  { Next transition }
     end{while};
  End;

{**************************************************************}

  Procedure PNet.Load(var F: Text);     { Reading P-net from F }
                      { F exists and is after successful Reset }
                                  { Name has already been read }
  Var n:Nodepoint;
      p:PNp;
      t:PNt;
      a:PNa;
      po:PPnt;
      s:string[Namelen];
      i,j,k,ni,no,np,format:Integer;
      found:Boolean;
      Xp,Yp,Siz,Col,MCol,Mar,Typ,Dir:Word;
      Nam:string;
      TNum:Integer;
      x,pr:Real;
      c:Char;
  Begin
     ReadLn(F,format);
     Readln(F,Nopl,Notr);
     NotrG := 0;

     SQS := nil;
     Terminate := false;
     Time := 0;

     Places := Newlist;             { Places }
     For i:=1 to Nopl do begin
        New(p);
        Readln(F,Nam);
        Read(F,Xp,Yp,Siz,Col,MCol,Mar);
        If format>3 then Read(F,Typ) else Typ := 0;
        ReadLn(F);
        p^.PInit(Xp,Yp,Siz,Col,MCol,Mar,Nam,Typ);
        If format>8 then begin
          Readln(F,c);
          p^.Timing := c='T';
        end;
        New(n);
        n^.el := p;
        Inslast(n,Places);
     end{for};

     Transitions := Newlist;           { Transitions }
     For i:=1 to Notr do begin
        Readln(F,Nam);
        Read(F,Xp,Yp,Siz,Col,Dir);
        If format>3 then Read(F,Typ) else Typ := 0;
        ReadLn(F);
        New(t);
        If format=2 then TNum:=i else Readln(F,TNum);
        t^.TInit(Xp,Yp,Siz,Col,Dir,Nam,TNum,Typ);
        If TNum>NotrG then NotrG := TNum;
	           { NotrG = last existing number !! }
        If format>7 then ReadLn(F,t^.Priority);
        If format>6 then
         With t^.RandDur do begin
          ReadLn(F,DistType);
          ReadLn(F,InitSeed);
          Case DistType of
          0,3: ReadLn(F,P1);      { Constant, Exponential }
          1,2: begin
                ReadLn(F,x,pr);
                while pr<>-1 do begin
                  if not FxAdd(x,pr) then begin
                    write('Wrong table distr. data');
                    readln;
                  end;
                  ReadLn(F,x,pr);
                end{while};
               end;
          4  : begin
                ReadLn(F,P1);     { Erlang }
                ReadLn(F,PI1);
               end;
          5,7: begin
                ReadLn(F,P1);     { Uniform, Normal }
                ReadLn(F,P2);
               end;
          6  : begin
                ReadLn(F,P1);     { Triangular }
                ReadLn(F,P2);
                ReadLn(F,P3);
               end;
          8  : begin
                ReadLn(F,P1);     { Normal in n sigma }
                ReadLn(F,P2);
                ReadLn(F,PI1);
               end;
          end{case};
        end{if&with};
        Readln(F,ni);         { Number of input arcs }
        For j:=1 to ni do begin
           new(a);
           Readln(F,s);                 { Place name }
           If format=1 then begin
               Readln(F,a^.Weight,a^.Color,a^.Width);
               a^.Style := SolidLn;
               a^.AType := 0;
              end{format1}
           else if format<5 then begin {format2-4: no type}
               ReadLn(F,a^.Color,a^.Width,a^.Style,a^.Weight);
               a^.AType := 0;
              end
           else
               ReadLn(F,a^.Color,a^.Width,a^.Style,a^.Weight,a^.AType);
           { No probability for input arcs }
           n := TakeFirst(Places);
           found := false;
           while not found do begin
              p := n^.el;
              found := s=p^.Name;
              n := TakeNext(n);
           end{while};
           a^.Place := p;
           a^.Transition := t;
           a^.Corners := Newlist;
           Readln(F,np);     { Number of points }
           for k:=1 to np do begin
              new(po);
              Readln(F,po^.Xp,po^.Yp);
              new(n); n^.el := po;
              Inslast(n,a^.Corners);
           end;
           new(n); n^.el := a;
           Inslast(n,t^.Inlist);
        end{for};

        t^.Outlist := Newlist;         { Output list }
        Readln(F,no);        { Number of output arcs }
        x := 0;                                { Cdf }
        for j:=1 to no do begin
           new(a);
           a^.AType := -1;
           Readln(F,s);                 { Place name }
           If format=1 then begin
               Readln(F,a^.Weight,a^.Color,a^.Width);
               a^.Style := SolidLn;
              end{format1}
           else
               ReadLn(F,a^.Color,a^.Width,a^.Style,a^.Weight);
           { No type for output arcs !! }
           If format>5 then begin
              ReadLn(F,a^.BranchPro);
              x := x + a^.BranchPro;
              a^.BranchCdf := x;
           end;
           n := TakeFirst(Places);
           found := false;
           while not found do begin
              p := n^.el;
              found := s=p^.Name;
              n := TakeNext(n);
           end{while};
           a^.Place := p;
           a^.Transition := t;
           a^.Corners := Newlist;
           Readln(F,np);     { Number of points }
           for k:=1 to np do begin
              new(po);
              Readln(F,po^.Xp,po^.Yp);
              new(n); n^.el := po;
              Inslast(n,a^.Corners);
           end;
           new(n); n^.el := a;
           Inslast(n,t^.Outlist);
        end{for};
        New(n);
        n^.el := t;
        Inslast(n,Transitions);
     end{for};
  End;

{**************************************************************}

  Procedure PNet.Destroy;
               { Destroying P-net from RAM - not net as such ! }

  Var n,m :Nodepoint;
      l :ListPoint;
      p :PNp;
      t :PNt;
      a :PNa;

  Begin

     n := TakeFirst(Places);               { Places }
     while n<>nil do begin
        p := n^.el;              { pointer to place }
        Dispose(p);
        Remove(n,Places);
        Dispose(n);
        n := TakeFirst(Places);
     end{while};
     l := Places;
     Dispose(l);

     n := TakeFirst(Transitions);     { Transitions }
     while n<>nil do begin
        t := n^.el;         { pointer to transition }

        m := TakeFirst(t^.Inlist);     { Input arcs }
        while m<>nil do begin
           a := m^.el;             { pointer to arc }
           a^.Destroy;
           Dispose(a);
           Remove(m,t^.Inlist);
           Dispose(m);
           m := TakeFirst(t^.Inlist);
        end{while};
        l := t^.Inlist;
        Dispose(l);

        m := TakeFirst(t^.Outlist);   { Output arcs }
        while m<>nil do begin
           a := m^.el;             { pointer to arc }
           a^.Destroy;
           Dispose(a);
           Remove(m,t^.Outlist);
           Dispose(m);
           m := TakeFirst(t^.Outlist);
        end{while};
        l := t^.Outlist;
        Dispose(l);

        Dispose(t);
        Remove(n,Transitions);
        Dispose(n);
        n := TakeFirst(Transitions);
     end{while};
     l := Transitions;
     Dispose(l);

  End{PNet.Destroy};

{**************************************************************}

End.

