Unit PNList;

{ This unit contains the types and routines for work with }
{ two-way linked (noncircular) lists.                     }
{ The type "List" represents the list as such.            }
{ It has these fields:                                    }
{              - size (number of nodes)                   }
{              - head (pointer to the first node)         }
{              - tail (pointer to the last node).         }
{ The type "Node" represents the node of the list. It has }
{ these fields:                                           }
{              - el (data part - pointer)                 }
{              - succ (successor - next node)             }
{              - pred (predecessor - previous node)       }

Interface

Type Nodepoint = ^Node;
     Listpoint = ^List;

     Node = record
	      el: pointer;      { data part = any pointer}
	      succ: Nodepoint;  { nil for the last node }
              pred: Nodepoint;  { nil for the first node }
	    end;

     List = record
	      size: word;
	      head: Nodepoint;  { nil for an empty list }
              tail: Nodepoint;  { nil for an empty list }
	    end;


function Newlist : Listpoint;
{ Returns pointer to a new empty list }

{ Note: procedure ListDestroy(l: Listpoint)  can not be  }
{       implemented here because of unknown data parts ! }

procedure Insfirst(n: Nodepoint; l: Listpoint);
{ Inserts the node "n" at the beginning of the list "l" }

procedure Inslast (n: Nodepoint; l: Listpoint);
{ Inserts the node "n" at the end of the list "l" }

procedure Insafter(m,n: Nodepoint; l: Listpoint);
{ Inserts the node "m" after "n" in the list "l" }

procedure Remove(n: Nodepoint; l: Listpoint);
{ Removes the node n from the list l }

function Remfirst(l: Listpoint): Nodepoint;
{ Removes and returns the first node of the list "l" }

function Takefirst(l: Listpoint): Nodepoint;
{ Returns the first node of the list "l" (doesn't remove)}

function Takenext(n: Nodepoint): Nodepoint;
{ Returns the next node after "n" (possibly NIL)}

function Remlast (l: Listpoint): Nodepoint;
{ Removes and returns the last node of the list "l" }

function Takelast(l: Listpoint): Nodepoint;
{ Returns the last node of the list "l" (doesn't remove)}

function Empty (l: Listpoint): Boolean;
{ Returns true if the list "l" is empty }

function Getsize(l: Listpoint): word;
{ Returns the size of the list "l" }

Implementation

{***********************************************************}

function Newlist;
var l: Listpoint;
begin
   new(l);
   l^.size := 0;
   l^.head := nil;
   l^.tail := nil;
   Newlist := l
end;

{***********************************************************}

procedure Insfirst;
begin
 with l^ do begin
   size := size + 1;
   if (size = 1) then
		   begin  {file was empty}
		     head := n;
		     tail := n;
		     n^.succ := nil;
		     n^.pred := nil
		   end
		 else
		   begin  {file was not empty}
		     n^.succ := head;
		     n^.pred := nil;
		     head^.pred := n;
		     head := n
		   end
 end  {of with}
end;

{***********************************************************}

procedure Inslast;
begin
 with l^ do begin
   size := size + 1;
   if (size = 1) then
		   begin  {file was empty}
		     head := n;
		     tail := n;
		     n^.succ := nil;
		     n^.pred := nil
		   end
		 else
		   begin  {file was not empty}
		     n^.succ := nil;
		     n^.pred := tail;
		     tail^.succ := n;
		     tail := n
		   end
 end  {of with}
end;

{***********************************************************}

procedure Insafter;
begin
 with l^ do begin
   size := size + 1;
   m^.pred := n;
   m^.succ := n^.succ;
   n^.succ := m;
   if m^.succ = nil then       {end of file }
		     tail := m
                    else
		     m^.succ^.pred := m;
 end  {of with}
end;

{***********************************************************}

procedure Remove;      { n from l }
begin
   if l^.head = n then n := RemFirst(l)
   else if l^.tail = n then n := RemLast(l)
   else begin
     l^.size := l^.size - 1;
     n^.pred^.succ := n^.succ;
     n^.succ^.pred := n^.pred;
   end{if};
end;

{***********************************************************}

function Remfirst;
begin
 with l^ do begin
   Remfirst := head;
   head := head^.succ;
   size := size - 1;
   if (head = nil) then tail := nil  {file becomes empty}
		   else head^.pred := nil
 end {of with}
end;

{***********************************************************}

function Takefirst;
begin
   If l=nil then TakeFirst := nil
            else Takefirst := l^.head
end;

{***********************************************************}

function Takenext;
begin
   Takenext := n^.succ;
end;

{***********************************************************}

function Remlast;
begin
 with l^ do begin
   Remlast := tail;
   tail := tail^.pred;
   size := size - 1;
   if (tail = nil) then head := nil  {file becomes empty}
		   else tail^.succ := nil
 end {of with}
end;

{***********************************************************}

function TakeLast;
begin
   If l=nil then TakeLast := nil
            else TakeLast:= l^.tail;
end;

{***********************************************************}

function Empty;
begin
   If l=nil then Empty := true
            else Empty := (l^.size = 0)
end;

{***********************************************************}

function Getsize;
begin
   If l=nil then GetSize := 0
            else Getsize := l^.size
end;

end.


