Unit Queuing;              { Queuing oriented objects }

Interface

  Uses Graph, GGraph, PNList, Petri, Inhibits, Icons, Simpet5,
       Statist;

  Type  QueuingPP = ^QueuingPlace;
        QueuingTP = ^QueuingTransition;

        QueuingArc = Object(InhibitorArc)
        End{QueuingArc};

        QueuingPlace = Object(InhibitorPlace)
           AccMarking : RAccum;        { Marking Statistics }
           TList : List;{ FIFO list of token arriving times }
           WaitTime : RTally;     { Waiting time statistics }
           Arrivals : Real;            { Number of arrivals }
           Procedure PSimInit;
           Procedure PSimFinish;
           Procedure PShow;
           Procedure PErase;
           Procedure AddTokens(n : LongInt); { Removing for n < 0 }
        End{QueuingPlace};

        QueuingTransition = Object(InhibitorTransition)
           AccBusy : RAccum;       { Utilization statistics }
           FireTime : RTally;      { Firing time statistics }
           FireStart : Real;            { Firing start time }
           Branch  : Integer;   { Output for branching type }
           Counter : LongInt;           { Number of firings }

           Procedure TSimInit;
           Procedure ShowShape;
           Procedure ShowStatus;
           Procedure TShow;           { Both shape & status }
           Procedure EraseShape;
           Procedure EraseStatus;
           Procedure TErase;          { Both shape & status }
           Procedure StartFire;             { Remove tokens }
           Procedure EndFire;                  { Add tokens }
           Procedure Fire;           { Firing: Remove & Add }
        End{QueuingTransition};

        TRecTypeP = ^TRecType;                { Time record }
        TRecType = record
           TRec : Real;
        End;

Implementation

{****************** Queuing Objects Methods *****************}

  Procedure QueuingPlace.PSimInit;    { Place initialization }
  Var i : LongInt;
      t : TRecTypeP;
      nod : Nodepoint;
  Begin
     AccMarking.Init(Marking);       { Statistics initiation }
     WaitTime.Init;
     TList.size := 0;                      { List initiation }
     TList.head := nil;
     TList.tail := nil;
     Arrivals := 0;           { Initial marking not included }
     If Timing then
      For i:=1 to Marking do begin { Records for initial marking tokens }
       t := New(TRecTypeP);           { Creating time record }
       t^.TRec := PNTime;
       nod := New(Nodepoint);         { Creating list record }
       nod^.el := t;
       Inslast(nod,Listpoint(@TList)); { Inserting to time FIFO list }
     end{If,For}
  End;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

  Procedure QueuingPlace.PSimFinish;  { Finishing User Experiment }
  Var i : LongInt;
      t : TRecTypeP;
      nod : Nodepoint;
  Begin
     While not Empty(Listpoint(@TList)) do begin
        nod := Remfirst(Listpoint(@TList));
        t := nod^.el;                 { Time record }
        Dispose(t);                   { Erasing records from RAM }
        Dispose(nod);
     End{While};
  End;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
  Procedure QueuingPlace.PShow;
  Begin
     Case PType of

      1 : Begin                      { 1 = Horizontal Queue }
           HQueueIcon(XPos,YPos,Size,Color,Name);
           ShowMarking(MColor);
          end;

      2 : Begin                        { 2 = Vertical Queue }
           VQueueIcon(XPos,YPos,Size,Color,Name);
           ShowMarking(MColor);
          end;

      else InhibitorPlace.PShow; { others = standard circle }
     end{case}
  end;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

  Procedure QueuingPlace.PErase;
  Begin
     Case PType of

      1 : Begin                      { 1 = Horizontal Queue }
           HQueueIcon(XPos,YPos,Size,Black,Name);
           ShowMarking(Black);
          end;

      2 : Begin                        { 2 = Vertical Queue }
           VQueueIcon(XPos,YPos,Size,Black,Name);
           ShowMarking(Black);
          end;

      else InhibitorPlace.PErase;{ others = standard circle }
     end{case}
  end;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

  Procedure QueuingPlace.AddTokens(n : LongInt);
  Var i : LongInt;                    { Removing for n < 0 }
      t : TRecTypeP;
      nod : Nodepoint;

  Begin
   Inc(Marking,n);
   If Mode=5 then begin
    AccMarking.RUpDate(Marking);
    If n>0 then Arrivals := Arrivals + n;
    If Timing then
     If n>0 then begin           { Adding time records to list }
       For i:=1 to n do begin
         t := New(TRecTypeP);           { Creating time record }
         t^.TRec := PNTime;
         nod := New(Nodepoint);         { Creating list record }
         nod^.el := t;
         Inslast(nod,Listpoint(@TList)); { Inserting to time FIFO list }
       end{For}
      end{then}
     Else If n<0 then begin  { Removing time records from list }
       For i:=1 to -n do begin
         nod := Remfirst(Listpoint(@TList));
         t := nod^.el;                           { Time record }
         WaitTime.RUpdate(PNTime - t^.TRec); { Updating statistics }
         Dispose(t);                { Erasing records from RAM }
         Dispose(nod);
       end{For}
     End{If};
   End{If};
  End;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

  Procedure QueuingTransition.TSimInit;
  Begin
     AccBusy.Init(0);
     FireTime.Init;
     Branch := 0;
     Counter := 0;
     With RandDur do
      If InitSeed=0 then begin
        Randomize;
        Seed := RandSeed;
       end
      else
        Seed := InitSeed;
  End;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

  Procedure QueuingTransition.ShowShape;
  Begin
     Case TType of

      1 : Begin                        { 1 = Timed Transition }
           If Direct=Horiz then
              HTimedTranIcon(XPos,YPos,Size,Color,Name)
           Else
              VTimedTranIcon(XPos,YPos,Size,Color,Name);
          end;

      2 : Begin                      { 2 = Customer Generator }
           GeneratorIcon(XPos,YPos,Size,Color,Name);
          end;

      3 : Begin            { 3 = Branching - Selects 1 Output }
           If Direct=Vert then
              VBranchingIcon(XPos,YPos,Size,Color,Name)
           Else
              HBranchingIcon(XPos,YPos,Size,Color,Name);
          end;

      else InhibitorTransition.ShowShape; { others = standard }
     end{case}
  End;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

  Procedure QueuingTransition.ShowStatus;
  Var s : String;
  Begin
     If Enabled or Firing then Begin
       If Firing then
         SetLineStyle(DottedLn,0,ThickWidth)
       else
         SetLineStyle(SolidLn, 0,ThickWidth);
       GSetColor(Red);
       Line(XPos - Size shr 2 + 3, YPos,
            XPos, YPos + Size shr 2);
       Line(XPos, YPos + Size shr 2,
            XPos + Size shr 1, YPos - Size shr 2);
       If Firing then Begin
        Str(FireEnd:3:2,s);
        GSetColor(TimCol);
        SetTextStyle(MenuFont,HorizDir,MenuSize);
        SetTextJustify(CenterText,BottomText);
        OutTextXY(Xpos, Ypos - Size shr 1 - 4, s);
       end{if};
     end{if};
  End;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

  Procedure QueuingTransition.TShow;
  Begin
     ShowShape;
     ShowStatus;
  end;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

  Procedure QueuingTransition.EraseShape;
  Begin
     Case TType of

      1 : Begin                        { 1 = Timed Transition }
           If Direct=Horiz then
              HTimedTranIcon(XPos,YPos,Size,Black,Name)
           Else
              VTimedTranIcon(XPos,YPos,Size,Black,Name);
          end;

      2 : Begin                      { 2 = Customer Generator }
           GeneratorIcon(XPos,YPos,Size,Black,Name);
          end;

      3 : Begin            { 3 = Branching - Selects 1 Output }
           If Direct=Vert then
              VBranchingIcon(XPos,YPos,Size,Black,Name)
           Else
              HBranchingIcon(XPos,YPos,Size,Black,Name);
          end;

      else InhibitorTransition.EraseShape;{ others = standard }
     end{case}
  End;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

  Procedure QueuingTransition.EraseStatus;
  Var s : String;
  Begin
     If Enabled or Firing then Begin
       SetLineStyle(SolidLn, 0,ThickWidth);
       GSetColor(Black);
       Line(XPos - Size shr 2 + 3, YPos,
            XPos, YPos + Size shr 2);
       Line(XPos, YPos + Size shr 2,
            XPos + Size shr 1, YPos - Size shr 2);
       If Firing then Begin
        Str(FireEnd:3:2,s);
        GSetColor(TimCol);
        SetTextStyle(MenuFont,HorizDir,MenuSize);
        SetTextJustify(CenterText,BottomText);
        OutTextXY(Xpos, Ypos - Size shr 1 - 4, s);
       end{if};
     end{if};
  End;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

  Procedure QueuingTransition.TErase;
  Begin
     EraseShape;
     EraseStatus;
  end;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

  Procedure QueuingTransition.StartFire;
  var n:Nodepoint;
      a:PetriAP;            { Arc }
      p:PetriPP;          { Place }
  Begin
      n := TakeFirst(Inlist);
      While n<>nil do Begin
        a := n^.el;
        p := a^.Place;
        If a^.AType=0 then QueuingPP(p)^.AddTokens(-a^.Weight);
        n := TakeNext(n);
      end{while};
      If Mode=5 then begin
         AccBusy.RUpDate(1);
         FireStart := PNTime;
      end;
  End;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

  Procedure QueuingTransition.EndFire;
  var n,m:Nodepoint;
      a:PetriAP;            { Arc }
      p:PetriPP;          { Place }
      arcs,outarc,arc:Integer;
      r:Real;
      stop:Boolean;
  Begin
    If TType<>3 then begin
      n := TakeFirst(Outlist);
      While n<>nil do begin
        a := n^.el;
        p := a^.Place;
        QueuingPP(p)^.AddTokens(a^.Weight);
        n := TakeNext(n);
      end{while};
     end {then}
    Else if Getsize(OutList)>0 then begin          { Branching }
      If Branch=0 then begin    { Try to generate if not given }
        n := TakeFirst(Outlist);
        r := Random;
        stop := false;
        arc := 1;
        While not stop do begin
          a := n^.el;
          If a^.BranchCdf >= r then begin      { Found }
             stop := true;
             Branch := arc;
           end
          else begin
             n := TakeNext(n);
             If n=nil then begin        { Was last arc }
                stop := true;
                If a^.BranchCdf <> 0 then Branch := arc;
              end
             else
                Inc(arc);
          end;
        end;
      end;

      arcs := Getsize(OutList);
      If Branch=0 then outarc := Random(arcs) + 1
      else begin
         outarc := Branch;
         Branch := 0;         { For next firing }
      end;
      n := TakeFirst(Outlist);
      arc := 1;
      While arc<outarc do begin
         n := TakeNext(n);
         Inc(arc)
      end;
      a := n^.el;
      p := a^.Place;
      QueuingPP(p)^.AddTokens(a^.Weight);
    End{if};
    If Mode=5 then begin
      FireTime.RUpdate(PNTime - FireStart); { Updating statistics }
      AccBusy.RUpDate(0);
      Inc(Counter);
    end;
  End;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

  Procedure QueuingTransition.Fire;
  Begin
      StartFire;
      EndFire;
  end;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

 End.
