Unit Simpet5;        { Version 5 - Simulation Environment }
{ This unit's initialisation code is a simulator based on }
{ Petri and Time Networks. Visual Programming. }
{$N+,E+}

Interface        { Everything is private except following: }

  Function  MarkingOf(PName: string) : LongInt;
  Function  PReference(PName: string) : Pointer;
  Function  TReference(TName: string) : Pointer;
  Procedure ChangeMarking(PName: string; M: LongInt; Var OK: Boolean);
  Procedure Terminate;     { Terminate user simulation run }
  Procedure FireDelay(dt:Real);    { Generate Firing delay }
  Function  PNTime : Real;        { Returns the model time }
  Procedure ShowNet; { Displays the P-net in current state }
  Procedure UserExp;                     { User experiment }
  Procedure ShowAllPlTr(Var F:Text);   { Shows All Pl & Tr }
  Procedure ShowAllinTable(Var F: Text); { Shows All Pl&Tr }

  Var Mode   : Byte;        { Simulation mode: 0,1,2,3,4,5 }
      {Timing : Boolean;  { Whether to record tokens timing }
      Confir : Boolean;       { Confirmation = Debugging ? }
      EditingTF : Boolean;       { Screen editing mode T/F }
      MaxTime: Real;                 { Experiment duration }
      TableFormat: Boolean;  { Results in table format T/F }

Implementation

  Uses Dos, Crt, Graph, GGraph, CrtES, EMouseGS, PNList,
     GrEditS, Petri, PNet6, WindowsS, GPNHelp, UserPart, User;


{ GGraph   = General graph }
{ CrtES    = Crt Enhanced }
{ EMouseGS = Enhanced Mouse functions (Graphics only) }
{ PNList   = Two way linked lists (dynamic data structures) }
{ GrEditS  = Graphical editor (objects: polyline, point)    }
{ Petri    = Unit with basic objects: place, transition, arc }
{ User     = User modification of objects, experiment control }
{ PNet6    = Completes declaration of all objects, mainly Net }
{ WindowsS = Windows in graphics }
{ PNHelp1  = Help routines - text mode }
{ GPNHelp  = Context Help routines - graphics }

Const x1q     =  200; y1q     =  150;    { Quit window - Wq }
      x2q     =  400; y2q     =  216;

{********************* Global Variables ********************}

Var  p:          PNp;      { Variables for general use }
     t:          PNt;
     nt:         PN;
     n:          Nodepoint;
     found,OK:   boolean;
     c:          char;
     i,j:        integer;
     Option,Opt: char;          { Menu option/previous }
     MinMode:    integer;     { Variables for graphics }
     MaxMode:    integer;
     ErrCode:    integer;
     F:          Text;                     { Disk file }
     Fname:      string;              { Disk file name }
     DefFName:   String;       { Default Net File Name }
     IORes,b:    word;                      { IOResult }
     Wq:         DWindowP;               { Quit window }
     Finish:     boolean;

{******************* Global Procedures *********************}

  Procedure LogoScr;                          { Logo screen }
  Var k    : real;
      x,y  : word;
      Size : integer;
      c    : char;
  Begin
     If MaxY>250 then Size := 4
                 else Size := 2;
     SetRGBPalette(1,0,0,0);
     SetColor(1);
     SetTextJustify(CenterText,CenterText);
     SetTextStyle(TriplexFont,HorizDir,Size*2);
     OutTextXY(VGAX div 2,VGAY div 8,'Welcome to');
     SetTextStyle(TriplexFont,HorizDir,Size*3);
     OutTextXY(VGAX div 2,2*(VGAY div 5),'PetriSim');
     SetTextStyle(TriplexFont,HorizDir,Size div 2);
     OutTextXY(VGAX div 2,8*(VGAY div 11),
                'Discrete Simulation Environment');
     SetTextStyle(TriplexFont,HorizDir,1);
     OutTextXY(VGAX div 2,11*(VGAY div 13),
     '  (c) jskl 2004                                   Version 5.0');
     OutTextXY(VGAX div 2,12*(VGAY div 13),'Press any key to start');
     Rectangle(0,0,VGAX,VGAY);
     k := 0;
     Repeat        { some show }
       k := k+0.001;
       Delay(40);
       SetRGBPalette(1,round(32+30*sin(14*k)),
                       round(32+30*sin(22*k)),
                       round(32+30*sin(26*k)));
     Until KeyPressed or (MousePosition(x,y)>0);
     If KeyPressed then c := ReadK;
     SetRGBPalette(1,0,0,$FC);   { Back normal blue }
  end;

{***********************************************************}

  Procedure Labels;                                { Labels }
  Var    Size : integer;
  Begin                               { Here are literals ! }
    If MaxY>250 then Size := 4
                else Size := 2;
    If GetMaxColor>1 then begin { not for monochrom }
     SetFillStyle(SolidFill,DarkGray);
     Bar(30,110,VGAX-30,VGAY-50);
     SetFillStyle(SolidFill,Black);
     Bar(100,195,VGAX-80,VGAY-105);
     SetFillStyle(SolidFill,Blue);
     Bar(85,180,VGAX-95,VGAY-120);
    end{if};
     SetColor(Yellow);
     SetTextJustify(CenterText,CenterText);
     SetTextStyle(TriplexFont,HorizDir,Size*2);
     OutTextXY(VGAX div 2,VGAY div 2,'PetriSim');
     SetTextStyle(TriplexFont,HorizDir,Size-2);
     OutTextXY(VGAX div 2,VGAY div 2  + 75,
             'Discrete Simulation Environment');
  End;

{***********************************************************}

   Procedure DrawNet;
   Begin
      ClearDevice;
      If SNet<>nil then Snet^.Show;
   End;

{***********************************************************}

   Function MarkingOf(PName: string) : LongInt;
   {Returns the marking of the place PName, -1 if not found }

   Var n : Nodepoint;
       p : PNp;
       Found : Boolean;

   Begin
     Found := false;
     n := TakeFirst(SNet^.Places);              { Places }
     while (n<>nil) and not Found do begin
        p := n^.el;                   { pointer to place }
        Found := p^.Name=PName;
        n := TakeNext(n);
     end{while};
     If Found then MarkingOf := p^.Marking
              else MarkingOf := -1;
   End;

{***********************************************************}

   Function PReference(PName: string) : Pointer;
   { Returns the reference to the place PName, NIL if not found }

   Var n : Nodepoint;
       p : PNp;
       Found : Boolean;

   Begin
     Found := false;
     n := TakeFirst(SNet^.Places);              { Places }
     while (n<>nil) and not Found do begin
        p := n^.el;                   { pointer to place }
        Found := p^.Name=PName;
        n := TakeNext(n);
     end{while};
     If Found then PReference := p
              else PReference := nil;
   End;

{***********************************************************}

   Function TReference(TName: string) : Pointer;
   { Returns the reference to the transition TName, NIL if not found }

   Var n : Nodepoint;
       t : PNt;
       Found : Boolean;

   Begin
     Found := false;
     n := TakeFirst(SNet^.Transitions);
     while (n<>nil) and not Found do begin
        t := n^.el;                   { pointer to place }
        Found := t^.Name=TName;
        n := TakeNext(n);
     end{while};
     If Found then TReference := t
              else TReference := nil;
   End;

{***********************************************************}

  Procedure ChangeMarking(PName: string; M: LongInt; Var OK: Boolean);
     { Changes marking of PName to M. OK=false if not found }

   Var n : Nodepoint;
       p : PNp;
       Found : Boolean;

   Begin
     Found := false;
     n := TakeFirst(SNet^.Places);              { Places }
     While (n<>nil) and not Found do begin
        p := n^.el;                   { pointer to place }
        Found := p^.Name=PName;
        n := TakeNext(n);
     end{while};
     If Found then begin
         p^.Marking := M;
         OK := true
       end{then}
     else OK := false;
   End;

{***********************************************************}

   Procedure Terminate;     { Terminate user simulation run }
   Begin
      SNet^.Terminate := true
   End;

{***********************************************************}

   Procedure FireDelay(dt:Real);    { Generate Firing delay }
            { Creates an event notice and inserts it to SQS }
   Var t : Real;
       n,nn : PNn;
       Found : Boolean;
   Begin
     If dt<0 then dt := 0;             { Not back in time ! }
     With Snet^ do begin
       FireTran^.Firing := true;
       t := Time + dt;
       FireTran^.FireEnd := t;
       New(n);
       n^.NTrans := FireTran;
       n^.NTime := t;
       n^.NNext := nil;
       If SQS=nil then SQS := n         { So far empty }
       Else If SQS^.NTime>t then begin   { will be 1st }
          n^.NNext := SQS;
          SQS := n
         end{1st}
       Else begin
          nn := SQS; { Searching where to insert: after nn }
          Found := false;
          While not Found do
             If nn^.NNext=nil then begin     { End reached }
                Found := true;
                nn^.NNext := n
              end
             Else If nn^.NNext^.NTime>t then begin { Found }
                     Found := true;
                     n^.NNext := nn^.NNext;
                     nn^.NNext := n
                   end
                  Else nn := nn^.NNext;
       End{If};
     end{with};
   End;

{***********************************************************}

   Function  PNTime : Real;        { Returns the model time }
   Begin
      PNTime := SNet^.Time
   End;

{***********************************************************}

   Procedure ShowNet; { Displays the P-net in current state }
   Var old : Byte;
       s   : String;
   Begin
      SetGraphMode(GrMode);
      DrawNet;
      Str(PNTime:8:3,s);
      If GBandW then begin
        old := CuMeCo;
        CuMeCo := White;
        Wx (15,1,'Time = ');
        Wex(21,1,s);
        CuMeCo := old;
       end
      else begin
        Wx (15,1,'Time = ');
        Wex(21,1,s);
      end{if};
   End;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

   Procedure UserExp;                     { User experiment }
    Var DeadLock : Boolean;
        nn   : PNn;
        EnTr,rt : Integer;
        tr,te: PNt;            { Current,Enabled transition }
        p    : PNp;
        n    : Nodepoint;
        s    : String;
        c    : Char;
        x    : Real;
        FirePri : Word; { Priority of transition(s) to fire }

   Begin
    With SNet^ do begin
     RestoreCrtMode;
     Time := 0;
     SQS  := nil;
     Terminate := false;
     DeadLock  := false;

     { Simulation Initialization of objects: }

     n := TakeFirst(Places);
     while (n<>nil) do begin
        p := n^.el;
        p^.PSimInit;
        n := TakeNext(n);
     end{while};

     n := Takefirst(Transitions);
     While (n<>nil) do begin
       tr := n^.el;
       tr^.TSimInit;
       n := Takenext(n)
     end{while};

     UserInit;             { User experiment initialisation }

     Repeat                               { Simulation loop }

{ Searching for enabled transitions }
       n := Takefirst(Transitions);
       EnTr := 0;
       FirePri := 65535;
       While (n<>nil) do begin
          tr := n^.el;
          If not tr^.Firing then
            If tr^.Enabled then begin
               If tr^.Priority<FirePri then begin { Higher - replace }
                 EnTr := 1;
                 te := tr;
                 FirePri := tr^.Priority;
                end
               else if tr^.Priority=FirePri then begin { Same, add }
	         Inc(EnTr);
	         te:=tr;
               end;            { Ignored if lower priority }
	    end;
          n := Takenext(n)
       end{while};

       If (EnTr=0)and(SQS=nil) then begin        { Deadlock }
          UserDead;
          DeadLock := true;
        end{Deadlock}

       Else If EnTr=0 then begin           { Advancing time }
          Time := SQS^.NTime;
          te := SQS^.NTrans;
          nn := SQS;
          SQS := SQS^.NNext;
          Dispose(nn);
          s := te^.Name;
          te^.Firing := false;
          te^.FireEnd := 0;
          FiringEnds(s,te^.TranNum,te);
          te^.EndFire;                      { Adding tokens }
        end{Advancing}

       Else begin                     { Firing in this time }
          If EnTr>1 then begin { Random selection if more of same priority }
              rt := Random(EnTr) + 1;
              n := Takefirst(Transitions);
              While rt>0 do begin
                 te := n^.el;
                 If not te^.Firing then
                   If te^.Enabled then
		     If te^.Priority=FirePri then Dec(rt);
                 n := Takenext(n)
              end{while};
          end{if};            { te = transition to be fired }

          FireTran := te;
          With te^ do begin
             FireEnd := 0;
             Firing := false;
             s := Name;
             StartFire;                   { Removing tokens }
             FiringStarts(s,TranNum,te);
             If not Firing then begin       { No user delay }
                x := te^.RandDur.RValue;
                If x=0 then begin
                  s := Name;
                  FiringEnds(s,TranNum,te);
                  EndFire;                  { Adding tokens }
                 end
                else
                  FireDelay(x);
             end{if};
          end{with};

        End{deadlock test};

     Until Terminate or DeadLock or KeyPressed;
     If KeyPressed then c:=ReadK;

     {If Terminate then} UserEval;
     { User experiment evaluation (not for deadlock) ??? }

     While SQS<>nil do begin              { Deleting SQS }
        nn := SQS;
        SQS := SQS^.NNext;
        Dispose(nn);
     end{while};

     n := TakeFirst(Places);     { Deleting time records }
     While (n<>nil) do begin
        p := n^.el;
        p^.PSimFinish;
        n := TakeNext(n);
     end{while};
                          { Transitions to normal state: }
     n := Takefirst(Transitions);
     While (n<>nil) do begin
        tr := n^.el;
        tr^.Firing := false;
        tr^.FireEnd := 0;
        n := Takenext(n)
     end{while};
     SetGraphMode(GrMode);
    End{With};
   End{UserExp};

{***********************************************************}

   Procedure SaveOptions;

   Var  F     : text;
        IORes : integer;
        Wi    : DWindowP;
        s     : string;

   Begin
      Assign(F,'PETRISIM.CNF');
      {$I-} Rewrite(F); {$I+}
      IORes := IOResult;
      If IORes>0 then begin
        New(Wi);
        With Wi^ do begin
          WOpen(131,79,461,229,cumeco,CuBkCo,cumeco,CuEmCo);
          If WOK then begin
           Wrt ( 2,1,'Some troubles when writing the file');
           WrtE( 2,3,'PETRISIM.CNF');
           Str(IORes:4,s);
           WrtNEN ( 2,5,'IOResult error code: ',s,'');
           WrtNEN ( 2,6,'Error message: ',ErrorMessage(IORes),'');
           Wrt ( 2,8,'Press any key');
           Wait;
           WClose;
          end{WOK};
        end{with};
        Dispose(Wi);
       end{then}

      else begin                            { OK }
        Writeln(F,DPSize:3,'  Place size');
        Writeln(F,DTSize:3,'  Transition size');
        Writeln(F,DPCol:3 ,'  Place color');
        Writeln(F,DPMCol:3,'  Place marking color');
        Writeln(F,DTCol:3 ,'  Transition color');
        Writeln(F,IACol:3 ,'  Input arc color');
        Writeln(F,OACol:3 ,'  Output arc color');
        Writeln(F,MarCol:3,'  Arc weight color');
        Writeln(F,CuMeCo:3,'  Menu normal color');
        Writeln(F,CuEmCo:3,'  Menu emphasized color');
        Writeln(F,CuPrCo:3,'  Prompt color');
        Writeln(F,HWNTCol:3,'  Help normal color');
        Writeln(F,HWBCol:3 ,'  Help background color');
        Writeln(F,HWETCol:3,'  Help emphasized color');
        Writeln(F,Mode:3,   '  Default Simulation Mode');
        If Selected then
         Writeln(F,SNet^.FName) { Default Net File Name }
        Else
         Writeln(F,'None');
        Writeln(F,'( Default Net File Name )');
        If Confir then Write(F,'T')
                  else Write(F,'F');
	Writeln(F,'  Confirmation (True/False)');
        Writeln(F,MaxTime:12:0,'  Experiment duration');
	Writeln(F,'X  Tokens timing (True/False) Not used');
        Writeln(F,MenuFont:3, '  Screen font');
        Writeln(F,MenuSize:3, '  Screen font size');
        If TableFormat then Write(F,'T')
                  else Write(F,'F');
	Writeln(F,'  Results in table form (True/False)');
        Close(F);
      end{if}
   End;

{***********************************************************}

   Procedure LoadOptions;

   Var  F     : text;
        IORes : integer;
        c     : char;

   Begin
      Assign(F,'PETRISIM.CNF');
      {$I-} Reset(F); {$I+}
      IORes := IOResult;
      If IORes=0 then begin  { File exists, OK }
        Readln(F,DPSize);    { Place size }
        Readln(F,DTSize);    { Transition size }
        Readln(F,DPCol);     { Place color }
        Readln(F,DPMCol);    { Place marking color }
        Readln(F,DTCol);     { Transition color }
        Readln(F,IACol);     { Input arc color }
        Readln(F,OACol);     { Output arc color }
        Readln(F,MarCol);    { Arc weight color }
        Readln(F,CuMeCo);    { Menu normal color }
        Readln(F,CuEmCo);    { Menu emphasized color }
        Readln(F,CuPrCo);    { Prompt color }
        Readln(F,HWNTCol);   { Graphics help normal color }
        Readln(F,HWBCol);    { Graphics help background color }
        Readln(F,HWETCol);   { Graphics help emphasized color }
        Readln(F,Mode);      { Default Simulation Mode }
        ReadLn(F,DefFName);  { Default Net File Name }
        ReadLn(F);           { Comment }
        If not SeekEof(F) then begin  { Nothing more if old }
           ReadLn(F,c);
           Confir := c='T';  { Confirmation T/F }
           ReadLn(F,MaxTime);{ Experiment duration }
           ReadLn(F,c);      { Not used, but kept in CNF file ! }
           {Timing := c='T';  { Tokens timing }
           If not SeekEof(F) then begin  { Nothing more if old }
              ReadLn(F,MenuFont);
              ReadLn(F,MenuSize);
              If not SeekEof(F) then begin  { Nothing more if old }
                 ReadLn(F,c);
                 TableFormat := c='T';
              end;
           end;
        end;
        Close(F);
      end{if};
   End;

{************************************************************}

   Procedure DefaultOptions;
   Begin
     DPsize      := DPSizeI;                 { Default sizes }
     DTsize      := DTSizeI;

     DPcol       := DPcolI;           { Default colors - net }
     DPMcol      := DPMColI;
     DTcol       := DTColI;
     IACol       := IAColI;
     OACol       := OAColI;
     MarCol      := MarColI;
     TimCol      := TimColI;

     CuMeCo      := MNTCol;           { Default colors - menu }
     CuEmCo      := METCol;
     CuPrCo      := PCol;

     HWNTCol := HWNTColI;     { Default colors - context help }
     HWBCol  := HWBColI;
     HWETCol := HWETColI;

     MenuFont := DefaultFont;          { Screen font and size }
     MenuSize := 1;

     Mode    := 1;                  { Default simulation mode }
     DefFName:='None';                { Default Net File Name }
     Confir := false;               { No confirmation default }
     MaxTime := 1000;                   { Experiment duration }
     {Timing := true;                  { Tokens timing default }

     TableFormat := true;                    { Results format }
   End;

{*************************************************************}

    Procedure FirstLoading;

    Var  Wi : DWindowP;
         s  : String;

    Begin
     New(Wi);
     With Wi^ do begin
       WOpen(131,47,461,197,cumeco,CuBkCo,cumeco,CuEmCo);
       If WOK then begin
        Assign(F,DefFName);
        {$I-} Reset(F); {$I+}
        IORes := IOResult;
        If IORes=0 then begin         { File exists, OK }
          Fname := DefFName;
          New(nt);
          nt^.FName := FName;
          Readln(F,nt^.Name);
          WrtNEN (3,4,'Loading the P-net ',nt^.Name,'');
          Wrt (3,6,'Wait please ...');
          New(n); n^.el := nt;
          Inslast(n,Nets);
          Snet := nt;
          Selected := true;
          Snet^.Load(F);
          Close(F);
          WClose;
          DrawNet;
        end{then}

       else if IORes=2 then begin      { File not found }
          Wrt (3,1,'The default file '); WrtE(3,2,DefFName);
          Wrt (3,3,'not found !');
          Wrt (3,8,'Press any key');
          Wait;
          WClose;
        end{then}

       else begin                      { Other problems }
        Wrt ( 2,1,'Some troubles with the default file');
        WrtE( 2,3,DefFName);
        Str (IORes:4,s);
        WrtNEN ( 2,5,'IOResult error code: ',s,'');
        WrtNEN ( 2,6,'Error message: ',ErrorMessage(IORes),'');
        Wrt ( 2,8,'Press any key');
        Wait;
        WClose;
       end{if};
      end{WOK};
     end{with};
     Dispose(Wi);
    End{FirstLoading};

{*************************************************************}

  Function Menu1(OldOp,MOut:char):char;         { Main menu }

  Const
     Menu1GA:     { Positions and options: data present }
     array[1..7] of PositionG =
                 (( x1:  0; y1:  1; x2: 48; y2: 16; opt:'E' ),
                  ( x1: 48; y1:  1; x2:128; y2: 16; opt:'S' ),
                  ( x1:128; y1:  1; x2:176; y2: 16; opt:'F' ),
                  ( x1:176; y1:  1; x2:272; y2: 16; opt:'M' ),
                  ( x1:272; y1:  1; x2:320; y2: 16; opt:'H' ),
                  ( x1:320; y1:  1; x2:368; y2: 16; opt:'Q' ),
                  ( x1:606; y1:  1; x2:638; y2: 16; opt:KF1 ));

     Menu1GA1:    { Positions and options: no data }
     array[1..7] of PositionG =
                (( x1:9999; y1:  0; x2: 48; y2: 15; opt:'E' ),
                 ( x1:9999; y1:  0; x2:128; y2: 15; opt:'S' ),
                  ( x1:128; y1:  0; x2:176; y2: 15; opt:'F' ),
                  ( x1:176; y1:  0; x2:272; y2: 15; opt:'M' ),
                  ( x1:272; y1:  0; x2:320; y2: 15; opt:'H' ),
                  ( x1:320; y1:  0; x2:368; y2: 15; opt:'Q' ),
                  ( x1:607; y1:  0; x2:639; y2: 15; opt:KF1 ));

  Begin
     MainTitles(true);
     WaitUp;
     MouseCursorOn;
     PullDown := false;
     If Selected then
      Menu1 := KeyOrMouseG(['E','S','F','M','H','Q',KF1],
                           Menu1GA,OldOp,MOut)
     else
      Menu1 := KeyOrMouseG(['F','M','H','Q',KF1],Menu1GA1,OldOp,MOut);
     PullDown := true;
     MouseCursorOff;
     MainTitles(false);
  End;

{***********************************************************}

  Procedure Files;                            { File option }

    Const  x1 =  128;   x2 =  217;       { File window - Wn }
           y1 =   23;   y2 =  119;

    Var Wn      :PWindowP;
        Opt     :char;
        PnOK    :boolean;
        OldOp,MOut : char;

{-----------------------------------------------------------}

    Function FileMenu(OldOp,MOut:char):char;
                                                { File menu }
    Const
     FileMenuGA:     { Positions and options: data read }
     array[1..5] of PositionG =
                 (( x1:131; y1: 31; x2:214; y2: 47; opt:'L' ),
                  ( x1:131; y1: 47; x2:214; y2: 63; opt:'C' ),
                  ( x1:131; y1: 63; x2:214; y2: 79; opt:'S' ),
                  ( x1:131; y1: 79; x2:214; y2: 95; opt:'P' ),
                  ( x1:131; y1: 95; x2:214; y2:111; opt:'D' ));

     FileMenuGA1:    { Positions and options: no data }
     array[1..5] of PositionG =
                 (( x1:131; y1: 31; x2:214; y2: 47; opt:'L' ),
                  ( x1:131; y1: 47; x2:214; y2: 63; opt:'C' ),
                 ( x1:9999; y1: 63; x2:214; y2: 79; opt:'S' ),
                 ( x1:9999; y1: 79; x2:214; y2: 95; opt:'P' ),
                  ( x1:131; y1: 95; x2:214; y2:111; opt:'D' ));

    Begin
    With Wn^ do begin
     WClear;
     WrtNEN(1,1,'','L','oad');
     WrtNEN(1,2,'','C','reate');
     If Selected then WrtNEN(1,3,'','S','ave')
                 else Wrt   (1,3,'Save');
     If Selected then WrtNEN(1,4,'','P','rint')
                 else Wrt   (1,4,'Print');
     WrtNEN(1,5,'','D','OS Shell');
     WaitUp;
     MouseCursorOn;
     If Selected then
      FileMenu := KeyOrMouseG(['C','L','S','P','D']
                              ,FileMenuGA,OldOp,MOut)
     else
      FileMenu := KeyOrMouseG(['C','L','D']
                              ,FileMenuGA1,OldOp,MOut);
     MouseCursorOff;
     WClear;
     Wrt(1,1,'Load');
     Wrt(1,2,'Create');
     Wrt(1,3,'Save');
     Wrt(1,4,'Print');
     Wrt(1,5,'DOS Shell');
    end{with};
    End;

{-----------------------------------------------------------}

    Procedure Printing;

    Const  x1 =  131;  x2 =  420;         { Printing window }
           y1 =   95;  y2 =  160;

           x1b =  131; x2b =  250;    { B&W or Color window }
           y1b =   95; y2b =  143;

    Var  W   :PWindowP;
         Wbc :PWindowP;
         Op  :char;

    {-------------------------------------------------------}

    Function MenuBC:char;                { Menu B&W/Color }
    Const
     MenuBCGA:      { Positions and options for this menu }
     array[1..2] of PositionG =
                 (( x1: 134; y1:103; x2:247; y2:119; opt:'B' ),
                  ( x1: 134; y1:119; x2:247; y2:135; opt:'C' ));
    begin
    With Wbc^ do begin
     WrtNEN(1,1,'','B','lack & white');
     WrtNEN(1,2,'','C','olor');
     WaitUp;
     MouseCursorOn;
      MenuBC := KeyOrMouseG(['B','C'],MenuBCGA,'C',KEsc);
     MouseCursorOff;
    end{with}
    end;

    {------------------- Printing.body ----------------------}

   Begin
    New(Wbc);
    Wbc^.WOpen(x1b,y1b,x2b,y2b,cumeco,CuBkCo,cumeco,CuEmCo);
    If Wbc^.WOK then begin
     Op := MenuBC;
     Wbc^.WClose;
    end{WOK};
    Dispose(Wbc);

    If Op in ['B','C'] then begin    { Not for Esc }
     New(W);
     With W^ do begin
      WOpen(x1,y1,x2,y2,cumeco,CuBkCo,cumeco,CuEmCo);
      If WOK then begin
        WrtE(2,1,'1. Press any key');
        WrtE(2,2,'2. Activate TSR (Print-Screen)');
        WrtE(2,3,'3. Press any key (when ready)');
        WaitUp;
        Wait;
        If Op='B' then GBandW := true;
        DrawNet;
        Wait;
        GBandW := false;
        WClose;
      end{WOK};
     end{with};
     Dispose(W);
    end{if};
   End{Printing};

{-----------------------------------------------------------}

   Procedure Loading;

    Var  Wi   :DWindowP;
         OK   :boolean;
         s,r,z :string;

   Begin
    New(Wi);
    With Wi^ do begin
     WOpen(131,47,461,197,cumeco,CuBkCo,cumeco,CuEmCo);
     If WOK then begin
      Wrt (3,2,'Please enter the file name'); WrtE(3,4,':');
      WaitUp;
      WRS (5,4,s);
      r := s;
      WClear;
      If s='' then { Write(#7) }
      else begin
       Assign(F,r);
       {$I-} Reset(F); {$I+}
       IORes := IOResult;
       If IORes=0 then begin            { File exists, OK }
          Fname := r;
          Wrt (3,6,'Wait please ...');
          New(nt);
          nt^.Name  := '';
          nt^.FName := FName;
          Readln(F,s);
          r := s;
          Upper(s);
          n := Takefirst(Nets);
          found := false;
          While (n<>nil) and not found do begin
             Net := n^.el;
             z := Net^.Name;
             Upper(z);
             found := z=s;
             n := Takenext(n)
          end{while};
          OK := true;
          If found then begin
               WClear;
               WrtNEN (3,1,'P-net ',r,'');
               WrtNEN (3,2,'to be read from ',Fname,'');
               Wrt (3,3,'already exists !!');
               WrtNEN(3,5,'','C/Left',' to change the name');
               WrtNEN(3,7,'','A/Right',' to abandon the operation');
               Repeat
                  c := UpCase(CWait(b));
               Until (c in ['C','A',KEsc]) or (b>0);
               If b>1 then c := 'A';
               If b=1 then c := 'C';
               If c='C' then
                 Repeat
                  WClear;
                  Repeat
                    Wrt(3,2,'Enter the new P-net name:'); WrtE(3,4,':');
                    WRSDOS(5,4,s);
                    {If s='' then Write(#7);}
                    WClear;
                  Until s<>'';            { Name must be given }
                  s := Copy(s,1,8);
                  nt^.Name := s;
                  r := s;
                  Upper(s);
                  n := Takefirst(Nets);
                  found := false;
                  While (n<>nil) and not found do begin
                    net := n^.el;
                    z := Net^.Name;
                    Upper(z);
                    found := z=s;
                    n := Takenext(n)
                  end{while};
                  If found then begin
                    WrtNEN (3,2,'P-net ',r,'');
                    Wrt (3,3,'already exists !');
                    WrtE(3,5,'Change the name');
                    Wrt (3,7,'Press any key'); Wait;
                  end;
                 until not found
               else begin
                  OK := false;
                  Dispose(nt);
               end{if};
            end{then}
          else
               nt^.Name := r;
          {endif};
             If OK then begin        { OK - loading }
               WClear;
               WrtNEN (3,4,'Loading the P-net ',nt^.Name,'');
               Wrt (3,6,'Wait please ...');
               New(n); n^.el := nt;
               Inslast(n,Nets);
               Snet := nt;
               Selected := true;
               Snet^.Load(F);
               Close(F);
             end{if}
       end{then}

      else if IORes=2 then begin           { File not found }
        WrtNEN (3,1,'File ',r,'');
        Wrt (3,2,'not found !');
        Wrt (3,8,'Press any key');
        Wait;
       end{then}

      else begin                           { Other problems }
        Wrt ( 2,1,'Some troubles when accessing the file');
        WrtE( 2,3,r);
        Str (IORes:4,s);
        WrtNEN ( 2,5,'IOResult error code: ',s,'');
        WrtNEN ( 2,6,'Error message: ',ErrorMessage(IORes),'');
        Wrt ( 2,8,'Press any key');
        Wait;
      end{if};
      end{if};
      WClose;
     end{WOK};
    end{with};
    Dispose(Wi);
   End{Loading};

{-----------------------------------------------------------}

    Procedure Saving;

    Const  x1b =  131; x2b =  250;     { All/Visible window }
           y1b =   79; y2b =  127;

    Var  Wav :PWindowP;
         Op  :char;
         Wi  :DWindowP;
         c   :char;
         b   :word;
         OK  :boolean;
         s   :string;

    {-------------------------------------------------------}

    Function MenuAV:char;                { Menu All/Visible }
    Const
     MenuBCGA:      { Positions and options for this menu }
     array[1..2] of PositionG =
                 (( x1: 134; y1: 87; x2:247; y2:103; opt:'A' ),
                  ( x1: 134; y1:103; x2:247; y2:119; opt:'V' ));
    begin
    With Wav^ do begin
     WrtNEN(1,1,'Save ','A','ll');
     WrtNEN(1,2,'Save ','V','isible');
     WaitUp;
     MouseCursorOn;
      MenuAV := KeyOrMouseG(['A','V'],MenuBCGA,'A',KEsc);
     MouseCursorOff;
    end{with}
    end;

    {------------------- Saving.body ----------------------}

   Begin
    New(Wav);
    Wav^.WOpen(x1b,y1b,x2b,y2b,cumeco,CuBkCo,cumeco,CuEmCo);
    If Wav^.WOK then begin
     Op := MenuAV;
     Wav^.WClose;
    end{WOK};
    Dispose(Wav);

    If Op in ['A','V'] then begin    { Not for Esc }

     c := ' '; OK := false;
     s := FName;
     New(Wi);
     With Wi^ do begin
      WOpen(131,79,461,229,cumeco,CuBkCo,cumeco,CuEmCo);
      If WOK then begin
       If s<>'' then begin       { Edit file name known }
        Wrt (3,1,'Saving the network to the file');
        WrtE(3,3,s);
        Wrt (3,4,'Do you agree ?');
        WrtNEN (3,6,'Press ','Y/Left',' to confirm');
        WrtNEN (3,7,'Press ','C/Right',' to change name');
        WrtNEN (3,8,'Press ','Esc',' to quit');
        WaitUp;
        Repeat
          c := UpCase(CWait(b));
        Until (c in ['Y','C',KEsc]) or (b>0);
        If b>1 then c := 'C';
        If b=1 then c := 'Y';
        WClear;
        end{then}
       else
        c := 'C';

      If c='C' then begin              { Reading file name }
        Wrt (3,2,'Please enter the file name:'); WrtE(3,4,':');
        WaitUp;
        WRS (5,4,s);
        If s='' then begin
         { Write(#7); }  c := KEsc;
        end;
        WClear;
      end{if};

    If c<>KEsc then begin       { Testing the file: }
      Assign(F,s);
      {$I-} Reset(F); {$I+}
      IORes := IOResult;

      If IORes=0 then begin                   { File exists }
         WrtNEN (3,2,'The file ',s,'');
         Wrt (3,3,'exists !');
         WrtNEN (3,6,'Press ','O/Left',' to overwrite');
         WrtNEN (3,7,'Press ','A/Right',' to abandon');
         Repeat
            c := UpCase(CWait(b));
         Until (c in ['O','A',KEsc]) or (b>0);
         If b>1 then c := 'A';
         If b=1 then c := 'O';
         OK := c='O';
         WClear;
        end{then}

       else if IORes=2 then OK := true                 { OK }

       else begin                          { Other problems }
         Wrt ( 2,1,'Some troubles when accessing the file');
         WrtE( 2,3,s);
         Str(IORes:4,s);
         WrtNEN ( 2,5,'IOResult error code: ',s,'');
         WrtNEN ( 2,6,'Error message: ',ErrorMessage(IORes),'');
         Wrt ( 2,8,'Press any key');
         Wait;
       end{if};
      end{if};

      If OK then begin                { KEsc = not OK }
         {$I-} Rewrite(F); {$I+}
         IORes := IOResult;
         If IORes>0 then begin
           Wrt ( 2,1,'Some troubles when writing the file');
           WrtE( 2,3,s);
           Str(IORes:4,s);
           WrtNEN ( 2,5,'IOResult error code: ',s,'');
           WrtNEN ( 2,6,'Error message: ',ErrorMessage(IORes),'');
           Wrt ( 2,8,'Press any key');
           Wait;
          end{then}

         else begin
            FName := s;                     { OK }
            SNet^.FName := FName;
            WrtNEN (3,3,'Saving the P-net ',Snet^.Name,'');
            WrtNEN (3,5,'to the file ',FName,'');
            Wrt (3,7,'Wait please ...');
            Snet^.Store(F,Op);
            Close(F);
         end{if}
       end{if};
       WClose;
      end{WOK};
     end{with};
     Dispose(Wi);
    End;
   End{Saving};

{---------------------  Files.body  -----------------------}

   Begin
    New(Wn);
    OldOp := 'L';
    MOut := KEsc;
    Wn^.WOpen(x1,y1,x2,y2,cumeco,CuBkCo,cumeco,CuEmCo);
    If Wn^.WOK then begin
     Repeat
      Opt := FileMenu(OldOp,MOut);
      OldOp := Opt;
      Case Opt of

      'C': begin            { Create and select a new P-net }
            New(Net);
            Net^.Create(PNOK);
            IF PNOK then begin
              New(n); n^.el := Net;
              Inslast(n,Nets);
              SNet := Net;
              Selected := true;
              FName := SNet^.FName;
              Wn^.WClose;
              DrawNet;
              MainTitles(false);
              Wn^.WOpen(x1,y1,x2,y2,cumeco,CuBkCo,cumeco,CuEmCo);
             end
            else Dispose(Net);
            OldOp := 'C';
        end{C};

      'L': begin         { Load edited P-net from text file }
            Loading;
            Wn^.WClose;
            If Selected then DrawNet else Labels;
            MainTitles(false);
            Wn^.WOpen(x1,y1,x2,y2,cumeco,CuBkCo,cumeco,CuEmCo);
            OldOp := 'L';
        end{L};

      'S': begin            { Save edited data to text file }
            Saving;
            OldOp := 'S';
        end{S};

      'P': begin                                 { Printing }
            Printing;
            Wn^.WClose;
            If Selected then DrawNet else Labels;
            MainTitles(false);
            Wn^.WOpen(x1,y1,x2,y2,cumeco,CuBkCo,cumeco,CuEmCo);
            OldOp := 'P';
        end{P};

      'D': begin                                { DOS Shell }

      { Maximum heap when compiling cca. 350 000! }
      { Works only when started from DOS (.EXE)   }

         Wn^.WClose;
         RestoreCrtMode;
         TextColor(LightGray);
         TextBackground(Black);
         ClrScr;
         Writeln('Type EXIT to return back to PetriSim.');
         GoToXY(1,4);
         Exec(CommandName,'');
         If DosError <> 0 then begin
              Write('Dos error #', DosError,': ');
              Writeln(ErrorMessage(DosError)); Writeln;
              Write('Press any key to continue');
              Wait;
         end{if};
         SetGraphMode(GrMode);
         OK := MouseInit;            { Mouse reinitialization }
         If Selected then DrawNet else Labels;
         MainTitles(false);
         Wn^.WOpen(x1,y1,x2,y2,cumeco,CuBkCo,cumeco,CuEmCo);
         OldOp := 'D';
        end{D};

      end{case};
     Until Opt in [KEsc,KLeft,KRight];
     Wn^.WClose;
     MenuOut1 := Opt;
    end{WOK};
    Dispose(Wn);
  End{Files};

{***********************************************************}

  Procedure Management;                 { Management option }

    Const  x1 =  176;   x2 =  288;            { Window - Wn }
           y1 =   23;   y2 =  135;

    Var Wn      :PWindowP;
        Wram    :DWindowP;
        MOpt    :char;
        s       :string;
        OldOp,MOut : char;

{-----------------------------------------------------------}

    Function ManMenu(OldOp,MOut:char):char;
                                          { Management menu }
    Const
     ManMenuGA:      { Positions and options: }
     array[1..6] of PositionG =
                 (( x1:179; y1: 31; x2:285; y2: 47; opt:'L' ),
                  ( x1:179; y1: 47; x2:285; y2: 63; opt:'S' ),
                  ( x1:179; y1: 63; x2:285; y2: 79; opt:'N' ),
                  ( x1:179; y1: 79; x2:285; y2: 95; opt:'D' ),
                  ( x1:179; y1: 95; x2:285; y2:111; opt:'O' ),
                  ( x1:179; y1:111; x2:285; y2:127; opt:'R' ));

     ManMenuGA1:     { Positions and options - no nets: }
     array[1..6] of PositionG =
                 (( x1:179; y1: 31; x2:285; y2: 47; opt:'L' ),
                  ( x1:9999; y1: 47; x2:285; y2: 63; opt:'S' ),
                  ( x1:9999; y1: 63; x2:285; y2: 79; opt:'N' ),
                  ( x1:9999; y1: 79; x2:285; y2: 95; opt:'D' ),
                  ( x1:179; y1: 95; x2:285; y2:111; opt:'O' ),
                  ( x1:179; y1:111; x2:285; y2:127; opt:'R' ));

    Begin
    With Wn^ do begin
     WClear;
     WrtNEN(1,1,'','L','ist of nets');
     If Selected then WrtNEN(1,2,'','S','elect')
                 else Wrt   (1,2,'Select');
     If Selected then WrtNEN(1,3,'Re','n','ame')
                 else Wrt   (1,3,'Rename');
     If Selected then WrtNEN(1,4,'','D','elete')
                 else Wrt   (1,4,'Delete');
     WrtNEN(1,5,'','O','ptions');
     WrtNEN(1,6,'Free ','R','AM');
     WaitUp;
     MouseCursorOn;
     If Selected then
       ManMenu := KeyOrMouseG(['L','S','N','D','O','R'],
                              ManMenuGA,OldOp,MOut)
     else
       ManMenu := KeyOrMouseG(['L','O','R'],ManMenuGA1,OldOp,MOut);
     MouseCursorOff;
     WClear;
     Wrt(1,1,'List of nets');
     Wrt(1,2,'Select');
     Wrt(1,3,'Rename');
     Wrt(1,4,'Delete');
     Wrt(1,5,'Options');
     Wrt(1,6,'Free RAM');
    end{with};
    End;

{-----------------------------------------------------------}

   Procedure Listing(x11,y11: integer);
   { Displaying, Selecting and Renaming P-nets.
     x11,y11 = window coordinates. }

    Const
     MenuList:  { Positions and options (to be filled in) }
     array[1..12] of PositionG =
                 (( x1:0; y1:0; x2:0; y2:0; opt:'0' ),
                  ( x1:0; y1:0; x2:0; y2:0; opt:'1' ),
                  ( x1:0; y1:0; x2:0; y2:0; opt:'2' ),
                  ( x1:0; y1:0; x2:0; y2:0; opt:'3' ),
                  ( x1:0; y1:0; x2:0; y2:0; opt:'4' ),
                  ( x1:0; y1:0; x2:0; y2:0; opt:'5' ),
                  ( x1:0; y1:0; x2:0; y2:0; opt:'6' ),
                  ( x1:0; y1:0; x2:0; y2:0; opt:'7' ),
                  ( x1:0; y1:0; x2:0; y2:0; opt:'8' ),
                  ( x1:0; y1:0; x2:0; y2:0; opt:'9' ),
                  ( x1:0; y1:0; x2:0; y2:0; opt:KPgDn ),
                  ( x1:0; y1:0; x2:0; y2:0; opt:KPgUp ));

    Var   Wnl,Wrn : DwindowP;
          line,pos : byte;
          i,Number  : integer;
          Done,First,Last : boolean;
          c  : char;
          s,z,r  :string;
          n,m : Nodepoint;
          cset : set of char;
          OldOp,MOut : char;
          Nt : PN;
          NetArray : array[0..9] of PN;
          NodArray : array[0..9] of Nodepoint;

   Begin
     Case MOpt of
     'L': SPrompt := 'Listing of P-nets in RAM';
     'S': SPrompt := 'Select the active P-net';
     'N': SPrompt := 'Change the name of a P-net';
     'D': SPrompt := 'Delete a P-net from RAM';
     end{case};
     DPrompt;
     Done := false;
     New(Wnl);
     With Wnl^ do begin
      WOpen(x11,y11,x11+147,y11+260,CuMeCo,CuBkCo,CuMeCo,CuEmCo);
      If WOK then begin
       n := TakeFirst(Nets);
       Repeat
        Wrt(2,1,'LIST OF P-NETS');
        First := n=TakeFirst(Nets);
        Last := true;
        Number := 0;
        cset := [];
        For i:=0 to 9 do begin
           With MenuList[i+1] do begin
            If n=nil then x1 := GetMaxX
            else begin                  { Net exists }
             If MOpt='L' then x1 := GetMaxX
                         else x1 := x11+7;
             y1 := y11+40 + i*16;
             x2 := x11+140;
             y2 := y11+56 + i*16;
             cset := cset + [Chr(Ord('0')+i)];
             Nt := n^.el;
             NetArray[i] := Nt;
             NodArray[i] := n;
             If Nt=Snet then
               WrtE(2,i+3,Nt^.Name)
             else
               Wrt (2,i+3,Nt^.Name);
             WrtE(15,i+3,Chr(Ord('0')+i));
             m := n; Inc(Number);
             n := TakeNext(n);
             Last := n=nil;
            end{net exists};
           end{with};
        end{for};
        i := 11;
        MenuList[12].x1 := GetMaxX;
        With MenuList[i] do begin
             If Last then x1 := GetMaxX
             else begin
                cset := cset + [KPgDn];
                x1 := x11+7;
                Wrt ( 2,i+3,'Next');
                WrtE(12,i+3,'PgDn');
                y1 := y11+40 + i*16;
                x2 := x11+140;
                y2 := y11+56 + i*16;
                opt := KPgDn;
                Inc(i);
             end{if};
        end{with};
        With MenuList[i] do begin
             If First then x1 := GetMaxX
             else begin
                cset := cset + [KPgUp];
                x1 := x11+7;
                Wrt ( 2,i+3,'Previous');
                WrtE(12,i+3,'PgUp');
                y1 := y11+40 + i*16;
                x2 := x11+140;
                y2 := y11+56 + i*16;
                opt := KPgUp;
             end{if};
        end{with};
        OldOp := '0'; MOut := KEsc;
        WaitUp;

        If First and Last and (MOpt='L') then begin
           Wrt(2,15,'Press any key');
           Wait;
           c := KEsc;
          end
        else begin
          If (MOpt='L') then
             If not Last then OldOp := KPgDn
                         else OldOp := KPgUp;
          MouseCursorOn;
          c := KeyOrMouseG(cset,MenuList,OldOp,MOut);
          MouseCursorOff;
        end{if};

        Case c of

        KPgDn : begin          { PgDn }
           WClear;
        end{PgDn};

        KPgUp : begin          { PgUp }
           n := m;
           For i:=1 to Number+9 do n := n^.pred;
           WClear;
        end{PgUp};

        KEsc : WClose;         { KEsc }

        Else begin             { Not PgDn, PgUp, KEsc }
           Done := true;
           Case MOpt of

           'S': begin                              { Select }
                 SNet := NetArray[Ord(c)-Ord('0')];
                 FName := SNet^.FName;
                 WClose;
                 Wn^.WClose;
                 DrawNet;
                 MainTitles(false);
                 Wn^.WOpen(x1,y1,x2,y2,cumeco,CuBkCo,cumeco,CuEmCo);
                end;

           'N': begin                              { Rename }

    New(Wrn);
    With Wrn^ do begin
     Net := NetArray[Ord(c)-Ord('0')];
     WOpen(x11+7,y11+7,x11+207,y11+140,cumeco,CuBkCo,cumeco,CuEmCo);
     If WOK then begin
      Wrt (2,2,'Enter the new name');
      WrtNEN (2,3,'of the P-net ',Net^.Name,'');
      WrtE(2,5,':');
      WaitUp;
      WRSDOS (4,5,s);
      Wrub (4,5,s);
      If s='' then begin
        { Write(#7); }
        WClose;
        Wnl^.WClose;
       end
      else begin
       s := Copy(s,1,8);
       WrtE(4,5,s);
       r := s;
       Upper(s);
       n := Takefirst(Nets);
       found := false;
       While (n<>nil) and not found do begin
         Nt := n^.el;
         z := Nt^.Name;
         Upper(z);
         found := (z=s) and (Nt<>Net);
         n := Takenext(n)
       end{while};
       If found then begin
         WClear;
         WrtNEN (2,2,'P-net ',r,'');
         Wrt (2,3,'already exists.');
         WrtE(2,4,'Use other name !');
         Wrt (2,7,'Press any key'); WaitUp; Wait;
         WClose;
         Wnl^.WClose;
        end{then}
       else begin
         Net^.Name  := r;
         { Net^.FName := r;    file name not changed ! }
         WClose;
         Wnl^.WClose;
         If Net=Snet then begin
            Wn^.WClose;
            DrawNet;
            MainTitles(false);
            Wn^.WOpen(x1,y1,x2,y2,cumeco,CuBkCo,cumeco,CuEmCo);
         end{if};
        end{if};
       end{if};
      end{WOK};
     end{with};
     Dispose(Wrn);

                end{N};

          'D': begin                              { Delete }
                Net := NetArray[Ord(c)-Ord('0')];
                n   := NodArray[Ord(c)-Ord('0')];
                New(Wq);
                With Wq^ do begin
                   WOpen(x1q,y1q,x2q,y2q,
                          LightRed,CuBkCo,LightRed,Yellow);
                   If WOK then begin
                    Wrt (2,1,'You are deleting');
                    WrtNEN (2,2,'the P-net ',Net^.Name,'');
                    WrtNEN (2,3,'Confirm:        (','Y','/N)');
                    c:=CWait(b);
                    WClose;
                   end{WOK};
                end{with};
                Dispose(Wq);
                If c in ['Y','y'] then begin
                 Net^.Destroy;
                 Dispose(Net);
                 Remove(n,Nets);
                 Dispose(n);
                 WClose;
                 If SNet=Net then    { Selected destroyed ! }
                  If Empty(Nets) then begin
                    SNet := nil;
                    Selected    := false;
                    Wn^.WClose;
                    ClearDevice;
                    Labels;
                    MainTitles(false);
                    Wn^.WOpen(x1,y1,x2,y2,cumeco,CuBkCo,cumeco,CuEmCo);
                   end
                  else begin      { First becomes selected }
                    n := TakeFirst(Nets);
                    SNet := n^.el;
                    FName := SNet^.FName;
                    Wn^.WClose;
                    DrawNet;
                    MainTitles(false);
                    Wn^.WOpen(x1,y1,x2,y2,cumeco,CuBkCo,cumeco,CuEmCo);
                 end{if};
                end{then}
               else WClose;
              end{D};

           End{case-opt};
        end;

        End{case-c};

      Until Done or (c=KEsc);
      end{WOK};
     end{with};
     Dispose(Wnl);
     EPrompt;
   End{Listing};

{-----------------------------------------------------------}

   Procedure Options;                      { Global options }

    Const  x1 =  179;   x2 =  317;    { Options window - Wo }
           y1 =  111;   y2 =  221;

           x1c =  182;  x2c =  364;    { Colors window - Wc }
           y1c =  165;  y2c =  373;

           Menu3GAF:        { Positions and options for Font menu }
           array[1..3] of PositionG =
                  (( x1: 185; y1:189; x2:315; y2:205; opt:'D' ),
                   ( x1: 185; y1:205; x2:315; y2:221; opt:'S' ),
                   ( x1: 185; y1:221; x2:315; y2:237; opt:'V' ));

    Var   Wo,Wc,Wrr :PwindowP;
          Wd   :DwindowP;
          c,d  :char;
          OK   :boolean;
          i    :LongInt;
          s,z  :string;
          OldOp,MOut,OldOp1,MOut1 : char;

  {.........................................................}

  Function MenuOpt(OldOp,MOut:char):char;
                                             { Options menu }

  Const
     MenuOptGA:     { Positions and options: }
     array[1..6] of PositionG =
                 (( x1:182; y1:120; x2:314; y2:136; opt:'P' ),
                  ( x1:182; y1:136; x2:314; y2:152; opt:'T' ),
                  ( x1:182; y1:152; x2:314; y2:168; opt:'C' ),
                  ( x1:182; y1:168; x2:314; y2:184; opt:'F' ),
                  ( x1:182; y1:184; x2:314; y2:200; opt:'S' ),
                  ( x1:182; y1:200; x2:314; y2:216; opt:'D' ));
  Begin
    With Wo^ do begin
     WClear;
     WrtNEN(1,1,'','P','lace size');
     WrtNEN(1,2,'','T','ransition size');
     WrtNEN(1,3,'','C','olors');
     WrtNEN(1,4,'','F','ont');
     WrtNEN(1,5,'','S','ave options');
     WrtNEN(1,6,'','D','efault options');
     WaitUp;
     MouseCursorOn;
      MenuOpt := KeyOrMouseG(['P','T','C','F','S','D'],
                             MenuOptGA,OldOp,MOut);
     MouseCursorOff;
     WClear;
     Wrt(1,1,'Place size');
     Wrt(1,2,'Transition size');
     Wrt(1,3,'Colors');
     Wrt(1,4,'Font');
     Wrt(1,5,'Save options');
     Wrt(1,6,'Default options');
    end{with};
  End;

  {.........................................................}

  Function MenuCol(OldOp,MOut:char):char;
                                              { Colors menu }

  Const
     MenuColGA:     { Positions and options: }
     array[1..12] of PositionG =
                 (( x1:185; y1:173; x2:361; y2:189; opt:'P' ),
                  ( x1:185; y1:189; x2:361; y2:205; opt:'A' ),
                  ( x1:185; y1:205; x2:361; y2:221; opt:'T' ),
                  ( x1:185; y1:221; x2:361; y2:237; opt:'I' ),
                  ( x1:185; y1:237; x2:361; y2:253; opt:'O' ),
                  ( x1:185; y1:253; x2:361; y2:269; opt:'W' ),
                  ( x1:185; y1:269; x2:361; y2:285; opt:'N' ),
                  ( x1:185; y1:285; x2:361; y2:301; opt:'E' ),
                  ( x1:185; y1:301; x2:361; y2:317; opt:'R' ),
                  ( x1:185; y1:317; x2:361; y2:333; opt:'H' ),
                  ( x1:185; y1:333; x2:361; y2:349; opt:'M' ),
                  ( x1:185; y1:349; x2:361; y2:365; opt:'B' ));
  Begin
    With Wc^ do begin
     WrtNEN (1,1,'','P','lace color');
     WrtNEN (1,2,'M','a','rking color');
     WrtNEN (1,3,'','T','ransition color');
     WrtNEN (1,4,'','I','nput arc color');
     WrtNEN (1,5,'','O','utput arc color');
     WrtNEN (1,6,'Arc ','w','eight color');
     WrtNEN (1,7,'Menu ','n','ormal color');
     WrtNEN (1,8,'Menu ','e','mphasized color');
     WrtNEN (1,9,'P','r','ompt color');
     WrtNEN (1,10,'','H','elp normal color');
     WrtNEN (1,11,'Help e','m','phasized color');
     WrtNEN (1,12,'Help ','b','ackground color');
     WaitUp;
     MouseCursorOn;
      MenuCol := KeyOrMouseG(['P','A','T','I','O','W','N','E',
                              'R','H','M','B'],MenuColGA,OldOp,MOut);
     MouseCursorOff;
     { Not necessary to switch off - covered ! }
    end{with};
  End;

  {................... Options.body .....................}

  Begin
    OldOp := 'P';
    MOut := KEsc;
    New(Wo);
    Wo^.WOpen(x1,y1,x2,y2,cumeco,CuBkCo,cumeco,CuEmCo);
    If Wo^.WOK then begin
     New(Wd);
     With Wd^ do begin
      Repeat
       c := MenuOpt(OldOp,MOut);
       Case c of

       'P': begin                             { Place Size }
           WOpen(182,135,386,202,cumeco,CuBkCo,cumeco,CuEmCo);
           If WOK then begin
            Repeat
              Str(DPSize:4,s);
              WrtNEN (1,1,'Place size =',s,'');
              Str(DPSizeI:4,s);
              WrtNEN (1,2,'Default value =',s,'');
              i := DPSize;
              Wrt (1,3,'Enter size'); WrtE(11,3,':');
              WaitUp;
              WRI(13,3,i,OK);
              OK := (i>0) and (i<VGAY div 2);
              {If not OK then Write(#7);}
              WClear;
            Until OK;
            DPSize := i;
            WClose;
           end{WOK};
           OldOp := 'P';
        end{P};

       'T': begin                         { Transition Size }
           WOpen(182,151,386,218,cumeco,CuBkCo,cumeco,CuEmCo);
           If WOK then begin
            Repeat
              Str(DTSize:4,s);
              WrtNEN (1,1,'Transition size =',s,'');
              Str(DTSizeI:4,s);
              WrtNEN (1,2,'Default value =',s,'');
              i := DTSize;
              Wrt (1,3,'Enter size: '); WrtE(11,3,':');
              WaitUp;
              WRI(13,3,i,OK);
              OK := (i>0) and (i<VGAY div 2);
              {If not OK then Write(#7);}
              WClear;
            Until OK;
            DTSize := i;
            WClose;
           end{WOK};
           OldOp := 'T';
        end{T};

       'C': begin                             { Colors }
           OldOp1 := 'P';
           MOut1 := KEsc;
           New(Wc);
           Wc^.WOpen(x1c,y1c,x2c,y2c,cumeco,CuBkCo,cumeco,CuEmCo);
           If Wc^.WOK then begin
            Repeat
             d := MenuCol(OldOp1,MOut1);
             If not(d in [KEsc]) then begin
              Case d of

        'P': begin                       { Place Color }
                Sprompt := 'Change place color';
                Dprompt;
                SetColor(DPCol);
                OutTextXY(490, VGAY - 8, 'Current');
                SetColor(DPColI);
                OutTextXY(583, VGAY - 8, 'Default');
                i := GetColor('0',KEsc,185,168);
                Eprompt;
                If i>=0 then DPCol := i;
                OldOp1 := 'P';
             end{P};

        'A': begin               { Place Marking Color }
                Sprompt := 'Change marking color';
                Dprompt;
                SetColor(DPMCol);
                OutTextXY(490, VGAY - 8, 'Current');
                SetColor(DPMColI);
                OutTextXY(583, VGAY - 8, 'Default');
                i := GetColor('0',KEsc,185,168);
                Eprompt;
                If i>=0 then DPMCol := i;
                OldOp1 := 'A';
             end{A};

        'T': begin                  { Transition Color }
                Sprompt := 'Change transition color';
                Dprompt;
                SetColor(DTCol);
                OutTextXY(490, VGAY - 8, 'Current');
                SetColor(DTColI);
                OutTextXY(583, VGAY - 8, 'Default');
                i := GetColor('0',KEsc,185,168);
                Eprompt;
                If i>=0 then DTCol := i;
                OldOp1 := 'T';
             end{T};

        'I': begin                   { Input Arc Color }
                Sprompt := 'Change input arc color';
                Dprompt;
                SetColor(IACol);
                OutTextXY(490, VGAY - 8, 'Current');
                SetColor(IAColI);
                OutTextXY(583, VGAY - 8, 'Default');
                i := GetColor('0',KEsc,185,168);
                Eprompt;
                If i>=0 then IACol := i;
                OldOp1 := 'I';
             end{I};

        'O': begin                   { Output Arc Color }
                Sprompt := 'Change output arc color';
                Dprompt;
                SetColor(OACol);
                OutTextXY(490, VGAY - 8, 'Current');
                SetColor(OAColI);
                OutTextXY(583, VGAY - 8, 'Default');
                i := GetColor('0',KEsc,185,168);
                Eprompt;
                If i>=0 then OACol := i;
                OldOp1 := 'O';
             end{O};

        'W': begin                   { Arc Weight Color }
                Sprompt := 'Change arc weight color';
                Dprompt;
                SetColor(MarCol);
                OutTextXY(490, VGAY - 8, 'Current');
                SetColor(MarColI);
                OutTextXY(583, VGAY - 8, 'Default');
                i := GetColor('0',KEsc,185,168);
                Eprompt;
                If i>=0 then MarCol := i;
                OldOp1 := 'W';
             end{W};

        'N': begin                  { Menu normal color }
                Sprompt := 'Change normal menu color';
                Dprompt;
                SetColor(CuMeCo);
                OutTextXY(490, VGAY - 8, 'Current');
                SetColor(MNTCol);
                OutTextXY(583, VGAY - 8, 'Default');
                i := GetColor('0',KEsc,185,168);
                Eprompt;
                If i>=0 then CuMeCo := i;
                OldOp1 := 'N';
             end{N};

        'E': begin              { Menu emphasized color }
                Sprompt := 'Change emphasized menu color';
                Dprompt;
                SetColor(CuEmCo);
                OutTextXY(490, VGAY - 8, 'Current');
                SetColor(METCol);
                OutTextXY(583, VGAY - 8, 'Default');
                i := GetColor('0',KEsc,185,168);
                Eprompt;
                If i>=0 then CuEmCo := i;
                OldOp1 := 'E';
             end{E};

        'R': begin                       { Prompt color }
                Sprompt := 'Change prompt color';
                Dprompt;
                SetColor(CuPrCo);
                OutTextXY(490, VGAY - 8, 'Current');
                SetColor(PCol);
                OutTextXY(583, VGAY - 8, 'Default');
                i := GetColor('0',KEsc,185,168);
                Eprompt;
                If i>=0 then CuPrCo := i;
                OldOp1 := 'R';
             end{R};

        'H': begin                       { Help normal color }
                Sprompt := 'Change normal help color';
                Dprompt;
                SetColor(HWNTCol);
                OutTextXY(490, VGAY - 8, 'Current');
                SetColor(HWNTColI);
                OutTextXY(583, VGAY - 8, 'Default');
                i := GetColor('0',KEsc,185,168);
                Eprompt;
                If i>=0 then begin
                   HWNTCol := i;
                   {Hfc := i;}
                end;
                OldOp1 := 'H';
             end{H};

        'M': begin                       { Help emphasized color }
                Sprompt := 'Change emphasized help color';
                Dprompt;
                SetColor(HWETCol);
                OutTextXY(490, VGAY - 8, 'Current');
                SetColor(HWETColI);
                OutTextXY(583, VGAY - 8, 'Default');
                i := GetColor('0',KEsc,185,168);
                Eprompt;
                If i>=0 then begin
                   HWETCol := i;
                   {Hec := i;}
                end;
                OldOp1 := 'M';
             end{M};

        'B': begin                       { Help background color }
                Sprompt := 'Change background help color';
                Dprompt;
                SetColor(HWBCol);
                OutTextXY(490, VGAY - 8, 'Current');
                SetColor(HWBColI);
                OutTextXY(583, VGAY - 8, 'Default');
                i := GetColor('0',KEsc,185,168);
                Eprompt;
                If i>=0 then begin
                   HWBCol := i;
                   {Hbc := i;}
                end;
                OldOp1 := 'B';
             end{B};

             end{case};

            end{if d<>KEsc};
           until d in [KEsc];
           Wc^.WClose;
          end{WOK};
          Dispose(Wc);
          OldOp := 'C';
        end{C};

       'F': begin
                 New(Wrr);
                 Wrr^.WOpen(182,181,318,246,
                             CuMeCo,CuBkCo,CuMeCo,CuEmCo);
                 If Wrr^.WOK then begin
                  Wrr^.WrtNEN(1,1,'','D','efault font');
                  Wrr^.WrtNEN(1,2,'','S','mall font');
                  Wrr^.WrtNEN(1,3,'','V','ery small font');
                  WaitUp;
                  MouseCursorOn;
                   d := KeyOrMouseG(['D','S','V'],Menu3GAF,'D',KEsc);
                  MouseCursorOff;
                  If      d='D' then begin
		     MenuFont := DefaultFont; Menusize := 1
                   end
		  else if d='S' then begin
		     MenuFont := SmallFont; Menusize := 5
                   end
		  else if d='V' then begin
		     MenuFont := SmallFont; Menusize := 4
                  end;
                  WaitUp;
                  Wrr^.WClose;
                  Wo^.WClear;
           end{WOK};
           Dispose(Wrr);
           OldOp := 'F';
        end{F};

       'S': begin
           SaveOptions;   { Saving options to PETRISIM.CNF }
           OldOp := 'S';
        end{S};

       'D': begin
           DefaultOptions;   { Set default options }
           OldOp := 'D';
        end{D};

       end{case};
      until c in [KEsc];
     end{with};
     Dispose(Wd);
     Wo^.WClose;
    end{WOK};
    Dispose(Wo);
   End{Options};

{------------------- Management.body -----------------------}

   Begin
     OldOp := 'L';
     MOut := KEsc;
     New(Wn);
     Repeat
      Wn^.WOpen(x1,y1,x2,y2,cumeco,CuBkCo,cumeco,CuEmCo);
      If Wn^.WOK then begin
       MOpt := ManMenu(OldOp,MOut);
       Case MOpt of

       'L': begin
              Listing(182,50);        { List of networks }
              OldOp := 'L';
            end;

       'S': begin
              Listing(182,66);        { Select a new net }
              OldOp := 'S';
            end;

       'N': begin
              Listing(182,82);            { Rename a net }
              OldOp := 'N';
            end;

       'D': begin
              Listing(182,98);            { Delete a net }
              If Selected then OldOp := 'D'
                          else OldOp := 'L';
            end;

       'O': begin
              Options;                  { Global options }
              OldOp := 'O';
            end;

       'R': begin                         { RAM capacity }
             New(Wram);
             With Wram^ do begin
               WOpen(179,127,383,220,
                      cumeco,CuBkCo,cumeco,CuEmCo);
               If WOK then begin
                Str(MemAvail:7,s);
                WrtNEN (1,1,'Total capacity :',s,'');
                Str(MaxAvail:7,s);
                WrtNEN (1,2,'Max. free block:',s,'');
                {Wrt (1,3,'The name of user net is');
                WrtE(1,4,UserNetInfo);}
                Wrt (1,5,'Press any key');
                WaitUp;
                Wait;
                WClose;
               end{WOK};
             end{with};
             Dispose(Wram);
             OldOp := 'R';
            end{R};

        end{case};
        Wn^.WClose;
       end{then WOK}
      Else MOpt:=KEsc;
     Until MOpt in [KEsc,KLeft,KRight];
    Dispose(Wn);
    MenuOut1 := MOpt;
   End{Management};

{***********************************************************}

  Procedure Quit;                             { Quit option }

  Begin                                      { Confirmation }
       If not Empty(Nets) then begin  { Only if nets in RAM }
         New(Wq);
         With Wq^ do begin
           WOpen(x1q,y1q,x2q,y2q,
                  LightRed,CuBkCo,LightRed,Yellow);
           If WOK then begin
            WaitUp;
            Wrt (2,1,'There are nets in RAM');
            Wrt (2,2,'Are you sure you want');
            WrtNEN (2,3,'to QUIT ?       (','Y','/N)');
            c:=CWait(b);
            Finish := c in ['Y','y'];
            WClose;
           end{WOK};
         end{with};
         Dispose(Wq)
        end{then}
       else
         Finish := true;       {end-if}
  End{Quit};

{**************************************************************}

  Procedure ShowAllPlTr(Var F: Text);      { Shows All Pl & Tr }
                    { F exists and is after successful Rewrite }
  Var n:Nodepoint;
      p:PNp;
      t:PNt;
  Begin
     WriteLn(F,'Report on the net  : ',SNet^.Name { ,
               '  Model: ',UserNetInfo} );
     WriteLn(F,'Experiment duration: ', MaxTime:12:3);
     WriteLn(F,'Current time       : ', PNTime:12:3);
     n := TakeFirst(SNet^.Places);        { Places }
     while n<>nil do begin
        p := n^.el;             { pointer to place }
        WriteLn(F,'==================================================');
        PReport(F,p^.Name);
        n := TakeNext(n);
     end{while};
     WriteLn(F,'==================================================');
     n := TakeFirst(SNet^.Transitions); { Transitions }
     while n<>nil do begin              
        t := n^.el;         { pointer to transition }
        WriteLn(F,'==================================================');
        TReport(F,t^.Name);
        n := TakeNext(n);  { Next transition }
     end{while};
     WriteLn(F,'==================================================');
  End;

{**************************************************************}

  Procedure ShowAllinTable(Var F: Text);   { Shows All Pl & Tr }
                    { F exists and is after successful Rewrite }
  Var n:Nodepoint;
      p:PNp;
      t:PNt;
      MV,MA,MMin,MMax,MVa,MS,WA,WMin,WMax,WVa,WS,Arr : Real;
      U,FAv,FMin,FMax,FVa,FS : Real;
      NF : LongInt;
      i : Integer;
  Begin
     WriteLn(F,'Report on the net  : ',SNet^.Name { ,
               '  Model: ',UserNetInfo  });
     WriteLn(F,'Experiment duration: ', MaxTime:12:3);
     WriteLn(F,'Current time       : ', PNTime:12:3);
     WriteLn(F);
     WriteLn(F,'Places:');
     WriteLn(F);
     WriteLn(F,
'Name       Marking        Mean    Min     Max     StDev      AvWait      Min        Max      StDev     Added');
     WriteLn(F,
'============================================================================================================');
     n := TakeFirst(SNet^.Places);        { Places }
     while n<>nil do begin
        p := n^.el;             { pointer to place }
        PResults(p^.Name,MV,MA,MMin,MMax,MVa,MS,WA,WMin,WMax,WVa,WS,Arr);
        Write(F,p^.Name);
        For i:=1 to 12-Length(p^.Name) do Write(F,' ');
        Write  (F,MV:6:0,MA:12:3,MMin:7:0,MMax:8:0,MS:10:2);
        If WA=-1 then
          Write(F,'                                           ')
        else
          Write(F,WA:12:3,WMin:9:2,WMax:11:2,WS:11:2);
        WriteLn(F,Arr:10:0);
        n := TakeNext(n);
     end{while};

     WriteLn(F);
     WriteLn(F,'Transitions:');
     WriteLn(F);
     WriteLn(F,
'Name      Utilization     MeanFire       Min        Max     StDev   Firings');
     WriteLn(F,
'===========================================================================');

     n := TakeFirst(SNet^.Transitions); { Transitions }
     while n<>nil do begin              
        t := n^.el;         { pointer to transition }
        TResults(t^.Name,U,FAv,FMin,FMax,FVa,FS,NF);
        Write(F,t^.Name);
        For i:=1 to 15-Length(t^.Name) do Write(F,' ');
        WriteLn(F,U:6:3,FAv:13:3,FMin:10:2,FMax:11:2,FS:10:3,NF:10);
        n := TakeNext(n);  { Next transition }
      end{while};
  End;

{******************** "Program.Body" ***********************}

Begin                   { This is the initialisation code ! }

   CheckBreak := true;                { Ctrl-Break disabled }
   GraphInit;             { General graphics initialization }
   GGraphInit;         { Initialization of GGraph constants }

   OK := MouseInit;                  { Mouse initialization }

   If not OK then begin
     Wex(3,10,'Mouse initialization error - Driver not detected');
     Wx (3,12,'Use cursor keys to move the pointer');
     Wx (3,14,'Enter = left button');
     Wx (3,15,'Esc   = right button');
     Wx (3,18,'Press any key');  c := ReadK;
     ClearDevice;
    end{then}
   else begin          { Mouse OK }
     SetMouseXRange(0,637);
     SetMouseYRange(0,477);
   end{if};

   LogoScr;
   ClearDevice;
   Labels;
                    { INITIALISATIONS: }
{ PNet6: }
 Selected    := false;
 Nets        := Newlist;
{ CrtES: }
 CuBkCo      := WBCol;
{ Local: }
 Fname       := '';
 Finish      := false;
 EditingTF   := false;

 DefaultOptions;                  { Set default options }
 LoadOptions;      { Loading from PETRISIM.CNF if found }

 If DefFName<>'None' then FirstLoading; { Loading default net }

 Opt := 'F';                           { Initial option }
 MenuOut1 := KEsc;

 Repeat                                     { Main loop }
   Option := Menu1(Opt,MenuOut1);
   MenuOut1 := Option;
   Case Option of

   'E': begin              { Editing the selected P-net }
          Snet^.Modify;
          DrawNet;
          Opt := 'E';
        end{E};

   'S': begin             { Simulate the selected P-net }
          SNet^.Time := 0;
          Snet^.Simulate;
          DrawNet;
          Opt := 'S';
        end{S};

   'F': begin
          Files;           { Files management, printing }
          Opt := 'F';
        end{F};

   'M': begin
          Management;           { Managing list of nets }
          DrawNet;
          Opt := 'M';
        end;

   'H': begin
          GNoHelp;
          Opt := 'H';
        end;

   'Q',KEsc: begin
          Quit;
          Opt := 'Q';
        end;

   KF1: begin      { Context help }
          GMainHelp;
          Opt := KF1;
        end;

   End{case};
  Until Finish;

  CloseGraph;
  DOSHalt(0);              { Settings for DOS }
End.

