Unit UserPart;   { Declarations related to user experiments }

Interface

  Uses Crt, CrtES, Graph, GGraph, Simpet5, Petri, Queuing,
       Statist, Randoms, UserData;

{-----------------------------------------------------------}

{ User Petri objects: Overide the Petri methods if necessary.
  Use either Petri, Inhibits or Queuing objects }

Type  UserPP = ^UserPlace;            { Pointers to objects }
      UserTP = ^UserTransition;
      UserAP = ^UserArc;

      UserPlace = Object(QueuingPlace)
      End{UserPlace};

      UserTransition = Object(QueuingTransition)
      End{UserTransition};

      UserArc = Object(QueuingArc)
      End{UserArc};

{ Routines to be used by USER.PAS: }
Procedure Warning(Name : string);      { Wrong name message }
Procedure Step(s : String);
                    { One simulation step in debugging mode }
Procedure StandardFastInit;
             { Typical user model initialization - defaults }
Function PResults(             { Place statistical results: }
           PName : string;      { Case sensitive place name }
       Var MValue,                 { Current (last) marking }
           MAverage,                      { Average marking }
           MMin,                          { Minimum marking }
           MMax,                          { Maximum marking }
           MVariance,                    { Marking variance }
           MStdDev,            { Marking standard deviation }
           WAverage,           { Average token waiting time }
           WMin,               { Minimum token waiting time }
           WMax,               { Maximum token waiting time }
           WVariance,               { Waiting time variance }
           WStdDev,       { Waiting time standard deviation }
           TokensIn : Real)  { # of initial+entering tokens }
           : Boolean;             { false if name not found }
Function TResults(        { Transition statistical results: }
           TName : string; { Case sensitive transition name }
       Var Utilization,   { Utilization (portion of firing) }
           AvFire,                { Average firing duration }
	   MinFire,              { Shortest firing duration }
	   MaxFire,               { Longest firing duration }
	   VarFire,              { Firing duration variance }
	   StdFire : Real;        { Firing duration std dev }
       Var FCounter: LongInt) { Number of completed firings }
           : Boolean;             { false if name not found }
Function GG1Results(    { Queue+Server statistical results: }
           PName,               { Case sensitive place name }
           TName : string; { Case sensitive transition name }
       Var LQ,                       { Average queue length }
           LQVar,                   { Queue length variance }
           WQ,                       { Average waiting time }
           WQVar,                   { Waiting time variance }
           Rho,   { Utilization = average number in service }
           L,              { Average system size = LQ + Rho }
           W,  { Average system time = WQ + average service }
           Lambda,                 { Effective arrival rate }
           Ts,                   { Average service duration }
	   TsVar: Real)         { Service duration variance }
           : Boolean;         { false if any name not found }
Function PReport(Var R : Text;      { Place typical report: }
           PName : string) : Boolean; { false if wrong name }
Function TReport(Var R : Text; { Transition typical report: }
           TName : string) : Boolean; { false if wrong name }
Function GG1Report(Var R : Text;    { G/G/1 typical report: }
           PName,TName : string)
	   : Boolean;             { false if any wrong name }
Function PrepareFile : String; { Returns new data file name }
Procedure ShowThemAll;      { Typical user model evaluation }
Procedure ShowThemInTable;  { Typical user model evaluation }

{ Routines required by PETRISIM: }
  Procedure FiringStarts(Var s:String; TNum:Integer; t:UserTP);
     { Starting firing the transition "s" with number TNum.
       Called when firing starts AFTER StartFire, that
       normally removes tokens - if not modified ! }
  Procedure FiringEnds(Var s:String; TNum:Integer; t:UserTP);
     { Finishing firing the transition "s" with number TNum.
       Called when firing ends BEFORE EndFire, that
       normally adds tokens - if not modified ! }
  Procedure UserDead;          { Called in case of deadlock }

Const                      { Text editor for the user model }
    EditorName  = '\WINDOWS\SYSTEM32\EDIT.COM';
    CommandName  = '\WINDOWS\SYSTEM32\COMMAND.COM';

{-----------------------------------------------------------}

Implementation

{ Here you CAN declare any number of any objects and you HAVE
  to define the bodies of the procedures mentioned above.   }
Const
   ResFileName = 'Result'; { Results file, 1/2 digits added }
   ResFileExt = 'Pas'{'Txt'};      { Results file extension }
{******************** USER procedures **********************}

Procedure Warning(Name:string);{ Used in case of wrong name }
Begin
   {ClrScr;}
   GoToXY(1,21); ClrEol;
   Write(' Wrong name used: ',Name, ' ( Names are case sensitive ! )');
   GoToXY(1,23); ClrEol;
   Write(' Compare the net and names used in user evaluation.');
   GoToXY(1,24); ClrEol;
   Write(' Zeros will be returned. Press Enter to continue.');
   ReadLn;
   {Terminate}
End;

{--------------------------------------------------------------}

Procedure Step(s : String);
                { One simulation step in debugging mode. }
Var c : Char;
Begin
   ClrScr; TextColor(Green);
   GoToXY(5,2);  Write('Simulation in Debugging Mode');
   GoToXY(5,4);  Write('Current Time = ',PNTime:10:3);
   GoToXY(5,11); Write('Event: ');
   TextColor(Yellow); Write(s);
   TextColor(Green);
   GoToXY(5,16); Write('Now select the option:');
   GoToXY(5,18); Write('(S)....Show P-net (Then Enter to continue)');
   GoToXY(5,20); Write('(C)....Continue');
   GoToXY(5,22); Write('(T)....Terminate the experiment');
   Repeat
      c := UpCase(ReadK);
   Until c in ['S','C','T'];
   Case c of
       'S': begin
              ShowNet; ReadLn;
              RestoreCrtMode
            end{S};
       'T': Terminate;
   End{case};
End{Step};

{--------------------------------------------------------------}

Function PResults(             { Place statistical results: }
           PName: string;       { Case sensitive place name }
       Var MValue,                 { Current (last) marking }
           MAverage,                      { Average marking }
           MMin,                          { Minimum marking }
           MMax,                          { Maximum marking }
           MVariance,                    { Marking variance }
           MStdDev,            { Marking standard deviation }
           WAverage,           { Average token waiting time }
                                      { -1 if Timing not on }
           WMin,               { Minimum token waiting time }
           WMax,               { Maximum token waiting time }
           WVariance,               { Waiting time variance }
           WStdDev,       { Waiting time standard deviation }
           TokensIn             { Number of entering tokens }
           : Real) : Boolean;     { false if name not found }

Var PStat : PRAccum;            { Links to place statistics }
    WStat : PRTally;
    P : Pointer;
Begin
   { Function PReference(PName: string) : Pointer; }
   P := PReference(PName);
   If P=nil then begin   { All zeros if wrong name }
     Warning(PName);
     PResults := false;
     MValue := 0;
     MAverage := 0;
     MMin := 0;
     MMax := 0;
     MVariance := 0;
     MStdDev := 0;
     WAverage := 0;
     WMin := 0;
     WMax := 0;
     WVariance := 0;
     WStdDev := 0;
     TokensIn := 0;
    end
   Else begin
     PResults := true;
     PStat := @UserPP(P)^.AccMarking;  { Marking Statistics }
     With PStat^ do begin
      RUpdate(UserPP(P)^.Marking);      { Last accumulation }
      MValue := RValue;
      MAverage := RAverage;
      MMin := RMin;
      MMax := RMax;
      MVariance := RVariance;
      MStdDev := RStdDev;
     end{with};
     WStat := @UserPP(P)^.WaitTime;{ Waiting time Statistics }
     With WStat^ do begin
      WAverage := RAverage;
      WMin := RMin;
      WMax := RMax;
      WVariance := RVariance;
      WStdDev := RStdDev;
     end{with};
     If not UserPP(P)^.Timing then WAverage := -1;
     TokensIn := UserPP(P)^.Arrivals;
   end{if};
End;

{--------------------------------------------------------------}

Function PReport(Var R : Text; PName : string) : Boolean;
                                    { Place typical report: }
Var MV,MA,MMin,MMax,MVa,MS,
    WA,WMin,WMax,WVa,WS,Arr : Real;
Begin
   If not PResults(PName,MV,MA,MMin,MMax,MVa,MS,
                   WA,WMin,WMax,WVa,WS,Arr) then
     PReport := false
   Else Begin
     PReport := true;
     WriteLn(R);
     WriteLn(R,'Report on place: ',PName);
     WriteLn(R,'Marking:  Current: ',MV:6:0);
     WriteLn(R,'Average: ',MA:7:3,'  Min: ',MMin:6:2,
               '  Max: ',MMax:8:2,'  StdDev: ',MS:7:3);
     If WA=-1 then
        WriteLn(R,'Tokens waiting time not recorded')
     else begin
        WriteLn(R,'Tokens waiting time');
        WriteLn(R,'Average: ',WA:7:2,'  Min: ',WMin:6:2,
                  '  Max: ',WMax:8:2,'  StdDev: ',WS:7:3);
     end;
     WriteLn(R,'Number of added tokens: ',Arr:8:0);
     WriteLn(R);
   End;
End;

{--------------------------------------------------------------}

Function TResults(        { Transition statistical results: }
           TName: string;  { Case sensitive transition name }
       Var Utilization,   { Utilization (portion of firing) }
           AvFire,                { Average firing duration }
	   MinFire,              { Shortest firing duration }
	   MaxFire,               { Longest firing duration }
	   VarFire,              { Firing duration variance }
	   StdFire : Real;        { Firing duration std dev }
       Var FCounter: LongInt) { Number of completed firings }
           : Boolean;                 { false if wrong name }

Var TStat : PRAccum;            { Transition statistics }
    FStat : PRTally;
    T : Pointer;
Begin
   { Function TReference(TName: string) : Pointer; }
   T := TReference(TName);
   If T=nil then begin   { All zeros if wrong name }
     Warning(TName);
     TResults := false;
     Utilization := 0;
     FCounter := 0;
     AvFire := 0;
     MinFire := 0;
     MaxFire := 0;
     VarFire := 0;
     StdFire := 0;
    end
   Else begin
     TResults := true;
     TStat := @UserTP(T)^.AccBusy;
     Utilization := TStat^.RAverage;   { Last accum. ignored }
     FCounter := UserTP(T)^.Counter;
     FStat := @UserTP(T)^.FireTime; { Firing time Statistics }
     With FStat^ do begin
      AvFire := RAverage;
      MinFire := RMin;
      MaxFire := RMax;
      VarFire := RVariance;
      StdFire := RStdDev;
     end{with};
   end{if};
End;

{--------------------------------------------------------------}

Function TReport(Var R : Text; TName : string) : Boolean;
                               { Transition typical report: }
Var U,FAv,FMin,FMax,FVa,FS : Real;
    N : LongInt;
Begin
   If not TResults(TName,U,FAv,FMin,FMax,FVa,FS,N) then
     TReport := false
   Else Begin
     TReport := true;
     WriteLn(R);
     WriteLn(R,'Report on transition: ',TName);
     WriteLn(R,'Utilization: ',U:5:3);
     WriteLn(R,'Firing duration');
     WriteLn(R,'Average: ',FAv:8:3,'  Min: ',FMin:10:6,
               '  Max: ',FMax:9:3,'  StdDev: ',FS:6:3);
     WriteLn(R,'Number of finished firings: ',N:8);
     WriteLn(R);
   End;
End;

{--------------------------------------------------------------}

Function GG1Results(    { Queue+Server statistical results: }
           PName,               { Case sensitive place name }
           TName : string; { Case sensitive transition name }
       Var LQ,                       { Average queue length }
           LQVar,                   { Queue length variance }
           WQ,                       { Average waiting time }
           WQVar,                   { Waiting time variance }
           Rho,   { Utilization = average number in service }
           L,              { Average system size = LQ + Rho }
           W,  { Average system time = WQ + average service }
           Lambda,                 { Effective arrival rate }
           Ts,                   { Average service duration }
	   TsVar: Real)         { Service duration variance }
           : Boolean;         { false if any name not found }

Var MV,MA,MMin,MMax,MVa,MS,
    WA,WMin,WMax,WVa,WS,Arr,U,FAv,FMin,FMax,FVa,FS : Real;
    N : LongInt;
    WrongName : Boolean;

Begin
   WrongName := not PResults(PName,MV,MA,MMin,MMax,
                             MVa,MS,WA,WMin,WMax,WVa,WS,Arr);
   If not WrongName then WrongName := not TResults(TName,U,FAv,
                             FMin,FMax,FVa,FS,N);
   If WrongName then begin
      LQ := 0;
      LQVar := 0;
      WQ := 0;
      WQVar := 0;
      Rho := 0;
      L := 0;
      W := 0;
      Lambda := 0;
      Ts := 0;
      TsVar := 0;
    end
   Else begin
      LQ := MA;
      LQVar := MVa;
      WQ := Wa;
      WQVar := WVa;
      Rho := U;
      L := LQ + Rho;
      W := WQ + FAv;
      If PNTime=0 then Lambda:=0 else Lambda := Arr/PNTime;
      Ts := FAv;
      TsVar := FVa;
   end;
   GG1Results := not WrongName;
End;

{--------------------------------------------------------------}

Function GG1Report(Var R : Text;    { G/G/1 typical report: }
           PName,TName : string)
	   : Boolean;             { false if any wrong name }
Var MV,MA,MMin,MMax,MVa,MS,
    WA,WMin,WMax,WVa,WS,Arr,U,FAv,FMin,FMax,FVa,FS,Lef : Real;
    N : LongInt;
    WrongName : Boolean;

Begin
   WrongName := not PResults(PName,MV,MA,MMin,MMax,
                             MVa,MS,WA,WMin,WMax,WVa,WS,Arr);
   If not WrongName then WrongName := not TResults(TName,U,FAv,
                             FMin,FMax,FVa,FS,N);
   If not WrongName then begin
     WriteLn(R);
     WriteLn(R,'Report on G/G/1 made of queue ',PName,' & server ',TName);
     WriteLn(R,'Queue length');
     WriteLn(R,'Average: ',MA:8:3,'  Min: ',MMin:7:2,
               '  Max: ',MMax:8:2,'  StdDev: ',MS:8:3);
     WriteLn(R,'Queue waiting time');
     WriteLn(R,'Average: ',WA:8:2,'  Min: ',WMin:7:2,
               '  Max: ',WMax:8:2,'  StdDev: ',WS:8:3);
     WriteLn(R,'Server utilization: ',U:5:3);
     WriteLn(R,'Service duration');
     WriteLn(R,'Average: ',FAv:8:3,'  Min: ',FMin:10:6,
               '  Max: ',FMax:9:3,'  StdDev: ',FS:6:3);
     WriteLn(R,'Average system size: ',(MA+U):10:3);
     WriteLn(R,'Average system wait: ',(WA+FAv):10:3);
     If PNTime=0 then Lef:=0 else Lef:=Arr/PNTime;
     WriteLn(R,'Effective arrival rate: ',(Lef):8:4);
     WriteLn(R);
   end;
   GG1Report := not WrongName;
End;

{--------------------------------------------------------------}

Procedure StandardFastInit;
             { Typical user model initialization - defaults }
Begin
   Randomize;
   TextBackGround(Black);
   TextColor(Green);
   ClrScr;
   If not Confir then begin
       GoToXY(5,2);
       Write('Simulation in fast mode');
       GoToXY(5,4);
       Write('Experiment Length = ',MaxTime:8:1);
       GoToXY(5,16);
       Write('Wait please ... (Press any key to stop the run)');
   End{if};
End{StandardInit};
{------------------------------------------------------}

Function PrepareFile : String;    { New data file name }
Var  F : Text;                    { 'Q' if none = Quit }
     FName : String;
     n, IORes : Integer;
     c : Char;
Begin
   n := 0;
   Repeat
      Str(n,FName);
      FName := ResFileName + FName + '.' + ResFileExt;
      Assign(F,FName);
      {$I-} Reset(F); {$I+}
      IORes := IOResult;
      If IORes=0 then begin
         n := n + 1;  { File exists, move }
         Close(F);
      end
   Until (IORes<>0) or (n=99);
   Repeat
      Assign(F,FName);
      {$I-} Rewrite(F); {$I+}
      IORes := IOResult;
      If IORes<>0 then begin
        WriteLn('Some troubles when writing the file ',FName);
        WriteLn;
        WriteLn('IOResult error code: ',IORes:4);
        WriteLn;
        WriteLn('Error message: ',ErrorMessage(IORes));
        WriteLn;
        Write('Enter a file name (''Q'' to quit): ');
        ReadLn(FName);
        If FName='q' then FName:='Q';
      End{if};
   Until (IORes=0) or (FName='Q');
   If (FName<>'Q') then Close(F);
   PrepareFile := FName;
End;

{--------------------------------------------------------------}

Procedure ShowThemAll;      { Typical user model evaluation }
Var F : Text;
    FName : String;
Begin
   TextBackGround(Black);
   TextColor(Green);
   ClrScr;
   FName := PrepareFile;
   If FName<>'Q' then begin
      Assign(F,FName);
      Rewrite(F);
      ShowAllPlTr(F);
      Close(F);
   end;
   Assign(F,'CON');
   Rewrite(F);
   ShowAllPlTr(F);
   WriteLn;
   If FName<>'Q' then Write('Results stored in the file ', FName)
                 else Write('Results not stored !');
   Write('  Press Enter to continue');
   Readln;
End{ShowThemAll};

{--------------------------------------------------------------}

Procedure ShowThemInTable;     { Typical user model evaluation }
Var F : Text;
    FName : String;
Begin
   TextBackGround(Black);
   TextColor(Green);
   ClrScr;
   FName := PrepareFile;
   If FName<>'Q' then begin
      Assign(F,FName);
      Rewrite(F);
      ShowAllinTable(F);
      Close(F);
   end;
   Assign(F,'CON');
   Rewrite(F);
   ShowAllinTable(F);
   WriteLn;
   If FName<>'Q' then Write('Results stored in the file ', FName)
                 else Write('Results not stored !');
   Write('  Press Enter to continue');
   Readln;
End{ShowThemInTable};

{****************** PetriSim procedures ********************}

Procedure UserDead;            { User deadlock message }
Begin
   GoToXY(20,12);
   Write('Deadlock - Press Enter to continue');
   Readln;
End;

{------------------------------------------------------}

Procedure FiringStarts(Var s:String; TNum:Integer; t:UserTP);
{ Starting firing the transition "s" with number TNum. }
{ Including procedure Firing_Starts. Don't change next line ! }
   {$I %%S.PAS}
Begin
   { Here is the model specific behavior: }
   Firing_Starts;           { Don't change this line ! }

   If PNTime>MaxTime then Terminate;

   If Confir then
       Step('Starting Firing of ' + s)
   Else begin
       GoToXY(5,6);
       Write('Current Time = ',PNTime:10:3);
   End{If};
End;

{------------------------------------------------------}

Procedure FiringEnds(Var s:String; TNum:Integer; t:UserTP);
{ Finishing firing the transition "s" with number TNum. }
{ Including procedure Firing_Ends. Don't change next line ! }
   {$I %%E.PAS}
Begin
   { Here is the model specific behavior: }
   Firing_Ends;             { Don't change this line ! }

   If Confir then
       Step('Finishing Firing of ' + s)
   Else begin

   End{If};
End;

End.


