Unit WindowsS;    { Version Simpet: Colors    April 1993 }
{$N+,E+}                                 { General graph }

Interface

Uses Crt, Graph, CrtES, GGRaph, EMouseGS;

{ GGraph    = General graph (various cards to VGA 640x480) }
{ CrtES     = Crt enhanced (some more routines) }
{ EMouseGS  = Enhanced mouse functions (Graphics only) }

(* This unit defines these windows:
      GWINDOW is a generic window
         OWINDOW is an output window
         IOWINDOW is an I/O window   (all no borders)
            PWINDOW is a single border IOWINDOW
            DWINDOW is a double border IOWINDOW

   Function GetColor prompts for entering a color. *)

{************************* Objects **************************}

Type  GWindowP  = ^GWindow;       { Pointers }
      OWindowP  = ^OWindow;
      IOWindowP = ^IOWindow;
      PWindowP  = ^PWindow;
      DWindowP  = ^DWindow;

      CharSet = Set of Char;

      GWindow = object      { Generic (ancestor) window }
        WOK : Boolean;       { Whether opening OK - RAM }
      {Methods:}
        Constructor GWOpen(xul,yul,xbr,ybr:integer;
                               BdC,BkC,TC,TEC:byte);
        Procedure GWClear;
        Procedure WClose;
       Private {attributes:}
        Wx1,Wy1   :integer;     { Upper left corner }
        Wx2,Wy2   :integer;   { Bottom right corner }
        BdColor   :byte;             { Border color }
        BkColor   :byte;         { Background color }
        TColor    :byte;               { Text color }
        TEColor   :byte;    { Text emphasized color }
        P         :pointer;     { Pointer to memory }
        Size      :Word;              { Memory size }
       End;

      OWindow = object (GWindow)    { Output window }
      {Methods:}
        Procedure Wr  (xt,yt:integer; S:string);
        Procedure Wrt (xt,yt:integer; S:string);
        Procedure WrtE(xt,yt:integer; S:string);
        Procedure WrtNEN(xt,yt:integer; s1,s2,s3:string);
        Procedure Wrub(xt,yt:integer; S:string);
      End;

      IOWindow = object (OWindow)      { I/O window }
      {Methods:}
        Procedure WReadStr(xt,yt:integer; var S:string; Tset: CharSet);
        Procedure WRS (xt,yt:integer; var S:string);
        Procedure WRSDOS (xt,yt:integer; var S:string);
        Procedure WRSC (xt,yt:integer; var S:string; c:char);
        Procedure WRI (xt,yt:integer; var i :longint;
                                      var given:boolean);
      End;

      PWindow = object (IOWindow)     { Single border I/O window }
      {Methods:}
        Constructor WOpen(xul,yul,xbr,ybr:integer;
                               BdC,BkC,TC,TEC:byte);
        Procedure WClear;
      End;

      DWindow = object (IOWindow)     { Double border I/O window }
      {Methods:}
        Constructor WOpen(xul,yul,xbr,ybr:integer;
                               BdC,BkC,TC,TEC:byte);
        Procedure WClear;
      End;

    Function GetColor(OldOp,MOut:char; x11,y11: integer) : integer;
    { Selects and returns color. x11,y11 = window coordinates.
      Returns -1 if no color selected (Esc). }

{***********************************************************}

Implementation

Const    dw = 2;    { Difference between borders of DWindow }
         Wx1e = 180; Wy1e = 220;         { Emergency window }
         Wx2e = 420; Wy2e = 250;
Var      Wcs  :PWindowP;                   { Colors windows }


{******************* Methods for PWindow *******************}

  Constructor GWindow.GWOpen(xul,yul,xbr,ybr:integer;
                                    BdC,BkC,TC,TEC:byte);
  Var  M : Array[1..3900] of Byte;   { Emergency memory }

  Begin
    Wx1 := xul; Wy1 := yul;
    Wx2 := xbr; Wy2 := ybr;
    BdColor := BdC;
    BkColor := BkC;
    TColor  := TC;
    TEColor := TEC;
    Size := ImageSize(Wx1,Wy1,Wx2,Wy2);
    WOK := Size <= MaxAvail;
    If WOK then begin
      GetMem(P, Size);
      GetImage(Wx1,Wy1,Wx2,Wy2,P^);
      SetFillStyle(SolidFill,BkColor);
      Bar(Wx1,Wy1,Wx2,Wy2);
     end{then}
    Else begin
      P := @M;
      {Size := ImageSize(Wx1e,Wy1e,Wx2e,Wy2e);}
      GetImage(Wx1e,Wy1e,Wx2e,Wy2e,P^);
      SetFillStyle(SolidFill,White);
      Bar(Wx1e,Wy1e,Wx2e,Wy2e);
      SetTextStyle(MenuFont, HorizDir, MenuSize);
      SetTextJustify(CenterText, CenterText);
      SetColor(Red);
      OutTextXY((Wx1e+Wx2e) div 2, (Wy1e+Wy2e) div 2,
                'Out of memory - Press Enter');
      ReadLn;
      PutImage(Wx1e,Wy1e,P^,NormalPut);
    end{if};
  End;

{***********************************************************}

  Procedure GWindow.WClose;
  Begin
   If WOK then begin
    PutImage(Wx1,Wy1,P^,NormalPut);
    FreeMem(P,Size);
   end;
  End;

{***********************************************************}

  Procedure GWindow.GWClear;
  Begin
   If WOK then begin
    SetFillStyle(SolidFill,BkColor);
    Bar(Wx1,Wy1,Wx2,Wy2);
   end;
  End;

{***********************************************************}

  Procedure OWindow.Wr(xt,yt: integer; s: string);

  Var x0,y0  :integer;   { Window upper left }

  Begin
   If WOK then begin
    x0 := Wx1 +  TextWidth('x')*xt;
    If x0<Wx2 then begin
      y0 := Wy1 + 16*yt;
      SetTextStyle(MenuFont, HorizDir, MenuSize);
      SetTextJustify(LeftText, CenterText);
      While (x0 + TextWidth(s)) > Wx2 do
        s := Copy(s,1,Length(s)-1);
      OutTextXY(x0, y0, s);
    end{if};
   end;
  End;

{***********************************************************}

  Procedure OWindow.Wrt (xt,yt: integer; S: string);
  Begin
   If WOK then begin
    SetColor(TColor);
    Wr(xt, yt, S);
   end;
  End;

{***********************************************************}

  Procedure OWindow.WrtE(xt,yt: integer; S: string);
  Begin
   If WOK then begin
    SetColor(TEColor);
    Wr(xt, yt, S);
   end;
  End;

{***********************************************************}

  Procedure OWindow.WrtNEN(xt,yt: integer; s1,s2,s3: string);
  Var x0,y0  :integer;   { Window upper left }
  Begin
   If WOK then begin
    x0 := Wx1 +  TextWidth('x')*xt;
    If x0<Wx2 then begin
      y0 := Wy1 + 16*yt;
      SetTextStyle(MenuFont, HorizDir, MenuSize);
      SetTextJustify(LeftText, CenterText);
      SetColor(TColor);
      OutTextXY(x0, y0, s1);
      SetColor(TEColor);
      OutTextXY(x0+TextWidth(s1), y0, s2);
      SetColor(TColor);
      OutTextXY(x0+TextWidth(s1+s2), y0, s3);
    end{if};
   end;
  End;

{***********************************************************}

  Procedure OWindow.Wrub(xt,yt: integer; S: string);
  Begin
   If WOK then begin
    SetColor(Black);
    Wr(xt, yt, S);
   end;
  End;

{******************** Readings *****************************}

Procedure IOWindow.WReadStr (xt,yt:integer; var s:string; Tset: CharSet);
{ Reading string to window - Esc = ''; Accepts only chars in Tset }
Var c   : char;
    x,y : word;
    num : integer;
    e   : string[1];

Begin
 s := '';
 If WOK then begin
  num := 0;
  WrtE(xt,yt,'_');
  If MouseOK then
    Repeat until KeyPressed or (MousePosition(x,y)<>0)
  else
    Repeat until KeyPressed;
  If KeyPressed then begin
   Repeat
      c := ReadK;
      If c in Tset then begin
           s := s + c; Inc(num);
           WRub(xt,yt,'_'); WrtE(xt,yt,c); Inc(xt); WrtE(xt,yt,'_');
        end
      else if (c=KBS) and (num>0) then begin   { Backspace }
           WRub(xt,yt,'_');
           e := Copy(s,num,1);
           s := Copy(s,1,num-1);
           Dec(num);
           Dec(xt);  Wrub(xt,yt,e); WrtE(xt,yt,'_');
      end{if};
   Until (c=KCR) or (c=KEsc);
   If c=KEsc then s := '';
  end{if}
  else
    Delay(WaitTime);
 end;
End;

{***********************************************************}

Procedure IOWindow.WRS (xt,yt:integer; var s:string);
                      { Reading string to window - Esc = '' }
Var st : CharSet;
Begin
  st := [' '..#127];
  WReadStr (xt,yt,s,st);
End;

{***********************************************************}

Procedure IOWindow.WRSDOS (xt,yt:integer; var s:string);
                      { Reading string to window - Esc = '' }
                                { Takes only DOS characters }
Var st : CharSet;
Begin
  st := ['0'..'9','a'..'z','A'..'Z','$','%','''','_','^',
                '@','{','}','~','`','!','#','(',')','&','-'];
  WReadStr (xt,yt,s,st);
End;

{***********************************************************}

Procedure IOWindow.WRSC (xt,yt:integer; var s:string; c:char);
{ Reading string to window - Esc = '' , 1st character given }

Var s1: string;

begin
  s := '';
  If WOK then begin
   If c=KEsc then s:='' else begin
      WrtE(xt,yt,c);
      WRS(xt+1,yt,s1);
      s := c + s1;
   end;
  end;
end;

{***********************************************************}

Procedure IOWindow.WRI (xt,yt:integer; var i :longint;
                                      var given:boolean);
      { Safe readln of longint, Enter or Esc may be pressed,
        deletes the wrong string ! }
Var s,sc,z :string;
    code   :integer;
    nr     :real;
    OK     :boolean;
    W      :PWindowP;
    x,y    :word;
    xw,yw  :integer;

Begin
 given := false;
 If WOK then begin
  WrtE(xt,yt,'_');
  If MouseOK then
    Repeat until KeyPressed or (MousePosition(x,y)<>0)
  else
    Repeat until KeyPressed;
  If KeyPressed then begin
   WRS(xt,yt,s); z:= s;
   While (s<>'') and (Copy(s,Length(s),1)=' ') do
     s := Copy(s,1,Length(s)-1);
   If s<>'' then begin
    given := true;
    Val(s,nr,code);
    OK := (code=0) and (Abs(nr)<2147483648.0); {max. longint}
    If OK then Val(s,i,code)
    else begin
      New(W);
      With W^ do begin
       xw := VGAX div 2 - 130;                { Screen center }
       yw := VGAY div 2 - 35;
       WOpen(xw,yw,xw+260,yw+70,LightRed,Black,LightBlue,LightRed);
       Repeat
         WrtE(1,1,'Error in integer input !');
         If code<>0 then begin
            Str(code:3,sc);
            Wrt(1,2,'Wrong character at position'+sc);
           end
         else
            Wrt(1,2,'Value out of LongInt range');
         WrtE( 1,3,'Try again: ');
         WRS (12,3,s);
         While (s<>'') and (Copy(s,Length(s),1)=' ') do
            s := Copy(s,1,Length(s)-1);
         Val(s,nr,code);
         OK := (code=0) and (Abs(nr)<2147483648.0); {max. longint}
         If OK then Val(s,i,code);
         WClear
       Until OK or (s='');
       given := Ok;
       WClose;
      end{with};
      Dispose(W);
      Wrub(xt,yt,z);     { Deleting the wrong string }
    end{if};
   end{if};
  end{if}
  else
    Delay(WaitTime);
 end;
end{WRI};

{******************* Methods for PWindow *******************}

  Constructor PWindow.WOpen(xul,yul,xbr,ybr:integer;
                                    BdC,BkC,TC,TEC:byte);
  Begin
    GWOpen(xul,yul,xbr,ybr,BdC,BkC,TC,TEC);
    If WOK then begin
      SetLineStyle(SolidLn,0,NormWidth);
      SetColor(BdColor);
      Rectangle(Wx1,Wy1,Wx2,Wy2);
    END{wok};
  End;

{***********************************************************}

  Procedure PWindow.WClear;
  Begin
    GWClear;
    If WOK then begin
      SetLineStyle(SolidLn,0,NormWidth);
      SetColor(BdColor);
      Rectangle(Wx1,Wy1,Wx2,Wy2);
    end{WOK};
  End;

{******************* Methods for DWindow *******************}

  Constructor DWindow.WOpen(xul,yul,xbr,ybr:integer;
                                    BdC,BkC,TC,TEC:byte);
  Begin
    GWOpen(xul,yul,xbr,ybr,BdC,BkC,TC,TEC);
    If WOK then begin
      SetLineStyle(SolidLn,0,NormWidth);
      SetColor(BdColor);
      Rectangle(Wx1,Wy1,Wx2,Wy2);
      Rectangle(Wx1+dw,Wy1+dw,Wx2-dw,Wy2-dw);
    END{wok};
  End;

{***********************************************************}

  Procedure DWindow.WClear;
  Begin
    GWClear;
    If WOK then begin
      SetLineStyle(SolidLn,0,NormWidth);
      SetColor(BdColor);
      Rectangle(Wx1,Wy1,Wx2,Wy2);
      Rectangle(Wx1+dw,Wy1+dw,Wx2-dw,Wy2-dw);
    END{wok};
  End;

{*****************************************************************}

    Function GetColor(OldOp,MOut:char; x11,y11: integer) : integer;
    { Selects and returns color. x11,y11 = window coordinates.
      Returns -1 if no color selected (Esc). }

    Const
     MenuCol:  { Positions and options (to be filled in) }
     array[1..16] of PositionG =
                 (( x1:0; y1:0; x2:0; y2:0; opt:'0' ),
                  ( x1:0; y1:0; x2:0; y2:0; opt:'1' ),
                  ( x1:0; y1:0; x2:0; y2:0; opt:'2' ),
                  ( x1:0; y1:0; x2:0; y2:0; opt:'3' ),
                  ( x1:0; y1:0; x2:0; y2:0; opt:'4' ),
                  ( x1:0; y1:0; x2:0; y2:0; opt:'5' ),
                  ( x1:0; y1:0; x2:0; y2:0; opt:'6' ),
                  ( x1:0; y1:0; x2:0; y2:0; opt:'7' ),
                  ( x1:0; y1:0; x2:0; y2:0; opt:'8' ),
                  ( x1:0; y1:0; x2:0; y2:0; opt:'9' ),
                  ( x1:0; y1:0; x2:0; y2:0; opt:'A' ),
                  ( x1:0; y1:0; x2:0; y2:0; opt:'B' ),
                  ( x1:0; y1:0; x2:0; y2:0; opt:'C' ),
                  ( x1:0; y1:0; x2:0; y2:0; opt:'D' ),
                  ( x1:0; y1:0; x2:0; y2:0; opt:'E' ),
                  ( x1:0; y1:0; x2:0; y2:0; opt:'F' ));

     Colors: array[0..15] of String[12] =
                  ('Black','Blue','Green','Cyan',
                   'Red','Magenta','Brown','LightGray',
                   'DarkGray','LightBlue','LightGreen','LightCyan',
                   'LightRed','LightMagenta','Yellow','White');

    Var i : integer;
        c : char;

    Begin
       GetColor := -1;
       New(Wcs);
       With Wcs^ do begin
        WOpen(x11,y11,x11+128,y11+272,CuMeCo,Black,Black,CuEmCo);
        If WOK then begin
         For i:=0 to 15 do begin
           With MenuCol[i+1] do begin
             x1 := x11+3;   y1 := y11+8 + i*16;
             x2 := x11+125; y2 := y11+24 + i*16;
           end{with};
           If i=0 then SetColor(White) else SetColor(i);
           Wr(1,i+1,Colors[i]);
           If i<10 then WrtE(14,i+1,Chr(Ord('0')+i))
                   else WrtE(14,i+1,Chr(Ord('A')+i-10));
         end{for};
         WaitUp;
         MouseCursorOn;
         c := KeyOrMouseG(['0','1','2','3','4','5','6','7',
                           '8','9','A','B','C','D','E','F']
                              ,MenuCol,OldOp,MOut);
         MouseCursorOff;
         If c=MOut then GetColor := -1 else
          If c in ['0','1','2','3','4','5','6','7','8','9']
             then GetColor := Ord(c) - Ord('0')
             else GetColor := Ord(c) - Ord('A') + 10;
         WClose;
        end;
       end{with};
       Dispose(Wcs)
    End;

{***********************************************************}

End.
